/****************************************************************************
** Meta object code from reading C++ file 'place.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/place.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'place.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary14GeoCoordinatesE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::GeoCoordinates::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary14GeoCoordinatesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::GeoCoordinates",
        "className",
        "latitude",
        "longitude",
        "isValid"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'latitude'
        QtMocHelpers::PropertyData<double>(2, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'longitude'
        QtMocHelpers::PropertyData<double>(3, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isValid'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GeoCoordinates, qt_meta_tag_ZN10KItinerary14GeoCoordinatesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::GeoCoordinates::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary14GeoCoordinatesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary14GeoCoordinatesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary14GeoCoordinatesE_t>.metaTypes,
    nullptr
} };

void KItinerary::GeoCoordinates::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<GeoCoordinates *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->latitude(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->longitude(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setLatitude(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setLongitude(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary13PostalAddressE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::PostalAddress::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary13PostalAddressE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::PostalAddress",
        "className",
        "streetAddress",
        "addressLocality",
        "postalCode",
        "addressRegion",
        "addressCountry",
        "isEmpty"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'streetAddress'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'addressLocality'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'postalCode'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'addressRegion'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'addressCountry'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isEmpty'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PostalAddress, qt_meta_tag_ZN10KItinerary13PostalAddressE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::PostalAddress::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13PostalAddressE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13PostalAddressE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary13PostalAddressE_t>.metaTypes,
    nullptr
} };

void KItinerary::PostalAddress::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<PostalAddress *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->streetAddress(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->addressLocality(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->postalCode(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->addressRegion(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->addressCountry(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setStreetAddress(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setAddressLocality(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setPostalCode(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setAddressRegion(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setAddressCountry(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary5PlaceE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Place::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary5PlaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Place",
        "className",
        "name",
        "address",
        "KItinerary::PostalAddress",
        "geo",
        "KItinerary::GeoCoordinates",
        "telephone",
        "identifier",
        "geoUri",
        "QUrl"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'address'
        QtMocHelpers::PropertyData<KItinerary::PostalAddress>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'geo'
        QtMocHelpers::PropertyData<KItinerary::GeoCoordinates>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'telephone'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'identifier'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'geoUri'
        QtMocHelpers::PropertyData<QUrl>(9, 0x80000000 | 10, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Place, qt_meta_tag_ZN10KItinerary5PlaceE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Place::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5PlaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5PlaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary5PlaceE_t>.metaTypes,
    nullptr
} };

void KItinerary::Place::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Place *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::GeoCoordinates >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::PostalAddress >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<KItinerary::PostalAddress*>(_v) = _t->address(); break;
        case 3: *reinterpret_cast<KItinerary::GeoCoordinates*>(_v) = _t->geo(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->telephone(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->identifier(); break;
        case 6: *reinterpret_cast<QUrl*>(_v) = _t->geoUri(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setAddress(*reinterpret_cast<KItinerary::PostalAddress*>(_v)); break;
        case 3: _t->setGeo(*reinterpret_cast<KItinerary::GeoCoordinates*>(_v)); break;
        case 4: _t->setTelephone(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setIdentifier(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary7AirportE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Airport::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary7AirportE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Airport",
        "className",
        "iataCode"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'iataCode'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Airport, qt_meta_tag_ZN10KItinerary7AirportE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Airport::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Place>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7AirportE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7AirportE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary7AirportE_t>.metaTypes,
    nullptr
} };

void KItinerary::Airport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Airport *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->iataCode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setIataCode(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary12BoatTerminalE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::BoatTerminal::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary12BoatTerminalE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::BoatTerminal",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BoatTerminal, qt_meta_tag_ZN10KItinerary12BoatTerminalE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::BoatTerminal::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Place>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12BoatTerminalE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12BoatTerminalE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary12BoatTerminalE_t>.metaTypes,
    nullptr
} };

void KItinerary::BoatTerminal::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<BoatTerminal *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary12TrainStationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::TrainStation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary12TrainStationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::TrainStation",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TrainStation, qt_meta_tag_ZN10KItinerary12TrainStationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::TrainStation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Place>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12TrainStationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12TrainStationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary12TrainStationE_t>.metaTypes,
    nullptr
} };

void KItinerary::TrainStation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TrainStation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary10BusStationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::BusStation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary10BusStationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::BusStation",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BusStation, qt_meta_tag_ZN10KItinerary10BusStationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::BusStation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Place>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10BusStationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10BusStationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary10BusStationE_t>.metaTypes,
    nullptr
} };

void KItinerary::BusStation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<BusStation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary17TouristAttractionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::TouristAttraction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary17TouristAttractionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::TouristAttraction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TouristAttraction, qt_meta_tag_ZN10KItinerary17TouristAttractionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::TouristAttraction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Place>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17TouristAttractionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17TouristAttractionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary17TouristAttractionE_t>.metaTypes,
    nullptr
} };

void KItinerary::TouristAttraction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TouristAttraction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
