/****************************************************************************
** Meta object code from reading C++ file 'bytearray.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/jsapi/bytearray.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bytearray.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary5JsApi9ByteArrayE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::JsApi::ByteArray::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary5JsApi9ByteArrayE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::JsApi::ByteArray",
        "inflate",
        "QJSValue",
        "",
        "input",
        "toBase64",
        "fromBase64",
        "b64",
        "decodeUtf8",
        "decodeLatin1",
        "toBitArray",
        "QVariant",
        "toProtobufStreamReader",
        "decodeRsp6Ticket",
        "text",
        "toArrayBuffer"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'inflate'
        QtMocHelpers::MethodData<QJSValue(const QByteArray &) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Method 'toBase64'
        QtMocHelpers::MethodData<QString(const QByteArray &) const>(5, 3, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Method 'fromBase64'
        QtMocHelpers::MethodData<QJSValue(const QString &) const>(6, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'decodeUtf8'
        QtMocHelpers::MethodData<QString(const QByteArray &) const>(8, 3, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Method 'decodeLatin1'
        QtMocHelpers::MethodData<QString(const QByteArray &) const>(9, 3, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Method 'toBitArray'
        QtMocHelpers::MethodData<QVariant(const QByteArray &) const>(10, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Method 'toProtobufStreamReader'
        QtMocHelpers::MethodData<QVariant(const QByteArray &) const>(12, 3, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Method 'decodeRsp6Ticket'
        QtMocHelpers::MethodData<QJSValue(const QString &) const>(13, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 14 },
        }}),
        // Method 'toArrayBuffer'
        QtMocHelpers::MethodData<QJSValue(const QByteArray &) const>(15, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QByteArray, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ByteArray, qt_meta_tag_ZN10KItinerary5JsApi9ByteArrayE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::JsApi::ByteArray::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5JsApi9ByteArrayE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5JsApi9ByteArrayE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary5JsApi9ByteArrayE_t>.metaTypes,
    nullptr
} };

void KItinerary::JsApi::ByteArray::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ByteArray *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QJSValue _r = _t->inflate((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->toBase64((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QJSValue _r = _t->fromBase64((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->decodeUtf8((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->decodeLatin1((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QVariant _r = _t->toBitArray((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 6: { QVariant _r = _t->toProtobufStreamReader((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 7: { QJSValue _r = _t->decodeRsp6Ticket((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 8: { QJSValue _r = _t->toArrayBuffer((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *KItinerary::JsApi::ByteArray::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KItinerary::JsApi::ByteArray::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5JsApi9ByteArrayE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KItinerary::JsApi::ByteArray::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
