/****************************************************************************
** Meta object code from reading C++ file 'jsonld.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/jsapi/jsonld.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jsonld.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary5JsApi6JsonLdE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::JsApi::JsonLd::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary5JsApi6JsonLdE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::JsApi::JsonLd",
        "newObject",
        "QJSValue",
        "",
        "typeName",
        "newFlightReservation",
        "newTrainReservation",
        "newBusReservation",
        "newLodgingReservation",
        "newEventReservation",
        "newFoodEstablishmentReservation",
        "newRentalCarReservation",
        "newBoatReservation",
        "trainToBusReservation",
        "trainRes",
        "busToTrainReservation",
        "busRes",
        "toDateTime",
        "dtStr",
        "format",
        "localeName",
        "toJson",
        "QVariant",
        "v",
        "clone",
        "toGeoCoordinates",
        "mapUrl",
        "readQDateTime",
        "obj",
        "propName",
        "apply",
        "lhs",
        "rhs"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'newObject'
        QtMocHelpers::MethodData<QJSValue(const QString &) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 4 },
        }}),
        // Method 'newFlightReservation'
        QtMocHelpers::MethodData<QJSValue() const>(5, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'newTrainReservation'
        QtMocHelpers::MethodData<QJSValue() const>(6, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'newBusReservation'
        QtMocHelpers::MethodData<QJSValue() const>(7, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'newLodgingReservation'
        QtMocHelpers::MethodData<QJSValue() const>(8, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'newEventReservation'
        QtMocHelpers::MethodData<QJSValue() const>(9, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'newFoodEstablishmentReservation'
        QtMocHelpers::MethodData<QJSValue() const>(10, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'newRentalCarReservation'
        QtMocHelpers::MethodData<QJSValue() const>(11, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'newBoatReservation'
        QtMocHelpers::MethodData<QJSValue() const>(12, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Method 'trainToBusReservation'
        QtMocHelpers::MethodData<QJSValue(const QJSValue &) const>(13, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 2, 14 },
        }}),
        // Method 'busToTrainReservation'
        QtMocHelpers::MethodData<QJSValue(const QJSValue &) const>(15, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 2, 16 },
        }}),
        // Method 'toDateTime'
        QtMocHelpers::MethodData<QDateTime(const QString &, const QJSValue &, const QJSValue &) const>(17, 3, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QString, 18 }, { 0x80000000 | 2, 19 }, { 0x80000000 | 2, 20 },
        }}),
        // Method 'toJson'
        QtMocHelpers::MethodData<QJSValue(const QVariant &) const>(21, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Method 'clone'
        QtMocHelpers::MethodData<QJSValue(const QJSValue &) const>(24, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 2, 23 },
        }}),
        // Method 'toGeoCoordinates'
        QtMocHelpers::MethodData<QJSValue(const QString &)>(25, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 26 },
        }}),
        // Method 'readQDateTime'
        QtMocHelpers::MethodData<QJSValue(const QVariant &, const QString &) const>(27, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 22, 28 }, { QMetaType::QString, 29 },
        }}),
        // Method 'apply'
        QtMocHelpers::MethodData<QJSValue(const QJSValue &, const QJSValue &) const>(30, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 2, 31 }, { 0x80000000 | 2, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JsonLd, qt_meta_tag_ZN10KItinerary5JsApi6JsonLdE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::JsApi::JsonLd::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5JsApi6JsonLdE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5JsApi6JsonLdE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary5JsApi6JsonLdE_t>.metaTypes,
    nullptr
} };

void KItinerary::JsApi::JsonLd::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JsonLd *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QJSValue _r = _t->newObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QJSValue _r = _t->newFlightReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 2: { QJSValue _r = _t->newTrainReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 3: { QJSValue _r = _t->newBusReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 4: { QJSValue _r = _t->newLodgingReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 5: { QJSValue _r = _t->newEventReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 6: { QJSValue _r = _t->newFoodEstablishmentReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 7: { QJSValue _r = _t->newRentalCarReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 8: { QJSValue _r = _t->newBoatReservation();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 9: { QJSValue _r = _t->trainToBusReservation((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 10: { QJSValue _r = _t->busToTrainReservation((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDateTime _r = _t->toDateTime((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 12: { QJSValue _r = _t->toJson((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 13: { QJSValue _r = _t->clone((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 14: { QJSValue _r = _t->toGeoCoordinates((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 15: { QJSValue _r = _t->readQDateTime((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 16: { QJSValue _r = _t->apply((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *KItinerary::JsApi::JsonLd::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KItinerary::JsApi::JsonLd::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary5JsApi6JsonLdE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KItinerary::JsApi::JsonLd::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
