/****************************************************************************
** Meta object code from reading C++ file 'vendor0080block.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/uic9183/vendor0080block.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vendor0080block.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary22Vendor0080BLOrderBlockE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Vendor0080BLOrderBlock::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary22Vendor0080BLOrderBlockE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Vendor0080BLOrderBlock",
        "validFrom",
        "validTo",
        "serialNumber"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'validFrom'
        QtMocHelpers::PropertyData<QDate>(1, QMetaType::QDate, QMC::DefaultPropertyFlags),
        // property 'validTo'
        QtMocHelpers::PropertyData<QDate>(2, QMetaType::QDate, QMC::DefaultPropertyFlags),
        // property 'serialNumber'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Vendor0080BLOrderBlock, qt_meta_tag_ZN10KItinerary22Vendor0080BLOrderBlockE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Vendor0080BLOrderBlock::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary22Vendor0080BLOrderBlockE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary22Vendor0080BLOrderBlockE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary22Vendor0080BLOrderBlockE_t>.metaTypes,
    nullptr
} };

void KItinerary::Vendor0080BLOrderBlock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Vendor0080BLOrderBlock *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDate*>(_v) = _t->validFrom(); break;
        case 1: *reinterpret_cast<QDate*>(_v) = _t->validTo(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->serialNumber(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary20Vendor0080BLSubBlockE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Vendor0080BLSubBlock::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary20Vendor0080BLSubBlockE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Vendor0080BLSubBlock",
        "content"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'content'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Vendor0080BLSubBlock, qt_meta_tag_ZN10KItinerary20Vendor0080BLSubBlockE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Vendor0080BLSubBlock::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary20Vendor0080BLSubBlockE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary20Vendor0080BLSubBlockE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary20Vendor0080BLSubBlockE_t>.metaTypes,
    nullptr
} };

void KItinerary::Vendor0080BLSubBlock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Vendor0080BLSubBlock *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->toString(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary17Vendor0080BLBlockE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Vendor0080BLBlock::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary17Vendor0080BLBlockE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Vendor0080BLBlock",
        "orderBlock",
        "KItinerary::Vendor0080BLOrderBlock",
        "",
        "i",
        "findSubBlock",
        "QVariant",
        "str",
        "orderBlockCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'orderBlock'
        QtMocHelpers::MethodData<KItinerary::Vendor0080BLOrderBlock(int) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::Int, 4 },
        }}),
        // Method 'findSubBlock'
        QtMocHelpers::MethodData<QVariant(const QString &) const>(5, 3, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'orderBlockCount'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Vendor0080BLBlock, qt_meta_tag_ZN10KItinerary17Vendor0080BLBlockE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Vendor0080BLBlock::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17Vendor0080BLBlockE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17Vendor0080BLBlockE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary17Vendor0080BLBlockE_t>.metaTypes,
    nullptr
} };

void KItinerary::Vendor0080BLBlock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Vendor0080BLBlock *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { KItinerary::Vendor0080BLOrderBlock _r = _t->orderBlock((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KItinerary::Vendor0080BLOrderBlock*>(_a[0]) = std::move(_r); }  break;
        case 1: { QVariant _r = _t->findSubBlock((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->orderBlockCount(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
