/****************************************************************************
** Meta object code from reading C++ file 'ssbv3ticket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/era/ssbv3ticket.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ssbv3ticket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary11SSBv3TicketE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::SSBv3Ticket::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary11SSBv3TicketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::SSBv3Ticket",
        "issueDate",
        "",
        "contextDate",
        "type1DepartureDay",
        "type2ValidFrom",
        "type2ValidUntil",
        "type3ValidFrom",
        "type3ValidUntil",
        "version",
        "issuerCode",
        "id",
        "ticketTypeCode",
        "numberOfAdultPassengers",
        "numberOfChildPassengers",
        "specimen",
        "classOfTravel",
        "tcn",
        "yearOfIssue",
        "issuingDay",
        "type1SubTicketType",
        "type1StationCodeNumericOrAlpha",
        "type1StationCodeListType",
        "type1DepartureStationNum",
        "type1DepartureStationAlpha",
        "type1ArrivalStationNum",
        "type1ArrivalStationAlpha",
        "type1DepartureDate",
        "type1DepartureTime",
        "type1TrainNumber",
        "type1CoachNumber",
        "type1SeatNumber",
        "type1OverbookingIndicator",
        "type1InformationMessages",
        "type1OpenText",
        "type2ReturnJourneyFlag",
        "type2FirstDayOfValidity",
        "type2LastDayOfValidity",
        "type2StationCodeNumericOrAlpha",
        "type2StationCodeListType",
        "type2DepartureStationNum",
        "type2DepartureStationAlpha",
        "type2ArrivalStationNum",
        "type2ArrivalStationAlpha",
        "type2InformationMessages",
        "type2OpenText",
        "type3ReturnJourneyFlag",
        "type3FirstDayOfValidity",
        "type3LastDayOfValidity",
        "type3StationCodeNumericOrAlpha",
        "type3StationCodeListType",
        "type3DepartureStationNum",
        "type3DepartureStationAlpha",
        "type3ArrivalStationNum",
        "type3ArrivalStationAlpha",
        "type3NameOfGroupLeader",
        "type3CountermarkNumber",
        "type3InformationMessages",
        "type3OpenText",
        "type4RPTSubTicketType",
        "type4FirstDayOfValidity",
        "type4LastDayOfValidity",
        "type4NumberOfDaysOfTravelAllowed",
        "type4CountryCode1",
        "type4CountryCode2",
        "type4CountryCode3",
        "type4CountryCode4",
        "type4SecondPage",
        "type4InformationMessages",
        "type4OpenText",
        "TicketType",
        "IRT_RES_BOA",
        "NRT",
        "GRT",
        "RPT"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'issueDate'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'issueDate'
        QtMocHelpers::MethodData<QDate() const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
        // Method 'type1DepartureDay'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(4, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'type1DepartureDay'
        QtMocHelpers::MethodData<QDate() const>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
        // Method 'type2ValidFrom'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(5, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'type2ValidFrom'
        QtMocHelpers::MethodData<QDate() const>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
        // Method 'type2ValidUntil'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(6, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'type2ValidUntil'
        QtMocHelpers::MethodData<QDate() const>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
        // Method 'type3ValidFrom'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(7, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'type3ValidFrom'
        QtMocHelpers::MethodData<QDate() const>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
        // Method 'type3ValidUntil'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(8, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'type3ValidUntil'
        QtMocHelpers::MethodData<QDate() const>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'version'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'issuerCode'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'id'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'ticketTypeCode'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'numberOfAdultPassengers'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'numberOfChildPassengers'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'specimen'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'classOfTravel'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'tcn'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'yearOfIssue'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'issuingDay'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1SubTicketType'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1StationCodeNumericOrAlpha'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1StationCodeListType'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1DepartureStationNum'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1DepartureStationAlpha'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type1ArrivalStationNum'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1ArrivalStationAlpha'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type1DepartureDate'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1DepartureTime'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1TrainNumber'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type1CoachNumber'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1SeatNumber'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type1OverbookingIndicator'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1InformationMessages'
        QtMocHelpers::PropertyData<int>(33, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type1OpenText'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type2ReturnJourneyFlag'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2FirstDayOfValidity'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2LastDayOfValidity'
        QtMocHelpers::PropertyData<int>(37, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2StationCodeNumericOrAlpha'
        QtMocHelpers::PropertyData<int>(38, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2StationCodeListType'
        QtMocHelpers::PropertyData<int>(39, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2DepartureStationNum'
        QtMocHelpers::PropertyData<int>(40, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2DepartureStationAlpha'
        QtMocHelpers::PropertyData<QString>(41, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type2ArrivalStationNum'
        QtMocHelpers::PropertyData<int>(42, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2ArrivalStationAlpha'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type2InformationMessages'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type2OpenText'
        QtMocHelpers::PropertyData<QString>(45, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type3ReturnJourneyFlag'
        QtMocHelpers::PropertyData<int>(46, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3FirstDayOfValidity'
        QtMocHelpers::PropertyData<int>(47, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3LastDayOfValidity'
        QtMocHelpers::PropertyData<int>(48, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3StationCodeNumericOrAlpha'
        QtMocHelpers::PropertyData<int>(49, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3StationCodeListType'
        QtMocHelpers::PropertyData<int>(50, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3DepartureStationNum'
        QtMocHelpers::PropertyData<int>(51, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3DepartureStationAlpha'
        QtMocHelpers::PropertyData<QString>(52, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type3ArrivalStationNum'
        QtMocHelpers::PropertyData<int>(53, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3ArrivalStationAlpha'
        QtMocHelpers::PropertyData<QString>(54, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type3NameOfGroupLeader'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type3CountermarkNumber'
        QtMocHelpers::PropertyData<int>(56, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3InformationMessages'
        QtMocHelpers::PropertyData<int>(57, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type3OpenText'
        QtMocHelpers::PropertyData<QString>(58, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type4RPTSubTicketType'
        QtMocHelpers::PropertyData<int>(59, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4FirstDayOfValidity'
        QtMocHelpers::PropertyData<int>(60, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4LastDayOfValidity'
        QtMocHelpers::PropertyData<int>(61, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4NumberOfDaysOfTravelAllowed'
        QtMocHelpers::PropertyData<int>(62, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4CountryCode1'
        QtMocHelpers::PropertyData<int>(63, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4CountryCode2'
        QtMocHelpers::PropertyData<int>(64, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4CountryCode3'
        QtMocHelpers::PropertyData<int>(65, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4CountryCode4'
        QtMocHelpers::PropertyData<int>(66, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4SecondPage'
        QtMocHelpers::PropertyData<int>(67, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4InformationMessages'
        QtMocHelpers::PropertyData<int>(68, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'type4OpenText'
        QtMocHelpers::PropertyData<QString>(69, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TicketType'
        QtMocHelpers::EnumData<enum TicketType>(70, 70, QMC::EnumFlags{}).add({
            {   71, TicketType::IRT_RES_BOA },
            {   72, TicketType::NRT },
            {   73, TicketType::GRT },
            {   74, TicketType::RPT },
        }),
    };
    return QtMocHelpers::metaObjectData<SSBv3Ticket, qt_meta_tag_ZN10KItinerary11SSBv3TicketE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::SSBv3Ticket::staticMetaObject = { {
    QtPrivate::MetaObjectForType<SSBTicketBase>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary11SSBv3TicketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary11SSBv3TicketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary11SSBv3TicketE_t>.metaTypes,
    nullptr
} };

void KItinerary::SSBv3Ticket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<SSBv3Ticket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDate _r = _t->issueDate((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDate _r = _t->issueDate();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDate _r = _t->type1DepartureDay((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDate _r = _t->type1DepartureDay();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDate _r = _t->type2ValidFrom((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDate _r = _t->type2ValidFrom();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDate _r = _t->type2ValidUntil((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDate _r = _t->type2ValidUntil();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDate _r = _t->type3ValidFrom((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDate _r = _t->type3ValidFrom();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDate _r = _t->type3ValidUntil((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDate _r = _t->type3ValidUntil();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->version(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->issuerCode(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->id(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->ticketTypeCode(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->numberOfAdultPassengers(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->numberOfChildPassengers(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->specimen(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->classOfTravel(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->tcn(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->yearOfIssue(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->issuingDay(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->type1SubTicketType(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->type1StationCodeNumericOrAlpha(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->type1StationCodeListType(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->type1DepartureStationNum(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->type1DepartureStationAlpha(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->type1ArrivalStationNum(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->type1ArrivalStationAlpha(); break;
        case 18: *reinterpret_cast<int*>(_v) = _t->type1DepartureDate(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->type1DepartureTime(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->type1TrainNumber(); break;
        case 21: *reinterpret_cast<int*>(_v) = _t->type1CoachNumber(); break;
        case 22: *reinterpret_cast<QString*>(_v) = _t->type1SeatNumber(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->type1OverbookingIndicator(); break;
        case 24: *reinterpret_cast<int*>(_v) = _t->type1InformationMessages(); break;
        case 25: *reinterpret_cast<QString*>(_v) = _t->type1OpenText(); break;
        case 26: *reinterpret_cast<int*>(_v) = _t->type2ReturnJourneyFlag(); break;
        case 27: *reinterpret_cast<int*>(_v) = _t->type2FirstDayOfValidity(); break;
        case 28: *reinterpret_cast<int*>(_v) = _t->type2LastDayOfValidity(); break;
        case 29: *reinterpret_cast<int*>(_v) = _t->type2StationCodeNumericOrAlpha(); break;
        case 30: *reinterpret_cast<int*>(_v) = _t->type2StationCodeListType(); break;
        case 31: *reinterpret_cast<int*>(_v) = _t->type2DepartureStationNum(); break;
        case 32: *reinterpret_cast<QString*>(_v) = _t->type2DepartureStationAlpha(); break;
        case 33: *reinterpret_cast<int*>(_v) = _t->type2ArrivalStationNum(); break;
        case 34: *reinterpret_cast<QString*>(_v) = _t->type2ArrivalStationAlpha(); break;
        case 35: *reinterpret_cast<int*>(_v) = _t->type2InformationMessages(); break;
        case 36: *reinterpret_cast<QString*>(_v) = _t->type2OpenText(); break;
        case 37: *reinterpret_cast<int*>(_v) = _t->type3ReturnJourneyFlag(); break;
        case 38: *reinterpret_cast<int*>(_v) = _t->type3FirstDayOfValidity(); break;
        case 39: *reinterpret_cast<int*>(_v) = _t->type3LastDayOfValidity(); break;
        case 40: *reinterpret_cast<int*>(_v) = _t->type3StationCodeNumericOrAlpha(); break;
        case 41: *reinterpret_cast<int*>(_v) = _t->type3StationCodeListType(); break;
        case 42: *reinterpret_cast<int*>(_v) = _t->type3DepartureStationNum(); break;
        case 43: *reinterpret_cast<QString*>(_v) = _t->type3DepartureStationAlpha(); break;
        case 44: *reinterpret_cast<int*>(_v) = _t->type3ArrivalStationNum(); break;
        case 45: *reinterpret_cast<QString*>(_v) = _t->type3ArrivalStationAlpha(); break;
        case 46: *reinterpret_cast<QString*>(_v) = _t->type3NameOfGroupLeader(); break;
        case 47: *reinterpret_cast<int*>(_v) = _t->type3CountermarkNumber(); break;
        case 48: *reinterpret_cast<int*>(_v) = _t->type3InformationMessages(); break;
        case 49: *reinterpret_cast<QString*>(_v) = _t->type3OpenText(); break;
        case 50: *reinterpret_cast<int*>(_v) = _t->type4RPTSubTicketType(); break;
        case 51: *reinterpret_cast<int*>(_v) = _t->type4FirstDayOfValidity(); break;
        case 52: *reinterpret_cast<int*>(_v) = _t->type4LastDayOfValidity(); break;
        case 53: *reinterpret_cast<int*>(_v) = _t->type4NumberOfDaysOfTravelAllowed(); break;
        case 54: *reinterpret_cast<int*>(_v) = _t->type4CountryCode1(); break;
        case 55: *reinterpret_cast<int*>(_v) = _t->type4CountryCode2(); break;
        case 56: *reinterpret_cast<int*>(_v) = _t->type4CountryCode3(); break;
        case 57: *reinterpret_cast<int*>(_v) = _t->type4CountryCode4(); break;
        case 58: *reinterpret_cast<int*>(_v) = _t->type4SecondPage(); break;
        case 59: *reinterpret_cast<int*>(_v) = _t->type4InformationMessages(); break;
        case 60: *reinterpret_cast<QString*>(_v) = _t->type4OpenText(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
