/****************************************************************************
** Meta object code from reading C++ file 'uic9183parser.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/uic9183/uic9183parser.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'uic9183parser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary13Uic9183ParserE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Uic9183Parser::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary13Uic9183ParserE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Uic9183Parser",
        "block",
        "QVariant",
        "",
        "name",
        "pnr",
        "carrierId",
        "issuer",
        "KItinerary::Organization",
        "validFrom",
        "validUntil",
        "person",
        "KItinerary::Person",
        "outboundDepartureStation",
        "KItinerary::TrainStation",
        "outboundArrivalStation",
        "returnDepartureStation",
        "returnArrivalStation",
        "seatingType",
        "ticketLayout",
        "rct2Ticket",
        "rawData"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'block'
        QtMocHelpers::MethodData<QVariant(const QString &) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'pnr'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'carrierId'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'issuer'
        QtMocHelpers::PropertyData<KItinerary::Organization>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'validFrom'
        QtMocHelpers::PropertyData<QDateTime>(9, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'validUntil'
        QtMocHelpers::PropertyData<QDateTime>(10, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'person'
        QtMocHelpers::PropertyData<KItinerary::Person>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'outboundDepartureStation'
        QtMocHelpers::PropertyData<KItinerary::TrainStation>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'outboundArrivalStation'
        QtMocHelpers::PropertyData<KItinerary::TrainStation>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'returnDepartureStation'
        QtMocHelpers::PropertyData<KItinerary::TrainStation>(16, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'returnArrivalStation'
        QtMocHelpers::PropertyData<KItinerary::TrainStation>(17, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'seatingType'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'ticketLayout'
        QtMocHelpers::PropertyData<QVariant>(19, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'rct2Ticket'
        QtMocHelpers::PropertyData<QVariant>(20, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'rawData'
        QtMocHelpers::PropertyData<QByteArray>(21, QMetaType::QByteArray, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Uic9183Parser, qt_meta_tag_ZN10KItinerary13Uic9183ParserE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Uic9183Parser::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13Uic9183ParserE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13Uic9183ParserE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary13Uic9183ParserE_t>.metaTypes,
    nullptr
} };

void KItinerary::Uic9183Parser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Uic9183Parser *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QVariant _r = _t->block((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Organization >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Person >(); break;
        case 10:
        case 9:
        case 8:
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::TrainStation >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->pnr(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->carrierId(); break;
        case 3: *reinterpret_cast<KItinerary::Organization*>(_v) = _t->issuer(); break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->validFrom(); break;
        case 5: *reinterpret_cast<QDateTime*>(_v) = _t->validUntil(); break;
        case 6: *reinterpret_cast<KItinerary::Person*>(_v) = _t->person(); break;
        case 7: *reinterpret_cast<KItinerary::TrainStation*>(_v) = _t->outboundDepartureStation(); break;
        case 8: *reinterpret_cast<KItinerary::TrainStation*>(_v) = _t->outboundArrivalStation(); break;
        case 9: *reinterpret_cast<KItinerary::TrainStation*>(_v) = _t->returnDepartureStation(); break;
        case 10: *reinterpret_cast<KItinerary::TrainStation*>(_v) = _t->returnArrivalStation(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->seatingType(); break;
        case 12: *reinterpret_cast<QVariant*>(_v) = _t->ticketLayoutVariant(); break;
        case 13: *reinterpret_cast<QVariant*>(_v) = _t->rct2TicketVariant(); break;
        case 14: *reinterpret_cast<QByteArray*>(_v) = _t->rawData(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
