/****************************************************************************
** Meta object code from reading C++ file 'iatabcbpsections.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/iata/iatabcbpsections.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'iatabcbpsections.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary30IataBcbpUniqueMandatorySectionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::IataBcbpUniqueMandatorySection::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary30IataBcbpUniqueMandatorySectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::IataBcbpUniqueMandatorySection",
        "formatCode",
        "numberOfLegs",
        "passengerName",
        "electronicTicketIndicator"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'formatCode'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'numberOfLegs'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'passengerName'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'electronicTicketIndicator'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IataBcbpUniqueMandatorySection, qt_meta_tag_ZN10KItinerary30IataBcbpUniqueMandatorySectionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::IataBcbpUniqueMandatorySection::staticMetaObject = { {
    QtPrivate::MetaObjectForType<IataBcbpSectionBase>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary30IataBcbpUniqueMandatorySectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary30IataBcbpUniqueMandatorySectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary30IataBcbpUniqueMandatorySectionE_t>.metaTypes,
    nullptr
} };

void KItinerary::IataBcbpUniqueMandatorySection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IataBcbpUniqueMandatorySection *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->formatCode(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->numberOfLegs(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->passengerName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->electronicTicketIndicator(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary32IataBcbpUniqueConditionalSectionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::IataBcbpUniqueConditionalSection::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary32IataBcbpUniqueConditionalSectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::IataBcbpUniqueConditionalSection",
        "dateOfIssue",
        "",
        "contextDate",
        "version",
        "fieldSize",
        "passengerDescription",
        "sourceOfCheckin",
        "sourceOfBoardingPassIssuance",
        "yearOfIssue",
        "dayOfIssue",
        "documentType",
        "airlineDesignatorOfBoardingPassIssuer",
        "baggageTagLicensePlateNumber1",
        "baggageTagLicensePlateNumber2",
        "baggageTagLicensePlateNumber3"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'dateOfIssue'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'dateOfIssue'
        QtMocHelpers::MethodData<QDate() const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'version'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'fieldSize'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'passengerDescription'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'sourceOfCheckin'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'sourceOfBoardingPassIssuance'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'yearOfIssue'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'dayOfIssue'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'documentType'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'airlineDesignatorOfBoardingPassIssuer'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'baggageTagLicensePlateNumber1'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'baggageTagLicensePlateNumber2'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'baggageTagLicensePlateNumber3'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IataBcbpUniqueConditionalSection, qt_meta_tag_ZN10KItinerary32IataBcbpUniqueConditionalSectionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::IataBcbpUniqueConditionalSection::staticMetaObject = { {
    QtPrivate::MetaObjectForType<IataBcbpSectionBase>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary32IataBcbpUniqueConditionalSectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary32IataBcbpUniqueConditionalSectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary32IataBcbpUniqueConditionalSectionE_t>.metaTypes,
    nullptr
} };

void KItinerary::IataBcbpUniqueConditionalSection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IataBcbpUniqueConditionalSection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDate _r = _t->dateOfIssue((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDate _r = _t->dateOfIssue();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->version(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->fieldSize(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->passengerDescription(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->sourceOfCheckin(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->sourceOfBoardingPassIssuance(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->yearOfIssue(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->dayOfIssue(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->documentType(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->airlineDesignatorOfBoardingPassIssuer(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->baggageTagLicensePlateNumber1(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->baggageTagLicensePlateNumber2(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->baggageTagLicensePlateNumber3(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary32IataBcbpRepeatedMandatorySectionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::IataBcbpRepeatedMandatorySection::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary32IataBcbpRepeatedMandatorySectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::IataBcbpRepeatedMandatorySection",
        "dateOfFlight",
        "",
        "contextDate",
        "operatingCarrierPNRCode",
        "fromCityAirportCode",
        "toCityAirportCode",
        "operatingCarrierDesignator",
        "flightNumber",
        "dayOfFlight",
        "compartmentCode",
        "seatNumber",
        "checkinSequenceNumber",
        "passengerStatus",
        "variableFieldSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'dateOfFlight'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'dateOfFlight'
        QtMocHelpers::MethodData<QDate() const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'operatingCarrierPNRCode'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'fromCityAirportCode'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'toCityAirportCode'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'operatingCarrierDesignator'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'flightNumber'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'dayOfFlight'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'compartmentCode'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'seatNumber'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'checkinSequenceNumber'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'passengerStatus'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'variableFieldSize'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IataBcbpRepeatedMandatorySection, qt_meta_tag_ZN10KItinerary32IataBcbpRepeatedMandatorySectionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::IataBcbpRepeatedMandatorySection::staticMetaObject = { {
    QtPrivate::MetaObjectForType<IataBcbpSectionBase>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary32IataBcbpRepeatedMandatorySectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary32IataBcbpRepeatedMandatorySectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary32IataBcbpRepeatedMandatorySectionE_t>.metaTypes,
    nullptr
} };

void KItinerary::IataBcbpRepeatedMandatorySection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IataBcbpRepeatedMandatorySection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDate _r = _t->dateOfFlight((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDate _r = _t->dateOfFlight();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->operatingCarrierPNRCode(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->fromCityAirportCode(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->toCityAirportCode(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->operatingCarrierDesignator(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->flightNumber(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->dayOfFlight(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->compartmentCode(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->seatNumber(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->checkinSequenceNumber(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->passengerStatus(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->variableFieldSize(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary34IataBcbpRepeatedConditionalSectionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::IataBcbpRepeatedConditionalSection::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary34IataBcbpRepeatedConditionalSectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::IataBcbpRepeatedConditionalSection",
        "conditionalFieldSize",
        "airlineNumericCode",
        "documentNumber",
        "selecteeIndicator",
        "internationalDocumentVerification",
        "marketingCarrierDesignator",
        "frequentFlyerAirlineDesignator",
        "frequenFlyerNumber",
        "idAdIndicator",
        "freeBaggageAllowance",
        "fastTrack"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'conditionalFieldSize'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'airlineNumericCode'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'documentNumber'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'selecteeIndicator'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'internationalDocumentVerification'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'marketingCarrierDesignator'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'frequentFlyerAirlineDesignator'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'frequenFlyerNumber'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'idAdIndicator'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'freeBaggageAllowance'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'fastTrack'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IataBcbpRepeatedConditionalSection, qt_meta_tag_ZN10KItinerary34IataBcbpRepeatedConditionalSectionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::IataBcbpRepeatedConditionalSection::staticMetaObject = { {
    QtPrivate::MetaObjectForType<IataBcbpSectionBase>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary34IataBcbpRepeatedConditionalSectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary34IataBcbpRepeatedConditionalSectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary34IataBcbpRepeatedConditionalSectionE_t>.metaTypes,
    nullptr
} };

void KItinerary::IataBcbpRepeatedConditionalSection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IataBcbpRepeatedConditionalSection *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->conditionalFieldSize(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->airlineNumericCode(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->documentNumber(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->selecteeIndicator(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->internationalDocumentVerification(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->marketingCarrierDesignator(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->frequentFlyerAirlineDesignator(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->frequenFlyerNumber(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->idAdIndicator(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->freeBaggageAllowance(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->fastTrack(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary23IataBcbpSecuritySectionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::IataBcbpSecuritySection::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary23IataBcbpSecuritySectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::IataBcbpSecuritySection",
        "type",
        "size",
        "securityData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'size'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'securityData'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IataBcbpSecuritySection, qt_meta_tag_ZN10KItinerary23IataBcbpSecuritySectionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::IataBcbpSecuritySection::staticMetaObject = { {
    QtPrivate::MetaObjectForType<IataBcbpSectionBase>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary23IataBcbpSecuritySectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary23IataBcbpSecuritySectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary23IataBcbpSecuritySectionE_t>.metaTypes,
    nullptr
} };

void KItinerary::IataBcbpSecuritySection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IataBcbpSecuritySection *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->size(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->securityData(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
