/****************************************************************************
** Meta object code from reading C++ file 'uic9183ticketlayout.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/uic9183/uic9183ticketlayout.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'uic9183ticketlayout.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary24Uic9183TicketLayoutFieldE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Uic9183TicketLayoutField::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary24Uic9183TicketLayoutFieldE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Uic9183TicketLayoutField",
        "row",
        "column",
        "height",
        "width",
        "format",
        "size",
        "text",
        "isNull",
        "next"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'row'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'column'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'height'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'width'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'format'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'size'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'text'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'isNull'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::Readable | QMC::Designable | QMC::Scriptable),
        // property 'next'
        QtMocHelpers::PropertyData<KItinerary::Uic9183TicketLayoutField>(9, 0x80000000 | 0, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Uic9183TicketLayoutField, qt_meta_tag_ZN10KItinerary24Uic9183TicketLayoutFieldE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Uic9183TicketLayoutField::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary24Uic9183TicketLayoutFieldE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary24Uic9183TicketLayoutFieldE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary24Uic9183TicketLayoutFieldE_t>.metaTypes,
    nullptr
} };

void KItinerary::Uic9183TicketLayoutField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Uic9183TicketLayoutField *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->row(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->column(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->height(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->width(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->format(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->size(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->text(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isNull(); break;
        case 8: *reinterpret_cast<KItinerary::Uic9183TicketLayoutField*>(_v) = _t->next(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary19Uic9183TicketLayoutE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Uic9183TicketLayout::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary19Uic9183TicketLayoutE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Uic9183TicketLayout",
        "text",
        "",
        "row",
        "column",
        "width",
        "height",
        "type",
        "size",
        "QSize",
        "firstField",
        "KItinerary::Uic9183TicketLayoutField"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'text'
        QtMocHelpers::MethodData<QString(int, int, int, int) const>(1, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'size'
        QtMocHelpers::PropertyData<QSize>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'firstField'
        QtMocHelpers::PropertyData<KItinerary::Uic9183TicketLayoutField>(10, 0x80000000 | 11, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Uic9183TicketLayout, qt_meta_tag_ZN10KItinerary19Uic9183TicketLayoutE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Uic9183TicketLayout::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary19Uic9183TicketLayoutE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary19Uic9183TicketLayoutE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary19Uic9183TicketLayoutE_t>.metaTypes,
    nullptr
} };

void KItinerary::Uic9183TicketLayout::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Uic9183TicketLayout *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->text((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<QSize*>(_v) = _t->size(); break;
        case 2: *reinterpret_cast<KItinerary::Uic9183TicketLayoutField*>(_v) = _t->firstField(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
