/****************************************************************************
** Meta object code from reading C++ file 'programmembership.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/programmembership.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'programmembership.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary17ProgramMembershipE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::ProgramMembership::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary17ProgramMembershipE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::ProgramMembership",
        "className",
        "programName",
        "membershipNumber",
        "member",
        "KItinerary::Person",
        "subjectOf",
        "QVariantList",
        "token",
        "tokenType",
        "KItinerary::Token::TokenType",
        "tokenData",
        "QVariant",
        "validFrom",
        "validUntil"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'programName'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'membershipNumber'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'member'
        QtMocHelpers::PropertyData<KItinerary::Person>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'subjectOf'
        QtMocHelpers::PropertyData<QVariantList>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'token'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'tokenType'
        QtMocHelpers::PropertyData<KItinerary::Token::TokenType>(9, 0x80000000 | 10, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
        // property 'tokenData'
        QtMocHelpers::PropertyData<QVariant>(11, 0x80000000 | 12, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
        // property 'validFrom'
        QtMocHelpers::PropertyData<QDateTime>(13, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'validUntil'
        QtMocHelpers::PropertyData<QDateTime>(14, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProgramMembership, qt_meta_tag_ZN10KItinerary17ProgramMembershipE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary17ProgramMembershipE[] = {
    QMetaObject::SuperData::link<KItinerary::Token::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::ProgramMembership::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17ProgramMembershipE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17ProgramMembershipE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary17ProgramMembershipE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary17ProgramMembershipE_t>.metaTypes,
    nullptr
} };

void KItinerary::ProgramMembership::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ProgramMembership *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Person >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->programName(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->membershipNumber(); break;
        case 3: *reinterpret_cast<KItinerary::Person*>(_v) = _t->member(); break;
        case 4: *reinterpret_cast<QVariantList*>(_v) = _t->subjectOf(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->token(); break;
        case 6: *reinterpret_cast<KItinerary::Token::TokenType*>(_v) = _t->tokenType(); break;
        case 7: *reinterpret_cast<QVariant*>(_v) = _t->tokenData(); break;
        case 8: *reinterpret_cast<QDateTime*>(_v) = _t->validFrom(); break;
        case 9: *reinterpret_cast<QDateTime*>(_v) = _t->validUntil(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setProgramName(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setMembershipNumber(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setMember(*reinterpret_cast<KItinerary::Person*>(_v)); break;
        case 4: _t->setSubjectOf(*reinterpret_cast<QVariantList*>(_v)); break;
        case 5: _t->setToken(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setValidFrom(*reinterpret_cast<QDateTime*>(_v)); break;
        case 9: _t->setValidUntil(*reinterpret_cast<QDateTime*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
