/****************************************************************************
** Meta object code from reading C++ file 'ssbv1ticket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/era/ssbv1ticket.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ssbv1ticket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary11SSBv1TicketE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::SSBv1Ticket::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary11SSBv1TicketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::SSBv1Ticket",
        "firstDayOfValidity",
        "",
        "contextDate",
        "departureTime",
        "version",
        "issuerCode",
        "rct2TypeIndicator",
        "numberOfTickets",
        "numberOfAdultPassengers",
        "numberOfChildPassengers",
        "firstDayOfValidityDay",
        "lastDayOfValidityDay",
        "customerNumberType",
        "customerNumber",
        "departureStationType",
        "departureStationNum",
        "departureStationAlpha",
        "arrivalStationType",
        "arrivalStationNum",
        "arrivalStationAlpha",
        "departureTimeSlot",
        "trainNumber",
        "reservationReference",
        "classOfTransport",
        "coachNumber",
        "seatNumber",
        "berthNumber",
        "overbookingIndicator",
        "issuerPNRNumber",
        "ticketType",
        "specimen",
        "viaStations"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'firstDayOfValidity'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'firstDayOfValidity'
        QtMocHelpers::MethodData<QDate() const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
        // Method 'departureTime'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(4, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'departureTime'
        QtMocHelpers::MethodData<QDateTime() const>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDateTime),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'version'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'issuerCode'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'rct2TypeIndicator'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'numberOfTickets'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'numberOfAdultPassengers'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'numberOfChildPassengers'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'firstDayOfValidityDay'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'lastDayOfValidityDay'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'customerNumberType'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'customerNumber'
        QtMocHelpers::PropertyData<quint64>(14, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'departureStationType'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'departureStationNum'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'departureStationAlpha'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'arrivalStationType'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'arrivalStationNum'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'arrivalStationAlpha'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'departureTimeSlot'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'trainNumber'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'reservationReference'
        QtMocHelpers::PropertyData<quint64>(23, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'classOfTransport'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'coachNumber'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'seatNumber'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'berthNumber'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'overbookingIndicator'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'issuerPNRNumber'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'ticketType'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'specimen'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'viaStations'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SSBv1Ticket, qt_meta_tag_ZN10KItinerary11SSBv1TicketE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::SSBv1Ticket::staticMetaObject = { {
    QtPrivate::MetaObjectForType<SSBTicketBase>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary11SSBv1TicketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary11SSBv1TicketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary11SSBv1TicketE_t>.metaTypes,
    nullptr
} };

void KItinerary::SSBv1Ticket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<SSBv1Ticket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDate _r = _t->firstDayOfValidity((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDate _r = _t->firstDayOfValidity();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDateTime _r = _t->departureTime((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDateTime _r = _t->departureTime();
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->version(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->issuerCode(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->rct2TypeIndicator(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->numberOfTickets(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->numberOfAdultPassengers(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->numberOfChildPassengers(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->firstDayOfValidityDay(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->lastDayOfValidityDay(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->customerNumberType(); break;
        case 9: *reinterpret_cast<quint64*>(_v) = _t->customerNumber(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->departureStationType(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->departureStationNum(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->departureStationAlpha(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->arrivalStationType(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->arrivalStationNum(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->arrivalStationAlpha(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->departureTimeSlot(); break;
        case 17: *reinterpret_cast<int*>(_v) = _t->trainNumber(); break;
        case 18: *reinterpret_cast<quint64*>(_v) = _t->reservationReference(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->classOfTransport(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->coachNumber(); break;
        case 21: *reinterpret_cast<int*>(_v) = _t->seatNumber(); break;
        case 22: *reinterpret_cast<QString*>(_v) = _t->berthNumber(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->overbookingIndicator(); break;
        case 24: *reinterpret_cast<QString*>(_v) = _t->issuerPNRNumber(); break;
        case 25: *reinterpret_cast<int*>(_v) = _t->ticketType(); break;
        case 26: *reinterpret_cast<int*>(_v) = _t->specimen(); break;
        case 27: *reinterpret_cast<QString*>(_v) = _t->viaStations(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
