/****************************************************************************
** Meta object code from reading C++ file 'iatabcbp.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/iata/iatabcbp.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'iatabcbp.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary8IataBcbpE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::IataBcbp::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary8IataBcbpE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::IataBcbp",
        "repeatedMandatorySection",
        "KItinerary::IataBcbpRepeatedMandatorySection",
        "",
        "leg",
        "repeatedConditionalSection",
        "KItinerary::IataBcbpRepeatedConditionalSection",
        "airlineUseSection",
        "uniqueMandatorySection",
        "KItinerary::IataBcbpUniqueMandatorySection",
        "uniqueConditionalSection",
        "KItinerary::IataBcbpUniqueConditionalSection",
        "securitySection",
        "KItinerary::IataBcbpSecuritySection",
        "rawData"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'repeatedMandatorySection'
        QtMocHelpers::MethodData<KItinerary::IataBcbpRepeatedMandatorySection(int) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::Int, 4 },
        }}),
        // Method 'repeatedConditionalSection'
        QtMocHelpers::MethodData<KItinerary::IataBcbpRepeatedConditionalSection(int) const>(5, 3, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 4 },
        }}),
        // Method 'airlineUseSection'
        QtMocHelpers::MethodData<QString(int) const>(7, 3, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'uniqueMandatorySection'
        QtMocHelpers::PropertyData<KItinerary::IataBcbpUniqueMandatorySection>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'uniqueConditionalSection'
        QtMocHelpers::PropertyData<KItinerary::IataBcbpUniqueConditionalSection>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'securitySection'
        QtMocHelpers::PropertyData<KItinerary::IataBcbpSecuritySection>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'rawData'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IataBcbp, qt_meta_tag_ZN10KItinerary8IataBcbpE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::IataBcbp::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary8IataBcbpE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary8IataBcbpE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary8IataBcbpE_t>.metaTypes,
    nullptr
} };

void KItinerary::IataBcbp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IataBcbp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { KItinerary::IataBcbpRepeatedMandatorySection _r = _t->repeatedMandatorySection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KItinerary::IataBcbpRepeatedMandatorySection*>(_a[0]) = std::move(_r); }  break;
        case 1: { KItinerary::IataBcbpRepeatedConditionalSection _r = _t->repeatedConditionalSection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KItinerary::IataBcbpRepeatedConditionalSection*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->airlineUseSection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::IataBcbpUniqueMandatorySection*>(_v) = _t->uniqueMandatorySection(); break;
        case 1: *reinterpret_cast<KItinerary::IataBcbpUniqueConditionalSection*>(_v) = _t->uniqueConditionalSection(); break;
        case 2: *reinterpret_cast<KItinerary::IataBcbpSecuritySection*>(_v) = _t->securitySection(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->rawData(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
