/****************************************************************************
** Meta object code from reading C++ file 'ticket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/ticket.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ticket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary4SeatE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Seat::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary4SeatE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Seat",
        "className",
        "seatNumber",
        "seatRow",
        "seatSection",
        "seatingType",
        "identifier"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'seatNumber'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'seatRow'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'seatSection'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'seatingType'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'identifier'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Seat, qt_meta_tag_ZN10KItinerary4SeatE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Seat::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary4SeatE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary4SeatE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary4SeatE_t>.metaTypes,
    nullptr
} };

void KItinerary::Seat::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Seat *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->seatNumber(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->seatRow(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->seatSection(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->seatingType(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->identifier(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSeatNumber(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setSeatRow(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setSeatSection(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setSeatingType(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setIdentifier(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary6TicketE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Ticket::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary6TicketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Ticket",
        "className",
        "name",
        "issuedBy",
        "KItinerary::Organization",
        "ticketNumber",
        "ticketedSeat",
        "KItinerary::Seat",
        "ticketToken",
        "ticketTokenType",
        "KItinerary::Token::TokenType",
        "ticketTokenData",
        "QVariant",
        "underName",
        "KItinerary::Person",
        "subjectOf",
        "QVariantList",
        "validFrom",
        "validUntil",
        "totalPrice",
        "priceCurrency",
        "identifier",
        "TicketTokenType",
        "Unknown",
        "Url",
        "QRCode",
        "AztecCode",
        "Code128",
        "DataMatrix",
        "PDF417"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'issuedBy'
        QtMocHelpers::PropertyData<KItinerary::Organization>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'ticketNumber'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'ticketedSeat'
        QtMocHelpers::PropertyData<KItinerary::Seat>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'ticketToken'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'ticketTokenType'
        QtMocHelpers::PropertyData<KItinerary::Token::TokenType>(9, 0x80000000 | 10, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
        // property 'ticketTokenData'
        QtMocHelpers::PropertyData<QVariant>(11, 0x80000000 | 12, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
        // property 'underName'
        QtMocHelpers::PropertyData<KItinerary::Person>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'subjectOf'
        QtMocHelpers::PropertyData<QVariantList>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'validFrom'
        QtMocHelpers::PropertyData<QDateTime>(17, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'validUntil'
        QtMocHelpers::PropertyData<QDateTime>(18, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'totalPrice'
        QtMocHelpers::PropertyData<double>(19, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'priceCurrency'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'identifier'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TicketTokenType'
        QtMocHelpers::EnumData<enum TicketTokenType>(22, 22, QMC::EnumFlags{}).add({
            {   23, TicketTokenType::Unknown },
            {   24, TicketTokenType::Url },
            {   25, TicketTokenType::QRCode },
            {   26, TicketTokenType::AztecCode },
            {   27, TicketTokenType::Code128 },
            {   28, TicketTokenType::DataMatrix },
            {   29, TicketTokenType::PDF417 },
        }),
    };
    return QtMocHelpers::metaObjectData<Ticket, qt_meta_tag_ZN10KItinerary6TicketE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary6TicketE[] = {
    QMetaObject::SuperData::link<KItinerary::Token::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Ticket::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary6TicketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary6TicketE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary6TicketE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary6TicketE_t>.metaTypes,
    nullptr
} };

void KItinerary::Ticket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Ticket *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Organization >(); break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Person >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Seat >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<KItinerary::Organization*>(_v) = _t->issuedBy(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->ticketNumber(); break;
        case 4: *reinterpret_cast<KItinerary::Seat*>(_v) = _t->ticketedSeat(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->ticketToken(); break;
        case 6: *reinterpret_cast<KItinerary::Token::TokenType*>(_v) = _t->ticketTokenType(); break;
        case 7: *reinterpret_cast<QVariant*>(_v) = _t->ticketTokenData(); break;
        case 8: *reinterpret_cast<KItinerary::Person*>(_v) = _t->underName(); break;
        case 9: *reinterpret_cast<QVariantList*>(_v) = _t->subjectOf(); break;
        case 10: *reinterpret_cast<QDateTime*>(_v) = _t->validFrom(); break;
        case 11: *reinterpret_cast<QDateTime*>(_v) = _t->validUntil(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->totalPrice(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->priceCurrency(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->identifier(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setIssuedBy(*reinterpret_cast<KItinerary::Organization*>(_v)); break;
        case 3: _t->setTicketNumber(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setTicketedSeat(*reinterpret_cast<KItinerary::Seat*>(_v)); break;
        case 5: _t->setTicketToken(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setUnderName(*reinterpret_cast<KItinerary::Person*>(_v)); break;
        case 9: _t->setSubjectOf(*reinterpret_cast<QVariantList*>(_v)); break;
        case 10: _t->setValidFrom(*reinterpret_cast<QDateTime*>(_v)); break;
        case 11: _t->setValidUntil(*reinterpret_cast<QDateTime*>(_v)); break;
        case 12: _t->setTotalPrice(*reinterpret_cast<double*>(_v)); break;
        case 13: _t->setPriceCurrency(*reinterpret_cast<QString*>(_v)); break;
        case 14: _t->setIdentifier(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
