/****************************************************************************
** Meta object code from reading C++ file 'reservation.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/reservation.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'reservation.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary11ReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Reservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary11ReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Reservation",
        "className",
        "reservationNumber",
        "reservationFor",
        "QVariant",
        "reservedTicket",
        "underName",
        "provider",
        "KItinerary::Organization",
        "url",
        "QUrl",
        "potentialAction",
        "QVariantList",
        "modifiedTime",
        "subjectOf",
        "reservationStatus",
        "ReservationStatus",
        "programMembershipUsed",
        "KItinerary::ProgramMembership",
        "pkpassPassTypeIdentifier",
        "pkpassSerialNumber",
        "totalPrice",
        "priceCurrency",
        "ReservationConfirmed",
        "ReservationCancelled",
        "ReservationHold",
        "ReservationPending"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'reservationNumber'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'reservationFor'
        QtMocHelpers::PropertyData<QVariant>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'reservedTicket'
        QtMocHelpers::PropertyData<QVariant>(5, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'underName'
        QtMocHelpers::PropertyData<QVariant>(6, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'provider'
        QtMocHelpers::PropertyData<KItinerary::Organization>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'potentialAction'
        QtMocHelpers::PropertyData<QVariantList>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'modifiedTime'
        QtMocHelpers::PropertyData<QDateTime>(13, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'subjectOf'
        QtMocHelpers::PropertyData<QVariantList>(14, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'reservationStatus'
        QtMocHelpers::PropertyData<enum ReservationStatus>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'programMembershipUsed'
        QtMocHelpers::PropertyData<KItinerary::ProgramMembership>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'pkpassPassTypeIdentifier'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'pkpassSerialNumber'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'totalPrice'
        QtMocHelpers::PropertyData<double>(21, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'priceCurrency'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ReservationStatus'
        QtMocHelpers::EnumData<enum ReservationStatus>(16, 16, QMC::EnumFlags{}).add({
            {   23, ReservationStatus::ReservationConfirmed },
            {   24, ReservationStatus::ReservationCancelled },
            {   25, ReservationStatus::ReservationHold },
            {   26, ReservationStatus::ReservationPending },
        }),
    };
    return QtMocHelpers::metaObjectData<Reservation, qt_meta_tag_ZN10KItinerary11ReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Reservation::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary11ReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary11ReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary11ReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::Reservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Reservation *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Organization >(); break;
        case 11:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::ProgramMembership >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->reservationNumber(); break;
        case 2: *reinterpret_cast<QVariant*>(_v) = _t->reservationFor(); break;
        case 3: *reinterpret_cast<QVariant*>(_v) = _t->reservedTicket(); break;
        case 4: *reinterpret_cast<QVariant*>(_v) = _t->underName(); break;
        case 5: *reinterpret_cast<KItinerary::Organization*>(_v) = _t->provider(); break;
        case 6: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 7: *reinterpret_cast<QVariantList*>(_v) = _t->potentialAction(); break;
        case 8: *reinterpret_cast<QDateTime*>(_v) = _t->modifiedTime(); break;
        case 9: *reinterpret_cast<QVariantList*>(_v) = _t->subjectOf(); break;
        case 10: *reinterpret_cast<enum ReservationStatus*>(_v) = _t->reservationStatus(); break;
        case 11: *reinterpret_cast<KItinerary::ProgramMembership*>(_v) = _t->programMembershipUsed(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->pkpassPassTypeIdentifier(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->pkpassSerialNumber(); break;
        case 14: *reinterpret_cast<double*>(_v) = _t->totalPrice(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->priceCurrency(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setReservationNumber(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setReservationFor(*reinterpret_cast<QVariant*>(_v)); break;
        case 3: _t->setReservedTicket(*reinterpret_cast<QVariant*>(_v)); break;
        case 4: _t->setUnderName(*reinterpret_cast<QVariant*>(_v)); break;
        case 5: _t->setProvider(*reinterpret_cast<KItinerary::Organization*>(_v)); break;
        case 6: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 7: _t->setPotentialAction(*reinterpret_cast<QVariantList*>(_v)); break;
        case 8: _t->setModifiedTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 9: _t->setSubjectOf(*reinterpret_cast<QVariantList*>(_v)); break;
        case 10: _t->setReservationStatus(*reinterpret_cast<enum ReservationStatus*>(_v)); break;
        case 11: _t->setProgramMembershipUsed(*reinterpret_cast<KItinerary::ProgramMembership*>(_v)); break;
        case 12: _t->setPkpassPassTypeIdentifier(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setPkpassSerialNumber(*reinterpret_cast<QString*>(_v)); break;
        case 14: _t->setTotalPrice(*reinterpret_cast<double*>(_v)); break;
        case 15: _t->setPriceCurrency(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary18LodgingReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::LodgingReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary18LodgingReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::LodgingReservation",
        "className",
        "checkinTime",
        "checkoutTime",
        "lodgingUnitDescription"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'checkinTime'
        QtMocHelpers::PropertyData<QDateTime>(2, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'checkoutTime'
        QtMocHelpers::PropertyData<QDateTime>(3, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'lodgingUnitDescription'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LodgingReservation, qt_meta_tag_ZN10KItinerary18LodgingReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::LodgingReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary18LodgingReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary18LodgingReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary18LodgingReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::LodgingReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<LodgingReservation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QDateTime*>(_v) = _t->checkinTime(); break;
        case 2: *reinterpret_cast<QDateTime*>(_v) = _t->checkoutTime(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->lodgingUnitDescription(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setCheckinTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 2: _t->setCheckoutTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 3: _t->setLodgingUnitDescription(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary17FlightReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::FlightReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary17FlightReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::FlightReservation",
        "className",
        "passengerSequenceNumber",
        "airplaneSeat",
        "boardingGroup"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'passengerSequenceNumber'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'airplaneSeat'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'boardingGroup'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FlightReservation, qt_meta_tag_ZN10KItinerary17FlightReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::FlightReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17FlightReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17FlightReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary17FlightReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::FlightReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<FlightReservation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->passengerSequenceNumber(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->airplaneSeat(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->boardingGroup(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setPassengerSequenceNumber(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setAirplaneSeat(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setBoardingGroup(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary16TrainReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::TrainReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary16TrainReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::TrainReservation",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TrainReservation, qt_meta_tag_ZN10KItinerary16TrainReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::TrainReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16TrainReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16TrainReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary16TrainReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::TrainReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TrainReservation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary14BusReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::BusReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary14BusReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::BusReservation",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BusReservation, qt_meta_tag_ZN10KItinerary14BusReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::BusReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary14BusReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary14BusReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary14BusReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::BusReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<BusReservation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary28FoodEstablishmentReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::FoodEstablishmentReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary28FoodEstablishmentReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::FoodEstablishmentReservation",
        "className",
        "endTime",
        "partySize",
        "startTime"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'endTime'
        QtMocHelpers::PropertyData<QDateTime>(2, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'partySize'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'startTime'
        QtMocHelpers::PropertyData<QDateTime>(4, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FoodEstablishmentReservation, qt_meta_tag_ZN10KItinerary28FoodEstablishmentReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::FoodEstablishmentReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary28FoodEstablishmentReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary28FoodEstablishmentReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary28FoodEstablishmentReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::FoodEstablishmentReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<FoodEstablishmentReservation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QDateTime*>(_v) = _t->endTime(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->partySize(); break;
        case 3: *reinterpret_cast<QDateTime*>(_v) = _t->startTime(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setEndTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 2: _t->setPartySize(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setStartTime(*reinterpret_cast<QDateTime*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary16EventReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::EventReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary16EventReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::EventReservation",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EventReservation, qt_meta_tag_ZN10KItinerary16EventReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::EventReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16EventReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16EventReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary16EventReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::EventReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<EventReservation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary20RentalCarReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::RentalCarReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary20RentalCarReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::RentalCarReservation",
        "className",
        "dropoffTime",
        "pickupTime",
        "pickupLocation",
        "KItinerary::Place",
        "dropoffLocation"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'dropoffTime'
        QtMocHelpers::PropertyData<QDateTime>(2, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'pickupTime'
        QtMocHelpers::PropertyData<QDateTime>(3, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'pickupLocation'
        QtMocHelpers::PropertyData<KItinerary::Place>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'dropoffLocation'
        QtMocHelpers::PropertyData<KItinerary::Place>(6, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RentalCarReservation, qt_meta_tag_ZN10KItinerary20RentalCarReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::RentalCarReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary20RentalCarReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary20RentalCarReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary20RentalCarReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::RentalCarReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RentalCarReservation *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Place >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QDateTime*>(_v) = _t->dropoffTime(); break;
        case 2: *reinterpret_cast<QDateTime*>(_v) = _t->pickupTime(); break;
        case 3: *reinterpret_cast<KItinerary::Place*>(_v) = _t->pickupLocation(); break;
        case 4: *reinterpret_cast<KItinerary::Place*>(_v) = _t->dropoffLocation(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setDropoffTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 2: _t->setPickupTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 3: _t->setPickupLocation(*reinterpret_cast<KItinerary::Place*>(_v)); break;
        case 4: _t->setDropoffLocation(*reinterpret_cast<KItinerary::Place*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary15TaxiReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::TaxiReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary15TaxiReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::TaxiReservation",
        "className",
        "pickupTime",
        "pickupLocation",
        "KItinerary::Place"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'pickupTime'
        QtMocHelpers::PropertyData<QDateTime>(2, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'pickupLocation'
        QtMocHelpers::PropertyData<KItinerary::Place>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaxiReservation, qt_meta_tag_ZN10KItinerary15TaxiReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::TaxiReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15TaxiReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15TaxiReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary15TaxiReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::TaxiReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TaxiReservation *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Place >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QDateTime*>(_v) = _t->pickupTime(); break;
        case 2: *reinterpret_cast<KItinerary::Place*>(_v) = _t->pickupLocation(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setPickupTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 2: _t->setPickupLocation(*reinterpret_cast<KItinerary::Place*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary15BoatReservationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::BoatReservation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary15BoatReservationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::BoatReservation",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BoatReservation, qt_meta_tag_ZN10KItinerary15BoatReservationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::BoatReservation::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Reservation>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15BoatReservationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15BoatReservationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary15BoatReservationE_t>.metaTypes,
    nullptr
} };

void KItinerary::BoatReservation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<BoatReservation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
