/****************************************************************************
** Meta object code from reading C++ file 'rct2ticket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/uic9183/rct2ticket.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rct2ticket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary10Rct2TicketE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Rct2Ticket::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary10Rct2TicketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Rct2Ticket",
        "firstDayOfValidity",
        "type",
        "Type",
        "title",
        "passengerName",
        "outboundDepartureTime",
        "outboundArrivalTime",
        "outboundDepartureStation",
        "outboundArrivalStation",
        "outboundClass",
        "returnDepartureTime",
        "returnArrivalTime",
        "returnDepartureStation",
        "returnArrivalStation",
        "returnClass",
        "trainNumber",
        "coachNumber",
        "seatNumber",
        "currency",
        "price",
        "Transport",
        "TransportReservation",
        "Reservation",
        "Upgrade",
        "RailPass",
        "Unknown"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'firstDayOfValidity'
        QtMocHelpers::PropertyData<QDate>(1, QMetaType::QDate, QMC::DefaultPropertyFlags),
        // property 'type'
        QtMocHelpers::PropertyData<enum Type>(2, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'passengerName'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'outboundDepartureTime'
        QtMocHelpers::PropertyData<QDateTime>(6, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'outboundArrivalTime'
        QtMocHelpers::PropertyData<QDateTime>(7, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'outboundDepartureStation'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'outboundArrivalStation'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'outboundClass'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'returnDepartureTime'
        QtMocHelpers::PropertyData<QDateTime>(11, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'returnArrivalTime'
        QtMocHelpers::PropertyData<QDateTime>(12, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'returnDepartureStation'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'returnArrivalStation'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'returnClass'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'trainNumber'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'coachNumber'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'seatNumber'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'currency'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'price'
        QtMocHelpers::PropertyData<double>(20, QMetaType::Double, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(3, 3, QMC::EnumFlags{}).add({
            {   21, Type::Transport },
            {   22, Type::TransportReservation },
            {   23, Type::Reservation },
            {   24, Type::Upgrade },
            {   25, Type::RailPass },
            {   26, Type::Unknown },
        }),
    };
    return QtMocHelpers::metaObjectData<Rct2Ticket, qt_meta_tag_ZN10KItinerary10Rct2TicketE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Rct2Ticket::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10Rct2TicketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10Rct2TicketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary10Rct2TicketE_t>.metaTypes,
    nullptr
} };

void KItinerary::Rct2Ticket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Rct2Ticket *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDate*>(_v) = _t->firstDayOfValidity(); break;
        case 1: *reinterpret_cast<enum Type*>(_v) = _t->type(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->passengerName(); break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->outboundDepartureTime(); break;
        case 5: *reinterpret_cast<QDateTime*>(_v) = _t->outboundArrivalTime(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->outboundDepartureStation(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->outboundArrivalStation(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->outboundClass(); break;
        case 9: *reinterpret_cast<QDateTime*>(_v) = _t->returnDepartureTime(); break;
        case 10: *reinterpret_cast<QDateTime*>(_v) = _t->returnArrivalTime(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->returnDepartureStation(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->returnArrivalStation(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->returnClass(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->trainNumber(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->coachNumber(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->seatNumber(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->currency(); break;
        case 18: *reinterpret_cast<double*>(_v) = _t->price(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
