/****************************************************************************
** Meta object code from reading C++ file 'elbticket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/era/elbticket.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'elbticket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary9ELBTicketE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::ELBTicket::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary9ELBTicketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::ELBTicket",
        "emissionDate",
        "",
        "contextDate",
        "validFromDate",
        "validUntilDate",
        "idFormat",
        "pectabCode",
        "ticketCode",
        "pnr",
        "tcnCode",
        "specimen",
        "barcodeVersion",
        "sequenceNumberCurrent",
        "sequenceNumberTotal",
        "futureUse",
        "travelerType",
        "numberAdults",
        "numberChildren",
        "year",
        "emissionDay",
        "beginValidityDay",
        "endValidityDay",
        "segment1",
        "KItinerary::ELBTicketSegment",
        "segment2",
        "rawData"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'emissionDate'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'emissionDate'
        QtMocHelpers::MethodData<QDate() const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
        // Method 'validFromDate'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(4, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'validFromDate'
        QtMocHelpers::MethodData<QDateTime() const>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDateTime),
        // Method 'validUntilDate'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(5, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'validUntilDate'
        QtMocHelpers::MethodData<QDateTime() const>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDateTime),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'idFormat'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'pectabCode'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'ticketCode'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'pnr'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'tcnCode'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'specimen'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'barcodeVersion'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'sequenceNumberCurrent'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'sequenceNumberTotal'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'futureUse'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'travelerType'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'numberAdults'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'numberChildren'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'year'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'emissionDay'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'beginValidityDay'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'endValidityDay'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'segment1'
        QtMocHelpers::PropertyData<KItinerary::ELBTicketSegment>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'segment2'
        QtMocHelpers::PropertyData<KItinerary::ELBTicketSegment>(25, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'rawData'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ELBTicket, qt_meta_tag_ZN10KItinerary9ELBTicketE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::ELBTicket::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary9ELBTicketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary9ELBTicketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary9ELBTicketE_t>.metaTypes,
    nullptr
} };

void KItinerary::ELBTicket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ELBTicket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDate _r = _t->emissionDate((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDate _r = _t->emissionDate();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDateTime _r = _t->validFromDate((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDateTime _r = _t->validFromDate();
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDateTime _r = _t->validUntilDate((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDateTime _r = _t->validUntilDate();
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->idFormat(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->pectabCode(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->ticketCode(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->pnr(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->tcnCode(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->specimen(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->barcodeVersion(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->sequenceNumberCurrent(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->sequenceNumberTotal(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->futureUse(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->travelerType(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->numberAdults(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->numberChildren(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->year(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->emissionDay(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->beginValidityDay(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->endValidityDay(); break;
        case 17: *reinterpret_cast<KItinerary::ELBTicketSegment*>(_v) = _t->segment1(); break;
        case 18: *reinterpret_cast<KItinerary::ELBTicketSegment*>(_v) = _t->segment2(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->rawData(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary16ELBTicketSegmentE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::ELBTicketSegment::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary16ELBTicketSegmentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::ELBTicketSegment",
        "departureDate",
        "",
        "contextDate",
        "departureStation",
        "arrivalStation",
        "trainNumber",
        "securityCode",
        "departureDay",
        "coachNumber",
        "seatNumber",
        "classOfTransport",
        "tariffCode",
        "classOfService",
        "isValid"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'departureDate'
        QtMocHelpers::MethodData<QDate(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDate, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'departureDate'
        QtMocHelpers::MethodData<QDate() const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QDate),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'departureStation'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'arrivalStation'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'trainNumber'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'securityCode'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'departureDay'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'coachNumber'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'seatNumber'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'classOfTransport'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'tariffCode'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'classOfService'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'isValid'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ELBTicketSegment, qt_meta_tag_ZN10KItinerary16ELBTicketSegmentE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::ELBTicketSegment::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16ELBTicketSegmentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16ELBTicketSegmentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary16ELBTicketSegmentE_t>.metaTypes,
    nullptr
} };

void KItinerary::ELBTicketSegment::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ELBTicketSegment *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDate _r = _t->departureDate((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDate _r = _t->departureDate();
            if (_a[0]) *reinterpret_cast<QDate*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->departureStation(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->arrivalStation(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->trainNumber(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->securityCode(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->departureDay(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->coachNumber(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->seatNumber(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->classOfTransport(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->tariffCode(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->classOfService(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
