/****************************************************************************
** Meta object code from reading C++ file 'extractordocumentnode.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/engine/extractordocumentnode.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'extractordocumentnode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary21ExtractorDocumentNodeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::ExtractorDocumentNode::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary21ExtractorDocumentNodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::ExtractorDocumentNode",
        "findChildNodes",
        "QVariantList",
        "",
        "QJSValue",
        "jsFilter",
        "isNull",
        "parent",
        "childNodes",
        "mimeType",
        "content",
        "contextDateTime",
        "result",
        "QJsonArray",
        "location",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'findChildNodes'
        QtMocHelpers::MethodData<QVariantList(const QJSValue &) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isNull'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'parent'
        QtMocHelpers::PropertyData<KItinerary::ExtractorDocumentNode>(7, 0x80000000 | 0, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'childNodes'
        QtMocHelpers::PropertyData<QVariantList>(8, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'mimeType'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'content'
        QtMocHelpers::PropertyData<QJSValue>(10, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'contextDateTime'
        QtMocHelpers::PropertyData<QDateTime>(11, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'result'
        QtMocHelpers::PropertyData<QJsonArray>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'location'
        QtMocHelpers::PropertyData<QVariant>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExtractorDocumentNode, qt_meta_tag_ZN10KItinerary21ExtractorDocumentNodeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::ExtractorDocumentNode::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary21ExtractorDocumentNodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary21ExtractorDocumentNodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary21ExtractorDocumentNodeE_t>.metaTypes,
    nullptr
} };

void KItinerary::ExtractorDocumentNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ExtractorDocumentNode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QVariantList _r = _t->findChildNodes((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isNull(); break;
        case 1: *reinterpret_cast<KItinerary::ExtractorDocumentNode*>(_v) = _t->parent(); break;
        case 2: *reinterpret_cast<QVariantList*>(_v) = _t->childNodesVariant(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->mimeType(); break;
        case 4: *reinterpret_cast<QJSValue*>(_v) = _t->contentJsValue(); break;
        case 5: *reinterpret_cast<QDateTime*>(_v) = _t->contextDateTime(); break;
        case 6: *reinterpret_cast<QJsonArray*>(_v) = _t->jsonLdResult(); break;
        case 7: *reinterpret_cast<QVariant*>(_v) = _t->location(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
