/****************************************************************************
** Meta object code from reading C++ file 'action.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/action.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'action.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary6ActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Action::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary6ActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Action",
        "className",
        "target",
        "QUrl",
        "result",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'target'
        QtMocHelpers::PropertyData<QUrl>(2, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'result'
        QtMocHelpers::PropertyData<QVariant>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Action, qt_meta_tag_ZN10KItinerary6ActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Action::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary6ActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary6ActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary6ActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::Action::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Action *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->target(); break;
        case 2: *reinterpret_cast<QVariant*>(_v) = _t->result(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setTarget(*reinterpret_cast<QUrl*>(_v)); break;
        case 2: _t->setResult(*reinterpret_cast<QVariant*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary12CancelActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::CancelAction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary12CancelActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::CancelAction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CancelAction, qt_meta_tag_ZN10KItinerary12CancelActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::CancelAction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Action>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12CancelActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12CancelActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary12CancelActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::CancelAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CancelAction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary13CheckInActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::CheckInAction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary13CheckInActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::CheckInAction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CheckInAction, qt_meta_tag_ZN10KItinerary13CheckInActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::CheckInAction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Action>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13CheckInActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13CheckInActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary13CheckInActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::CheckInAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CheckInAction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary14DownloadActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::DownloadAction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary14DownloadActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::DownloadAction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DownloadAction, qt_meta_tag_ZN10KItinerary14DownloadActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::DownloadAction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Action>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary14DownloadActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary14DownloadActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary14DownloadActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::DownloadAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<DownloadAction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary10JoinActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::JoinAction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary10JoinActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::JoinAction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JoinAction, qt_meta_tag_ZN10KItinerary10JoinActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::JoinAction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Action>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10JoinActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10JoinActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary10JoinActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::JoinAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<JoinAction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary13ReserveActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::ReserveAction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary13ReserveActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::ReserveAction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ReserveAction, qt_meta_tag_ZN10KItinerary13ReserveActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::ReserveAction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Action>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13ReserveActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13ReserveActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary13ReserveActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::ReserveAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ReserveAction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary12UpdateActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::UpdateAction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary12UpdateActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::UpdateAction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UpdateAction, qt_meta_tag_ZN10KItinerary12UpdateActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::UpdateAction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Action>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12UpdateActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12UpdateActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary12UpdateActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::UpdateAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<UpdateAction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary10ViewActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::ViewAction::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary10ViewActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::ViewAction",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewAction, qt_meta_tag_ZN10KItinerary10ViewActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::ViewAction::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Action>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10ViewActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary10ViewActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary10ViewActionE_t>.metaTypes,
    nullptr
} };

void KItinerary::ViewAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ViewAction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
