/****************************************************************************
** Meta object code from reading C++ file 'vdvticketcontent.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/vdv/vdvticketcontent.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vdvticketcontent.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary15VdvTicketHeaderE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketHeader::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary15VdvTicketHeaderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketHeader",
        "ticketId",
        "kvpOrgId",
        "productId",
        "pvOrgId",
        "validityBegin",
        "validityEnd"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'ticketId'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'kvpOrgId'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'productId'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'pvOrgId'
        QtMocHelpers::PropertyData<uint>(4, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'validityBegin'
        QtMocHelpers::PropertyData<QDateTime>(5, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'validityEnd'
        QtMocHelpers::PropertyData<QDateTime>(6, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketHeader, qt_meta_tag_ZN10KItinerary15VdvTicketHeaderE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketHeader::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15VdvTicketHeaderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15VdvTicketHeaderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary15VdvTicketHeaderE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketHeader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketHeader *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->ticketId; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->kvpOrgId; break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->productId; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->pvOrgId; break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->validityBegin; break;
        case 5: *reinterpret_cast<QDateTime*>(_v) = _t->validityEnd; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->ticketId, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->kvpOrgId, *reinterpret_cast<uint*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->productId, *reinterpret_cast<uint*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->pvOrgId, *reinterpret_cast<uint*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->validityBegin, *reinterpret_cast<QDateTime*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->validityEnd, *reinterpret_cast<QDateTime*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary18VdvTicketBasicDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketBasicData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary18VdvTicketBasicDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketBasicData",
        "paymentType",
        "travelerType",
        "includedTravelerType1",
        "includedTravelerCount1",
        "includedTravelerType2",
        "includedTravelerCount2",
        "categroy",
        "serviceClass",
        "price",
        "vat",
        "priceCategory",
        "productNumber"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'paymentType'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'travelerType'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'includedTravelerType1'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'includedTravelerCount1'
        QtMocHelpers::PropertyData<uint>(4, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'includedTravelerType2'
        QtMocHelpers::PropertyData<uint>(5, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'includedTravelerCount2'
        QtMocHelpers::PropertyData<uint>(6, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'categroy'
        QtMocHelpers::PropertyData<uint>(7, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'serviceClass'
        QtMocHelpers::PropertyData<uint>(8, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'price'
        QtMocHelpers::PropertyData<uint>(9, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'vat'
        QtMocHelpers::PropertyData<uint>(10, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'priceCategory'
        QtMocHelpers::PropertyData<uint>(11, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'productNumber'
        QtMocHelpers::PropertyData<uint>(12, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketBasicData, qt_meta_tag_ZN10KItinerary18VdvTicketBasicDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketBasicData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary18VdvTicketBasicDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary18VdvTicketBasicDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary18VdvTicketBasicDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketBasicData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketBasicData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->paymentType; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->travelerType; break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->includedTravelerType1; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->includedTravelerCount1; break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->includedTravelerType2; break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->includedTravelerCount2; break;
        case 6: *reinterpret_cast<uint*>(_v) = _t->categroy; break;
        case 7: *reinterpret_cast<uint*>(_v) = _t->serviceClass; break;
        case 8: *reinterpret_cast<uint*>(_v) = _t->price; break;
        case 9: *reinterpret_cast<uint*>(_v) = _t->vat; break;
        case 10: *reinterpret_cast<uint*>(_v) = _t->priceCategory; break;
        case 11: *reinterpret_cast<uint*>(_v) = _t->productNumber; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->paymentType, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->travelerType, *reinterpret_cast<uint*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->includedTravelerType1, *reinterpret_cast<uint*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->includedTravelerCount1, *reinterpret_cast<uint*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->includedTravelerType2, *reinterpret_cast<uint*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->includedTravelerCount2, *reinterpret_cast<uint*>(_v)); break;
        case 6: QtMocHelpers::setProperty(_t->categroy, *reinterpret_cast<uint*>(_v)); break;
        case 7: QtMocHelpers::setProperty(_t->serviceClass, *reinterpret_cast<uint*>(_v)); break;
        case 8: QtMocHelpers::setProperty(_t->price, *reinterpret_cast<uint*>(_v)); break;
        case 9: QtMocHelpers::setProperty(_t->vat, *reinterpret_cast<uint*>(_v)); break;
        case 10: QtMocHelpers::setProperty(_t->priceCategory, *reinterpret_cast<uint*>(_v)); break;
        case 11: QtMocHelpers::setProperty(_t->productNumber, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary21VdvTicketTravelerDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketTravelerData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary21VdvTicketTravelerDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketTravelerData",
        "gender",
        "birthDate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'gender'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'birthDate'
        QtMocHelpers::PropertyData<QDate>(2, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketTravelerData, qt_meta_tag_ZN10KItinerary21VdvTicketTravelerDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketTravelerData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary21VdvTicketTravelerDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary21VdvTicketTravelerDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary21VdvTicketTravelerDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketTravelerData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketTravelerData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->gender; break;
        case 1: *reinterpret_cast<QDate*>(_v) = _t->birthDate; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->gender, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->birthDate, *reinterpret_cast<QDate*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary25VdvTicketValidityAreaDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketValidityAreaData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary25VdvTicketValidityAreaDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketValidityAreaData",
        "type",
        "orgId"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'orgId'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketValidityAreaData, qt_meta_tag_ZN10KItinerary25VdvTicketValidityAreaDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketValidityAreaData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary25VdvTicketValidityAreaDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary25VdvTicketValidityAreaDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary25VdvTicketValidityAreaDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketValidityAreaData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketValidityAreaData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->type; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->orgId; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->type, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->orgId, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary31VdvTicketValidityAreaDataType31E_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketValidityAreaDataType31::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary31VdvTicketValidityAreaDataType31E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketValidityAreaDataType31",
        "startId",
        "destinationId",
        "wayTextId",
        "ticketRelation",
        "pointCloudId"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'startId'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'destinationId'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'wayTextId'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'ticketRelation'
        QtMocHelpers::PropertyData<uint>(4, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'pointCloudId'
        QtMocHelpers::PropertyData<uint>(5, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketValidityAreaDataType31, qt_meta_tag_ZN10KItinerary31VdvTicketValidityAreaDataType31E_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketValidityAreaDataType31::staticMetaObject = { {
    QtPrivate::MetaObjectForType<VdvTicketValidityAreaData>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary31VdvTicketValidityAreaDataType31E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary31VdvTicketValidityAreaDataType31E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary31VdvTicketValidityAreaDataType31E_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketValidityAreaDataType31::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketValidityAreaDataType31 *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->startId; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->destinationId; break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->wayTextId; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->ticketRelation; break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->pointCloudId; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->startId, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->destinationId, *reinterpret_cast<uint*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->wayTextId, *reinterpret_cast<uint*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->ticketRelation, *reinterpret_cast<uint*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->pointCloudId, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary30VdvTicketCommonTransactionDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketCommonTransactionData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary30VdvTicketCommonTransactionDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketCommonTransactionData",
        "kvpOrgId",
        "terminalTypeCode",
        "terminalNumber",
        "terminalOrganizationNumber",
        "transactionDateTime",
        "locationTypeCode",
        "locationNumber",
        "locationOrganizationNumber"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'kvpOrgId'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'terminalTypeCode'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'terminalNumber'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'terminalOrganizationNumber'
        QtMocHelpers::PropertyData<uint>(4, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'transactionDateTime'
        QtMocHelpers::PropertyData<QDateTime>(5, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'locationTypeCode'
        QtMocHelpers::PropertyData<uint>(6, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'locationNumber'
        QtMocHelpers::PropertyData<uint>(7, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'locationOrganizationNumber'
        QtMocHelpers::PropertyData<uint>(8, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketCommonTransactionData, qt_meta_tag_ZN10KItinerary30VdvTicketCommonTransactionDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketCommonTransactionData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary30VdvTicketCommonTransactionDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary30VdvTicketCommonTransactionDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary30VdvTicketCommonTransactionDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketCommonTransactionData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketCommonTransactionData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->kvpOrgId; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->terminalTypeCode; break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->terminalNumber; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->terminalOrganizationNumber; break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->transactionDateTime; break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->locationTypeCode; break;
        case 6: *reinterpret_cast<uint*>(_v) = _t->locationNumber; break;
        case 7: *reinterpret_cast<uint*>(_v) = _t->locationOrganizationNumber; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->kvpOrgId, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->terminalTypeCode, *reinterpret_cast<uint*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->terminalNumber, *reinterpret_cast<uint*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->terminalOrganizationNumber, *reinterpret_cast<uint*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->transactionDateTime, *reinterpret_cast<QDateTime*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->locationTypeCode, *reinterpret_cast<uint*>(_v)); break;
        case 6: QtMocHelpers::setProperty(_t->locationNumber, *reinterpret_cast<uint*>(_v)); break;
        case 7: QtMocHelpers::setProperty(_t->locationOrganizationNumber, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary18VdvTicketIssueDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketIssueData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary18VdvTicketIssueDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketIssueData",
        "samSeq1",
        "version",
        "samSeq2",
        "samId"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'samSeq1'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'version'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'samSeq2'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'samId'
        QtMocHelpers::PropertyData<uint>(4, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketIssueData, qt_meta_tag_ZN10KItinerary18VdvTicketIssueDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketIssueData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary18VdvTicketIssueDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary18VdvTicketIssueDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary18VdvTicketIssueDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketIssueData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketIssueData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->samSeq1; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->version; break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->samSeq2; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->samId; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->samSeq1, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->version, *reinterpret_cast<uint*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->samSeq2, *reinterpret_cast<uint*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->samId, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary16VdvTicketTrailerE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicketTrailer::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary16VdvTicketTrailerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicketTrailer",
        "version"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'version'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VdvTicketTrailer, qt_meta_tag_ZN10KItinerary16VdvTicketTrailerE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicketTrailer::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16VdvTicketTrailerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary16VdvTicketTrailerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary16VdvTicketTrailerE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicketTrailer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicketTrailer *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->version; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->version, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
