/****************************************************************************
** Meta object code from reading C++ file 'bustrip.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/bustrip.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bustrip.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary7BusTripE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::BusTrip::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary7BusTripE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::BusTrip",
        "className",
        "arrivalPlatform",
        "arrivalBusStop",
        "KItinerary::BusStation",
        "arrivalTime",
        "departurePlatform",
        "departureBusStop",
        "departureTime",
        "busName",
        "busNumber",
        "provider",
        "KItinerary::Organization",
        "departureDay"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'arrivalPlatform'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'arrivalBusStop'
        QtMocHelpers::PropertyData<KItinerary::BusStation>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'arrivalTime'
        QtMocHelpers::PropertyData<QDateTime>(5, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'departurePlatform'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'departureBusStop'
        QtMocHelpers::PropertyData<KItinerary::BusStation>(7, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'departureTime'
        QtMocHelpers::PropertyData<QDateTime>(8, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'busName'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'busNumber'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'provider'
        QtMocHelpers::PropertyData<KItinerary::Organization>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'departureDay'
        QtMocHelpers::PropertyData<QDate>(13, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BusTrip, qt_meta_tag_ZN10KItinerary7BusTripE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::BusTrip::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7BusTripE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7BusTripE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary7BusTripE_t>.metaTypes,
    nullptr
} };

void KItinerary::BusTrip::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<BusTrip *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::BusStation >(); break;
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Organization >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->arrivalPlatform(); break;
        case 2: *reinterpret_cast<KItinerary::BusStation*>(_v) = _t->arrivalBusStop(); break;
        case 3: *reinterpret_cast<QDateTime*>(_v) = _t->arrivalTime(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->departurePlatform(); break;
        case 5: *reinterpret_cast<KItinerary::BusStation*>(_v) = _t->departureBusStop(); break;
        case 6: *reinterpret_cast<QDateTime*>(_v) = _t->departureTime(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->busName(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->busNumber(); break;
        case 9: *reinterpret_cast<KItinerary::Organization*>(_v) = _t->provider(); break;
        case 10: *reinterpret_cast<QDate*>(_v) = _t->departureDay(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setArrivalPlatform(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setArrivalBusStop(*reinterpret_cast<KItinerary::BusStation*>(_v)); break;
        case 3: _t->setArrivalTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 4: _t->setDeparturePlatform(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setDepartureBusStop(*reinterpret_cast<KItinerary::BusStation*>(_v)); break;
        case 6: _t->setDepartureTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 7: _t->setBusName(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setBusNumber(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setProvider(*reinterpret_cast<KItinerary::Organization*>(_v)); break;
        case 10: _t->setDepartureDay(*reinterpret_cast<QDate*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
