/****************************************************************************
** Meta object code from reading C++ file 'organization.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/organization.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'organization.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary12OrganizationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Organization::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary12OrganizationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Organization",
        "className",
        "name",
        "identifier",
        "description",
        "image",
        "QUrl",
        "logo",
        "email",
        "telephone",
        "url",
        "address",
        "KItinerary::PostalAddress",
        "geo",
        "KItinerary::GeoCoordinates",
        "potentialAction",
        "QVariantList",
        "geoUri"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'identifier'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'image'
        QtMocHelpers::PropertyData<QUrl>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'logo'
        QtMocHelpers::PropertyData<QUrl>(7, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'email'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'telephone'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(10, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'address'
        QtMocHelpers::PropertyData<KItinerary::PostalAddress>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'geo'
        QtMocHelpers::PropertyData<KItinerary::GeoCoordinates>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'potentialAction'
        QtMocHelpers::PropertyData<QVariantList>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'geoUri'
        QtMocHelpers::PropertyData<QUrl>(17, 0x80000000 | 6, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Organization, qt_meta_tag_ZN10KItinerary12OrganizationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Organization::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12OrganizationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary12OrganizationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary12OrganizationE_t>.metaTypes,
    nullptr
} };

void KItinerary::Organization::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Organization *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::GeoCoordinates >(); break;
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::PostalAddress >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->identifier(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 4: *reinterpret_cast<QUrl*>(_v) = _t->image(); break;
        case 5: *reinterpret_cast<QUrl*>(_v) = _t->logo(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->email(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->telephone(); break;
        case 8: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 9: *reinterpret_cast<KItinerary::PostalAddress*>(_v) = _t->address(); break;
        case 10: *reinterpret_cast<KItinerary::GeoCoordinates*>(_v) = _t->geo(); break;
        case 11: *reinterpret_cast<QVariantList*>(_v) = _t->potentialAction(); break;
        case 12: *reinterpret_cast<QUrl*>(_v) = _t->geoUri(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setIdentifier(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setDescription(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setImage(*reinterpret_cast<QUrl*>(_v)); break;
        case 5: _t->setLogo(*reinterpret_cast<QUrl*>(_v)); break;
        case 6: _t->setEmail(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setTelephone(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 9: _t->setAddress(*reinterpret_cast<KItinerary::PostalAddress*>(_v)); break;
        case 10: _t->setGeo(*reinterpret_cast<KItinerary::GeoCoordinates*>(_v)); break;
        case 11: _t->setPotentialAction(*reinterpret_cast<QVariantList*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary7AirlineE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Airline::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary7AirlineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Airline",
        "className",
        "iataCode"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'iataCode'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Airline, qt_meta_tag_ZN10KItinerary7AirlineE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Airline::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Organization>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7AirlineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7AirlineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary7AirlineE_t>.metaTypes,
    nullptr
} };

void KItinerary::Airline::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Airline *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->iataCode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setIataCode(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary13LocalBusinessE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::LocalBusiness::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary13LocalBusinessE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::LocalBusiness",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LocalBusiness, qt_meta_tag_ZN10KItinerary13LocalBusinessE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::LocalBusiness::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Organization>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13LocalBusinessE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary13LocalBusinessE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary13LocalBusinessE_t>.metaTypes,
    nullptr
} };

void KItinerary::LocalBusiness::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<LocalBusiness *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary15LodgingBusinessE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::LodgingBusiness::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary15LodgingBusinessE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::LodgingBusiness",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LodgingBusiness, qt_meta_tag_ZN10KItinerary15LodgingBusinessE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::LodgingBusiness::staticMetaObject = { {
    QtPrivate::MetaObjectForType<LocalBusiness>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15LodgingBusinessE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary15LodgingBusinessE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary15LodgingBusinessE_t>.metaTypes,
    nullptr
} };

void KItinerary::LodgingBusiness::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<LodgingBusiness *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary17FoodEstablishmentE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::FoodEstablishment::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary17FoodEstablishmentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::FoodEstablishment",
        "className"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FoodEstablishment, qt_meta_tag_ZN10KItinerary17FoodEstablishmentE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::FoodEstablishment::staticMetaObject = { {
    QtPrivate::MetaObjectForType<LocalBusiness>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17FoodEstablishmentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary17FoodEstablishmentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary17FoodEstablishmentE_t>.metaTypes,
    nullptr
} };

void KItinerary::FoodEstablishment::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<FoodEstablishment *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
