// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "validator-logging.h"

 namespace KItinerary {
Q_LOGGING_CATEGORY(ValidatorLog, "org.kde.kitinerary.extractorValidator", QtInfoMsg)
} 
