/****************************************************************************
** Meta object code from reading C++ file 'protobufstreamreader.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/protobuf/protobufstreamreader.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'protobufstreamreader.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary20ProtobufStreamReaderE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::ProtobufStreamReader::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary20ProtobufStreamReaderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::ProtobufStreamReader",
        "fieldNumber",
        "",
        "wireType",
        "WireType",
        "readVarintField",
        "readString",
        "readSubMessage",
        "atEnd",
        "skip",
        "VARINT",
        "I64",
        "LEN",
        "SGROUP",
        "EGROUP",
        "I32"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'fieldNumber'
        QtMocHelpers::MethodData<quint64()>(1, 2, QMC::AccessPublic, QMetaType::ULongLong),
        // Method 'wireType'
        QtMocHelpers::MethodData<enum WireType()>(3, 2, QMC::AccessPublic, 0x80000000 | 4),
        // Method 'readVarintField'
        QtMocHelpers::MethodData<quint64()>(5, 2, QMC::AccessPublic, QMetaType::ULongLong),
        // Method 'readString'
        QtMocHelpers::MethodData<QString()>(6, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'readSubMessage'
        QtMocHelpers::MethodData<KItinerary::ProtobufStreamReader()>(7, 2, QMC::AccessPublic, 0x80000000 | 0),
        // Method 'atEnd'
        QtMocHelpers::MethodData<bool() const>(8, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'skip'
        QtMocHelpers::MethodData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'WireType'
        QtMocHelpers::EnumData<enum WireType>(4, 4, QMC::EnumFlags{}).add({
            {   10, WireType::VARINT },
            {   11, WireType::I64 },
            {   12, WireType::LEN },
            {   13, WireType::SGROUP },
            {   14, WireType::EGROUP },
            {   15, WireType::I32 },
        }),
    };
    return QtMocHelpers::metaObjectData<ProtobufStreamReader, qt_meta_tag_ZN10KItinerary20ProtobufStreamReaderE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::ProtobufStreamReader::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary20ProtobufStreamReaderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary20ProtobufStreamReaderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary20ProtobufStreamReaderE_t>.metaTypes,
    nullptr
} };

void KItinerary::ProtobufStreamReader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ProtobufStreamReader *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { quint64 _r = _t->fieldNumber();
            if (_a[0]) *reinterpret_cast<quint64*>(_a[0]) = std::move(_r); }  break;
        case 1: { enum WireType _r = _t->wireType();
            if (_a[0]) *reinterpret_cast<enum WireType*>(_a[0]) = std::move(_r); }  break;
        case 2: { quint64 _r = _t->readVarintField();
            if (_a[0]) *reinterpret_cast<quint64*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->readString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { KItinerary::ProtobufStreamReader _r = _t->readSubMessage();
            if (_a[0]) *reinterpret_cast<KItinerary::ProtobufStreamReader*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->atEnd();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->skip(); break;
        default: ;
        }
    }
}
QT_WARNING_POP
