/****************************************************************************
** Meta object code from reading C++ file 'rentalcar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/datatypes/rentalcar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rentalcar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary9RentalCarE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::RentalCar::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary9RentalCarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::RentalCar",
        "className",
        "name",
        "model",
        "rentalCompany",
        "KItinerary::Organization",
        "brand",
        "KItinerary::Brand"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'className'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'model'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'rentalCompany'
        QtMocHelpers::PropertyData<KItinerary::Organization>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'brand'
        QtMocHelpers::PropertyData<KItinerary::Brand>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RentalCar, qt_meta_tag_ZN10KItinerary9RentalCarE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::RentalCar::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary9RentalCarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary9RentalCarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary9RentalCarE_t>.metaTypes,
    nullptr
} };

void KItinerary::RentalCar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RentalCar *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Brand >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Organization >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->model(); break;
        case 3: *reinterpret_cast<KItinerary::Organization*>(_v) = _t->rentalCompany(); break;
        case 4: *reinterpret_cast<KItinerary::Brand*>(_v) = _t->brand(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setModel(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setRentalCompany(*reinterpret_cast<KItinerary::Organization*>(_v)); break;
        case 4: _t->setBrand(*reinterpret_cast<KItinerary::Brand*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
