/****************************************************************************
** Meta object code from reading C++ file 'vdvticket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/vdv/vdvticket.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vdvticket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary9VdvTicketE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::VdvTicket::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary9VdvTicketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::VdvTicket",
        "beginDateTime",
        "endDateTime",
        "issuerId",
        "operatorId",
        "serviceClass",
        "ServiceClass",
        "person",
        "KItinerary::Person",
        "ticketNumber",
        "rawData",
        "UnknownClass",
        "FirstClass",
        "SecondClass",
        "FirstClassUpgrade"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'beginDateTime'
        QtMocHelpers::PropertyData<QDateTime>(1, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'endDateTime'
        QtMocHelpers::PropertyData<QDateTime>(2, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'issuerId'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'operatorId'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'serviceClass'
        QtMocHelpers::PropertyData<enum ServiceClass>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'person'
        QtMocHelpers::PropertyData<KItinerary::Person>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'ticketNumber'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'rawData'
        QtMocHelpers::PropertyData<QByteArray>(10, QMetaType::QByteArray, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ServiceClass'
        QtMocHelpers::EnumData<enum ServiceClass>(6, 6, QMC::EnumFlags{}).add({
            {   11, ServiceClass::UnknownClass },
            {   12, ServiceClass::FirstClass },
            {   13, ServiceClass::SecondClass },
            {   14, ServiceClass::FirstClassUpgrade },
        }),
    };
    return QtMocHelpers::metaObjectData<VdvTicket, qt_meta_tag_ZN10KItinerary9VdvTicketE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::VdvTicket::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary9VdvTicketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary9VdvTicketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary9VdvTicketE_t>.metaTypes,
    nullptr
} };

void KItinerary::VdvTicket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VdvTicket *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KItinerary::Person >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDateTime*>(_v) = _t->beginDateTime(); break;
        case 1: *reinterpret_cast<QDateTime*>(_v) = _t->endDateTime(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->issuerId(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->operatorId(); break;
        case 4: *reinterpret_cast<enum ServiceClass*>(_v) = _t->serviceClass(); break;
        case 5: *reinterpret_cast<KItinerary::Person*>(_v) = _t->person(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->ticketNumber(); break;
        case 7: *reinterpret_cast<QByteArray*>(_v) = _t->rawData(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
