/****************************************************************************
** Meta object code from reading C++ file 'dosipas2.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/era/dosipas2.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dosipas2.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary7Dosipas2v28DataTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Dosipas::v2::DataType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary7Dosipas2v28DataTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Dosipas::v2::DataType",
        "dataFormat",
        "data",
        "content",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataFormat'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'data'
        QtMocHelpers::PropertyData<QByteArray>(2, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'content'
        QtMocHelpers::PropertyData<QVariant>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataType, qt_meta_tag_ZN10KItinerary7Dosipas2v28DataTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Dosipas::v2::DataType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v28DataTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v28DataTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary7Dosipas2v28DataTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Dosipas::v2::DataType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<DataType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->dataFormat; break;
        case 1: *reinterpret_cast<QByteArray*>(_v) = _t->data; break;
        case 2: *reinterpret_cast<QVariant*>(_v) = _t->content(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary7Dosipas2v214Level1DataTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Dosipas::v2::Level1DataType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level1DataTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Dosipas::v2::Level1DataType",
        "securityProviderNum",
        "securityProviderNumIsSet",
        "securityProviderIA5",
        "securityProviderIA5IsSet",
        "keyId",
        "keyIdIsSet",
        "dataSequence",
        "QList<DataType>",
        "level1KeyAlg",
        "level1KeyAlgIsSet",
        "level2KeyAlg",
        "level2KeyAlgIsSet",
        "level1SigningAlg",
        "level1SigningAlgIsSet",
        "level2SigningAlg",
        "level2SigningAlgIsSet",
        "level2PublicKey",
        "level2PublicKeyIsSet",
        "endOfValidityYear",
        "endOfValidityYearIsSet",
        "endOfValidityDay",
        "endOfValidityDayIsSet",
        "endOfValidityTime",
        "endOfValidityTimeIsSet",
        "validityDuration",
        "validityDurationIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'securityProviderNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'securityProviderNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'securityProviderIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'securityProviderIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'keyId'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'keyIdIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'dataSequence'
        QtMocHelpers::PropertyData<QList<DataType>>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'level1KeyAlg'
        QtMocHelpers::PropertyData<QByteArray>(9, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level1KeyAlgIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'level2KeyAlg'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level2KeyAlgIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'level1SigningAlg'
        QtMocHelpers::PropertyData<QByteArray>(13, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level1SigningAlgIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'level2SigningAlg'
        QtMocHelpers::PropertyData<QByteArray>(15, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level2SigningAlgIsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'level2PublicKey'
        QtMocHelpers::PropertyData<QByteArray>(17, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level2PublicKeyIsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'endOfValidityYear'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'endOfValidityYearIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'endOfValidityDay'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'endOfValidityDayIsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'endOfValidityTime'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'endOfValidityTimeIsSet'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validityDuration'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validityDurationIsSet'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Level1DataType, qt_meta_tag_ZN10KItinerary7Dosipas2v214Level1DataTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Dosipas::v2::Level1DataType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level1DataTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level1DataTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level1DataTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Dosipas::v2::Level1DataType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Level1DataType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->securityProviderNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->securityProviderNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->securityProviderIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->securityProviderIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->keyId; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->keyIdIsSet(); break;
        case 6: *reinterpret_cast<QList<DataType>*>(_v) = _t->dataSequence; break;
        case 7: *reinterpret_cast<QByteArray*>(_v) = _t->level1KeyAlg; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->level1KeyAlgIsSet(); break;
        case 9: *reinterpret_cast<QByteArray*>(_v) = _t->level2KeyAlg; break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->level2KeyAlgIsSet(); break;
        case 11: *reinterpret_cast<QByteArray*>(_v) = _t->level1SigningAlg; break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->level1SigningAlgIsSet(); break;
        case 13: *reinterpret_cast<QByteArray*>(_v) = _t->level2SigningAlg; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->level2SigningAlgIsSet(); break;
        case 15: *reinterpret_cast<QByteArray*>(_v) = _t->level2PublicKey; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->level2PublicKeyIsSet(); break;
        case 17: *reinterpret_cast<int*>(_v) = _t->endOfValidityYear; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->endOfValidityYearIsSet(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->endOfValidityDay; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->endOfValidityDayIsSet(); break;
        case 21: *reinterpret_cast<int*>(_v) = _t->endOfValidityTime; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->endOfValidityTimeIsSet(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->validityDuration; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->validityDurationIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary7Dosipas2v214Level2DataTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Dosipas::v2::Level2DataType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level2DataTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Dosipas::v2::Level2DataType",
        "level1Data",
        "Level1DataType",
        "level1Signature",
        "level1SignatureIsSet",
        "level2Data",
        "DataType",
        "level2DataIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'level1Data'
        QtMocHelpers::PropertyData<Level1DataType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'level1Signature'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level1SignatureIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'level2Data'
        QtMocHelpers::PropertyData<DataType>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'level2DataIsSet'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Level2DataType, qt_meta_tag_ZN10KItinerary7Dosipas2v214Level2DataTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Dosipas::v2::Level2DataType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level2DataTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level2DataTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary7Dosipas2v214Level2DataTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Dosipas::v2::Level2DataType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Level2DataType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Level1DataType*>(_v) = _t->level1Data; break;
        case 1: *reinterpret_cast<QByteArray*>(_v) = _t->level1Signature; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->level1SignatureIsSet(); break;
        case 3: *reinterpret_cast<DataType*>(_v) = _t->level2Data; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->level2DataIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary7Dosipas2v216UicBarcodeHeaderE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Dosipas::v2::UicBarcodeHeader::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary7Dosipas2v216UicBarcodeHeaderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Dosipas::v2::UicBarcodeHeader",
        "format",
        "level2SignedData",
        "Level2DataType",
        "level2Signature",
        "level2SignatureIsSet",
        "rawData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'format'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level2SignedData'
        QtMocHelpers::PropertyData<Level2DataType>(2, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'level2Signature'
        QtMocHelpers::PropertyData<QByteArray>(4, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'level2SignatureIsSet'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'rawData'
        QtMocHelpers::PropertyData<QByteArray>(6, QMetaType::QByteArray, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UicBarcodeHeader, qt_meta_tag_ZN10KItinerary7Dosipas2v216UicBarcodeHeaderE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Dosipas::v2::UicBarcodeHeader::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v216UicBarcodeHeaderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary7Dosipas2v216UicBarcodeHeaderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary7Dosipas2v216UicBarcodeHeaderE_t>.metaTypes,
    nullptr
} };

void KItinerary::Dosipas::v2::UicBarcodeHeader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<UicBarcodeHeader *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->format; break;
        case 1: *reinterpret_cast<Level2DataType*>(_v) = _t->level2SignedData; break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->level2Signature; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->level2SignatureIsSet(); break;
        case 4: *reinterpret_cast<QByteArray*>(_v) = _t->rawData(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
