/****************************************************************************
** Meta object code from reading C++ file 'vendor0080vublockdata.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/uic9183/vendor0080vublockdata.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vendor0080vublockdata.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary22Vendor0080VUCommonDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Vendor0080VUCommonData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary22Vendor0080VUCommonDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Vendor0080VUCommonData",
        "terminalNumber",
        "samNumber",
        "numberOfPersons",
        "numberOfTickets"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'terminalNumber'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'samNumber'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'numberOfPersons'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'numberOfTickets'
        QtMocHelpers::PropertyData<uint>(4, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Vendor0080VUCommonData, qt_meta_tag_ZN10KItinerary22Vendor0080VUCommonDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Vendor0080VUCommonData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary22Vendor0080VUCommonDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary22Vendor0080VUCommonDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary22Vendor0080VUCommonDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Vendor0080VUCommonData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Vendor0080VUCommonData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->terminalNumber; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->samNumber; break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->numberOfPersons; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->numberOfTickets; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->terminalNumber, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->samNumber, *reinterpret_cast<uint*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->numberOfPersons, *reinterpret_cast<uint*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->numberOfTickets, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary22Vendor0080VUTicketDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Vendor0080VUTicketData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary22Vendor0080VUTicketDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Vendor0080VUTicketData",
        "authorizationNumber",
        "kvpOrgId",
        "productNumber",
        "pvOrgId",
        "validFrom",
        "validUntil",
        "price",
        "samSequnceNumber",
        "areaListLength"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'authorizationNumber'
        QtMocHelpers::PropertyData<uint>(1, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'kvpOrgId'
        QtMocHelpers::PropertyData<uint>(2, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'productNumber'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'pvOrgId'
        QtMocHelpers::PropertyData<uint>(4, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'validFrom'
        QtMocHelpers::PropertyData<QDateTime>(5, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'validUntil'
        QtMocHelpers::PropertyData<QDateTime>(6, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'price'
        QtMocHelpers::PropertyData<uint>(7, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'samSequnceNumber'
        QtMocHelpers::PropertyData<uint>(8, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'areaListLength'
        QtMocHelpers::PropertyData<uint>(9, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Vendor0080VUTicketData, qt_meta_tag_ZN10KItinerary22Vendor0080VUTicketDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Vendor0080VUTicketData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary22Vendor0080VUTicketDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary22Vendor0080VUTicketDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary22Vendor0080VUTicketDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Vendor0080VUTicketData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Vendor0080VUTicketData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->authorizationNumber; break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->kvpOrgId; break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->productNumber; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->pvOrgId; break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->validFrom; break;
        case 5: *reinterpret_cast<QDateTime*>(_v) = _t->validUntil; break;
        case 6: *reinterpret_cast<uint*>(_v) = _t->price; break;
        case 7: *reinterpret_cast<uint*>(_v) = _t->samSequnceNumber; break;
        case 8: *reinterpret_cast<uint*>(_v) = _t->areaListLength; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->authorizationNumber, *reinterpret_cast<uint*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->kvpOrgId, *reinterpret_cast<uint*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->productNumber, *reinterpret_cast<uint*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->pvOrgId, *reinterpret_cast<uint*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->validFrom, *reinterpret_cast<QDateTime*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->validUntil, *reinterpret_cast<QDateTime*>(_v)); break;
        case 6: QtMocHelpers::setProperty(_t->price, *reinterpret_cast<uint*>(_v)); break;
        case 7: QtMocHelpers::setProperty(_t->samSequnceNumber, *reinterpret_cast<uint*>(_v)); break;
        case 8: QtMocHelpers::setProperty(_t->areaListLength, *reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
