/****************************************************************************
** Meta object code from reading C++ file 'countrydb.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/knowledgedb/countrydb.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'countrydb.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary11KnowledgeDbE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::KnowledgeDb::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary11KnowledgeDbE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::KnowledgeDb",
        "DrivingSide",
        "Unknown",
        "Left",
        "Right",
        "PowerPlugTypes",
        "PowerPlugType",
        "TypeA",
        "TypeB",
        "TypeC",
        "TypeD",
        "TypeE",
        "TypeF",
        "TypeG",
        "TypeH",
        "TypeI",
        "TypeJ",
        "TypeK",
        "TypeL",
        "TypeM",
        "TypeN"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DrivingSide'
        QtMocHelpers::EnumData<DrivingSide>(1, 1, QMC::EnumIsScoped).add({
            {    2, DrivingSide::Unknown },
            {    3, DrivingSide::Left },
            {    4, DrivingSide::Right },
        }),
        // flag 'PowerPlugTypes'
        QtMocHelpers::EnumData<PowerPlugTypes>(5, 6, QMC::EnumIsFlag).add({
            {    2, PowerPlugType::Unknown },
            {    7, PowerPlugType::TypeA },
            {    8, PowerPlugType::TypeB },
            {    9, PowerPlugType::TypeC },
            {   10, PowerPlugType::TypeD },
            {   11, PowerPlugType::TypeE },
            {   12, PowerPlugType::TypeF },
            {   13, PowerPlugType::TypeG },
            {   14, PowerPlugType::TypeH },
            {   15, PowerPlugType::TypeI },
            {   16, PowerPlugType::TypeJ },
            {   17, PowerPlugType::TypeK },
            {   18, PowerPlugType::TypeL },
            {   19, PowerPlugType::TypeM },
            {   20, PowerPlugType::TypeN },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10KItinerary11KnowledgeDbE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10KItinerary11KnowledgeDbE =
    KItinerary::KnowledgeDb::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary11KnowledgeDbE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10KItinerary11KnowledgeDbE =
    qt_staticMetaObjectContent_ZN10KItinerary11KnowledgeDbE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10KItinerary11KnowledgeDbE =
    qt_staticMetaObjectContent_ZN10KItinerary11KnowledgeDbE.relocatingData;

Q_CONSTINIT const QMetaObject KItinerary::KnowledgeDb::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10KItinerary11KnowledgeDbE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10KItinerary11KnowledgeDbE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10KItinerary11KnowledgeDbE.metaTypes,
    nullptr
} };

QT_WARNING_POP
