/****************************************************************************
** Meta object code from reading C++ file 'fcbticket1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kitinerary-26.04.3/src/lib/era/fcbticket1.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fcbticket1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1313ExtensionDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ExtensionData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1313ExtensionDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionId",
        "extensionData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'extensionId'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'extensionData'
        QtMocHelpers::PropertyData<QByteArray>(2, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExtensionData, qt_meta_tag_ZN10KItinerary3Fcb3v1313ExtensionDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ExtensionData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313ExtensionDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313ExtensionDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313ExtensionDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ExtensionData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ExtensionData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->extensionId; break;
        case 1: *reinterpret_cast<QByteArray*>(_v) = _t->extensionData; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::GeoCoordinateType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::GeoCoordinateType",
        "geoUnit",
        "KItinerary::Fcb::v13::GeoUnitType",
        "coordinateSystem",
        "KItinerary::Fcb::v13::GeoCoordinateSystemType",
        "hemisphereLongitude",
        "KItinerary::Fcb::v13::HemisphereLongitudeType",
        "hemisphereLatitude",
        "KItinerary::Fcb::v13::HemisphereLatitudeType",
        "longitude",
        "latitude",
        "accuracy",
        "accuracyIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'geoUnit'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::GeoUnitType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'coordinateSystem'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::GeoCoordinateSystemType>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'hemisphereLongitude'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::HemisphereLongitudeType>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'hemisphereLatitude'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::HemisphereLatitudeType>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'longitude'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'latitude'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'accuracy'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::GeoUnitType>(11, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'accuracyIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GeoCoordinateType, qt_meta_tag_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::GeoCoordinateType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317GeoCoordinateTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::GeoCoordinateType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<GeoCoordinateType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::GeoUnitType*>(_v) = _t->geoUnit; break;
        case 1: *reinterpret_cast<KItinerary::Fcb::v13::GeoCoordinateSystemType*>(_v) = _t->coordinateSystem; break;
        case 2: *reinterpret_cast<KItinerary::Fcb::v13::HemisphereLongitudeType*>(_v) = _t->hemisphereLongitude; break;
        case 3: *reinterpret_cast<KItinerary::Fcb::v13::HemisphereLatitudeType*>(_v) = _t->hemisphereLatitude; break;
        case 4: *reinterpret_cast<int*>(_v) = _t->longitude; break;
        case 5: *reinterpret_cast<int*>(_v) = _t->latitude; break;
        case 6: *reinterpret_cast<KItinerary::Fcb::v13::GeoUnitType*>(_v) = _t->accuracy; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->accuracyIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1315DeltaCoordinateE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::DeltaCoordinate::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1315DeltaCoordinateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::DeltaCoordinate",
        "longitude",
        "latitude"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'longitude'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'latitude'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DeltaCoordinate, qt_meta_tag_ZN10KItinerary3Fcb3v1315DeltaCoordinateE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::DeltaCoordinate::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315DeltaCoordinateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315DeltaCoordinateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315DeltaCoordinateE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::DeltaCoordinate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<DeltaCoordinate *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->longitude; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->latitude; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1311IssuingDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::IssuingData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1311IssuingDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::IssuingData",
        "securityProviderNum",
        "securityProviderNumIsSet",
        "securityProviderIA5",
        "securityProviderIA5IsSet",
        "issuerNum",
        "issuerNumIsSet",
        "issuerIA5",
        "issuerIA5IsSet",
        "issuingYear",
        "issuingDay",
        "issuingTime",
        "issuingTimeIsSet",
        "issuerName",
        "issuerNameIsSet",
        "specimen",
        "securePaperTicket",
        "activated",
        "currency",
        "currencyFract",
        "issuerPNR",
        "issuerPNRIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet",
        "issuedOnTrainNum",
        "issuedOnTrainNumIsSet",
        "issuedOnTrainIA5",
        "issuedOnTrainIA5IsSet",
        "issuedOnLine",
        "issuedOnLineIsSet",
        "pointOfSale",
        "KItinerary::Fcb::v13::GeoCoordinateType",
        "pointOfSaleIsSet",
        "issueingDateTime"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'securityProviderNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'securityProviderNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'securityProviderIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'securityProviderIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuerNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuerIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuingYear'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuingDay'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuingTime'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuingTimeIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuerName'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerNameIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'specimen'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'securePaperTicket'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'activated'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'currency'
        QtMocHelpers::PropertyData<QByteArray>(18, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'currencyFract'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerPNR'
        QtMocHelpers::PropertyData<QByteArray>(20, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerPNRIsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuedOnTrainNum'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuedOnTrainNumIsSet'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuedOnTrainIA5'
        QtMocHelpers::PropertyData<QByteArray>(27, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuedOnTrainIA5IsSet'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuedOnLine'
        QtMocHelpers::PropertyData<int>(29, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuedOnLineIsSet'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'pointOfSale'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::GeoCoordinateType>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'pointOfSaleIsSet'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issueingDateTime'
        QtMocHelpers::PropertyData<QDateTime>(34, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IssuingData, qt_meta_tag_ZN10KItinerary3Fcb3v1311IssuingDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1311IssuingDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::IssuingData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311IssuingDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311IssuingDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1311IssuingDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311IssuingDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::IssuingData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IssuingData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->securityProviderNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->securityProviderNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->securityProviderIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->securityProviderIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->issuerNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->issuerNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->issuerIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->issuerIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->issuingYear; break;
        case 9: *reinterpret_cast<int*>(_v) = _t->issuingDay; break;
        case 10: *reinterpret_cast<int*>(_v) = _t->issuingTime; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->issuingTimeIsSet(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->issuerName; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->issuerNameIsSet(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->specimen; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->securePaperTicket; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->activated; break;
        case 17: *reinterpret_cast<QByteArray*>(_v) = _t->currency; break;
        case 18: *reinterpret_cast<int*>(_v) = _t->currencyFract; break;
        case 19: *reinterpret_cast<QByteArray*>(_v) = _t->issuerPNR; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->issuerPNRIsSet(); break;
        case 21: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->issuedOnTrainNum; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->issuedOnTrainNumIsSet(); break;
        case 25: *reinterpret_cast<QByteArray*>(_v) = _t->issuedOnTrainIA5; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->issuedOnTrainIA5IsSet(); break;
        case 27: *reinterpret_cast<int*>(_v) = _t->issuedOnLine; break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->issuedOnLineIsSet(); break;
        case 29: *reinterpret_cast<KItinerary::Fcb::v13::GeoCoordinateType*>(_v) = _t->pointOfSale; break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->pointOfSaleIsSet(); break;
        case 31: *reinterpret_cast<QDateTime*>(_v) = _t->issueingDateTime(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1318CustomerStatusTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::CustomerStatusType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1318CustomerStatusTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::CustomerStatusType",
        "statusProviderNum",
        "statusProviderNumIsSet",
        "statusProviderIA5",
        "statusProviderIA5IsSet",
        "customerStatus",
        "customerStatusIsSet",
        "customerStatusDescr",
        "customerStatusDescrIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'statusProviderNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'statusProviderNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'statusProviderIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'statusProviderIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'customerStatus'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'customerStatusIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'customerStatusDescr'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'customerStatusDescrIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomerStatusType, qt_meta_tag_ZN10KItinerary3Fcb3v1318CustomerStatusTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::CustomerStatusType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318CustomerStatusTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318CustomerStatusTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318CustomerStatusTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::CustomerStatusType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CustomerStatusType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->statusProviderNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->statusProviderNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->statusProviderIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->statusProviderIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->customerStatus; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->customerStatusIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->customerStatusDescr; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->customerStatusDescrIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::TravelerType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::TravelerType",
        "firstName",
        "firstNameIsSet",
        "secondName",
        "secondNameIsSet",
        "lastName",
        "lastNameIsSet",
        "idCard",
        "idCardIsSet",
        "passportId",
        "passportIdIsSet",
        "title",
        "titleIsSet",
        "gender",
        "KItinerary::Fcb::v13::GenderType",
        "genderIsSet",
        "customerIdIA5",
        "customerIdIA5IsSet",
        "customerIdNum",
        "customerIdNumIsSet",
        "yearOfBirth",
        "yearOfBirthIsSet",
        "dayOfBirth",
        "dayOfBirthIsSet",
        "ticketHolder",
        "passengerType",
        "KItinerary::Fcb::v13::PassengerType",
        "passengerTypeIsSet",
        "passengerWithReducedMobility",
        "passengerWithReducedMobilityIsSet",
        "countryOfResidence",
        "countryOfResidenceIsSet",
        "countryOfPassport",
        "countryOfPassportIsSet",
        "countryOfIdCard",
        "countryOfIdCardIsSet",
        "status",
        "QList<KItinerary::Fcb::v13::CustomerStatusType>",
        "statusIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'firstName'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'firstNameIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'secondName'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'secondNameIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'lastName'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'lastNameIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'idCard'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'idCardIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'passportId'
        QtMocHelpers::PropertyData<QByteArray>(9, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'passportIdIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'title'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'titleIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'gender'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::GenderType>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'genderIsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'customerIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(16, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'customerIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'customerIdNum'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'customerIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'yearOfBirth'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'yearOfBirthIsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'dayOfBirth'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dayOfBirthIsSet'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ticketHolder'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'passengerType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::PassengerType>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'passengerTypeIsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'passengerWithReducedMobility'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'passengerWithReducedMobilityIsSet'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'countryOfResidence'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'countryOfResidenceIsSet'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'countryOfPassport'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'countryOfPassportIsSet'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'countryOfIdCard'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'countryOfIdCardIsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'status'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::CustomerStatusType>>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'statusIsSet'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TravelerType, qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1312TravelerTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::TravelerType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1312TravelerTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::TravelerType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TravelerType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->firstName; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->firstNameIsSet(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->secondName; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->secondNameIsSet(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->lastName; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->lastNameIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->idCard; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->idCardIsSet(); break;
        case 8: *reinterpret_cast<QByteArray*>(_v) = _t->passportId; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->passportIdIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->title; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->titleIsSet(); break;
        case 12: *reinterpret_cast<KItinerary::Fcb::v13::GenderType*>(_v) = _t->gender; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->genderIsSet(); break;
        case 14: *reinterpret_cast<QByteArray*>(_v) = _t->customerIdIA5; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->customerIdIA5IsSet(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->customerIdNum; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->customerIdNumIsSet(); break;
        case 18: *reinterpret_cast<int*>(_v) = _t->yearOfBirth; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->yearOfBirthIsSet(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->dayOfBirth; break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->dayOfBirthIsSet(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->ticketHolder; break;
        case 23: *reinterpret_cast<KItinerary::Fcb::v13::PassengerType*>(_v) = _t->passengerType; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->passengerTypeIsSet(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->passengerWithReducedMobility; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->passengerWithReducedMobilityIsSet(); break;
        case 27: *reinterpret_cast<int*>(_v) = _t->countryOfResidence; break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->countryOfResidenceIsSet(); break;
        case 29: *reinterpret_cast<int*>(_v) = _t->countryOfPassport; break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->countryOfPassportIsSet(); break;
        case 31: *reinterpret_cast<int*>(_v) = _t->countryOfIdCard; break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->countryOfIdCardIsSet(); break;
        case 33: *reinterpret_cast<QList<KItinerary::Fcb::v13::CustomerStatusType>*>(_v) = _t->status; break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->statusIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::TravelerData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::TravelerData",
        "traveler",
        "QList<KItinerary::Fcb::v13::TravelerType>",
        "travelerIsSet",
        "preferredLanguage",
        "preferredLanguageIsSet",
        "groupName",
        "groupNameIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'traveler'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TravelerType>>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'travelerIsSet'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'preferredLanguage'
        QtMocHelpers::PropertyData<QByteArray>(4, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'preferredLanguageIsSet'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'groupName'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'groupNameIsSet'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TravelerData, qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::TravelerData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312TravelerDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::TravelerData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TravelerData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<KItinerary::Fcb::v13::TravelerType>*>(_v) = _t->traveler; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->travelerIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->preferredLanguage; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->preferredLanguageIsSet(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->groupName; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->groupNameIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1313TrainLinkTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::TrainLinkType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1313TrainLinkTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::TrainLinkType",
        "departureDateTime",
        "",
        "issueingDateTime",
        "trainNum",
        "trainNumIsSet",
        "trainIA5",
        "trainIA5IsSet",
        "travelDate",
        "departureTime",
        "departureUTCOffset",
        "departureUTCOffsetIsSet",
        "fromStationNum",
        "fromStationNumIsSet",
        "fromStationIA5",
        "fromStationIA5IsSet",
        "toStationNum",
        "toStationNumIsSet",
        "toStationIA5",
        "toStationIA5IsSet",
        "fromStationNameUTF8",
        "fromStationNameUTF8IsSet",
        "toStationNameUTF8",
        "toStationNameUTF8IsSet"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'departureDateTime'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'trainNum'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainNumIsSet'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trainIA5'
        QtMocHelpers::PropertyData<QByteArray>(6, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainIA5IsSet'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'travelDate'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureTime'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureUTCOffset'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNum'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(14, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNum'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(18, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TrainLinkType, qt_meta_tag_ZN10KItinerary3Fcb3v1313TrainLinkTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::TrainLinkType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313TrainLinkTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313TrainLinkTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313TrainLinkTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::TrainLinkType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TrainLinkType *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDateTime _r = _t->departureDateTime((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->trainNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->trainNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->trainIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->trainIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->travelDate; break;
        case 5: *reinterpret_cast<int*>(_v) = _t->departureTime; break;
        case 6: *reinterpret_cast<int*>(_v) = _t->departureUTCOffset; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->departureUTCOffsetIsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->fromStationNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->fromStationNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->fromStationIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->fromStationIA5IsSet(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->toStationNum; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->toStationNumIsSet(); break;
        case 14: *reinterpret_cast<QByteArray*>(_v) = _t->toStationIA5; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->toStationIA5IsSet(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->fromStationNameUTF8; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->fromStationNameUTF8IsSet(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->toStationNameUTF8; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->toStationNameUTF8IsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1314ViaStationTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ViaStationType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1314ViaStationTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ViaStationType",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "stationNum",
        "stationNumIsSet",
        "stationIA5",
        "stationIA5IsSet",
        "alternativeRoutes",
        "QList<KItinerary::Fcb::v13::ViaStationType>",
        "alternativeRoutesIsSet",
        "route",
        "routeIsSet",
        "border",
        "carrierNum",
        "QList<int>",
        "carrierNumIsSet",
        "carrierIA5",
        "QList<QByteArray>",
        "carrierIA5IsSet",
        "seriesId",
        "seriesIdIsSet",
        "routeId",
        "routeIdIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stationNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationIA5'
        QtMocHelpers::PropertyData<QByteArray>(5, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'alternativeRoutes'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::ViaStationType>>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'alternativeRoutesIsSet'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'route'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::ViaStationType>>(10, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'routeIsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'border'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'carrierNum'
        QtMocHelpers::PropertyData<QList<int>>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'seriesId'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'seriesIdIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'routeId'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'routeIdIsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViaStationType, qt_meta_tag_ZN10KItinerary3Fcb3v1314ViaStationTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1314ViaStationTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ViaStationType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314ViaStationTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314ViaStationTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1314ViaStationTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314ViaStationTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ViaStationType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ViaStationType *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 12:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->stationNum; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->stationNumIsSet(); break;
        case 3: *reinterpret_cast<QByteArray*>(_v) = _t->stationIA5; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->stationIA5IsSet(); break;
        case 5: *reinterpret_cast<QList<KItinerary::Fcb::v13::ViaStationType>*>(_v) = _t->alternativeRoutes; break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->alternativeRoutesIsSet(); break;
        case 7: *reinterpret_cast<QList<KItinerary::Fcb::v13::ViaStationType>*>(_v) = _t->route; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->routeIsSet(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->border; break;
        case 10: *reinterpret_cast<QList<int>*>(_v) = _t->carrierNum; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 12: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->carrierIA5; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->seriesId; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->seriesIdIsSet(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->routeId; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->routeIdIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v138ZoneTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ZoneType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v138ZoneTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ZoneType",
        "carrierNum",
        "carrierNumIsSet",
        "carrierIA5",
        "carrierIA5IsSet",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "entryStationNum",
        "entryStationNumIsSet",
        "entryStationIA5",
        "entryStationIA5IsSet",
        "terminatingStationNum",
        "terminatingStationNumIsSet",
        "terminatingStationIA5",
        "terminatingStationIA5IsSet",
        "city",
        "cityIsSet",
        "zoneId",
        "QList<int>",
        "zoneIdIsSet",
        "binaryZoneId",
        "binaryZoneIdIsSet",
        "nutsCode",
        "nutsCodeIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'carrierNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'entryStationNum'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'entryStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'entryStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(9, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'entryStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'terminatingStationNum'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'terminatingStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'terminatingStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(13, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'terminatingStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'city'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cityIsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'zoneId'
        QtMocHelpers::PropertyData<QList<int>>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'zoneIdIsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'binaryZoneId'
        QtMocHelpers::PropertyData<QByteArray>(20, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'binaryZoneIdIsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'nutsCode'
        QtMocHelpers::PropertyData<QByteArray>(22, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'nutsCodeIsSet'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ZoneType, qt_meta_tag_ZN10KItinerary3Fcb3v138ZoneTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v138ZoneTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ZoneType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v138ZoneTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v138ZoneTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v138ZoneTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v138ZoneTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ZoneType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ZoneType *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->carrierNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->carrierIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 4: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 5: *reinterpret_cast<int*>(_v) = _t->entryStationNum; break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->entryStationNumIsSet(); break;
        case 7: *reinterpret_cast<QByteArray*>(_v) = _t->entryStationIA5; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->entryStationIA5IsSet(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->terminatingStationNum; break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->terminatingStationNumIsSet(); break;
        case 11: *reinterpret_cast<QByteArray*>(_v) = _t->terminatingStationIA5; break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->terminatingStationIA5IsSet(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->city; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->cityIsSet(); break;
        case 15: *reinterpret_cast<QList<int>*>(_v) = _t->zoneId; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->zoneIdIsSet(); break;
        case 17: *reinterpret_cast<QByteArray*>(_v) = _t->binaryZoneId; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->binaryZoneIdIsSet(); break;
        case 19: *reinterpret_cast<QByteArray*>(_v) = _t->nutsCode; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->nutsCodeIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v138LineTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::LineType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v138LineTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::LineType",
        "carrierNum",
        "carrierNumIsSet",
        "carrierIA5",
        "carrierIA5IsSet",
        "lineId",
        "QList<int>",
        "lineIdIsSet",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "entryStationNum",
        "entryStationNumIsSet",
        "entryStationIA5",
        "entryStationIA5IsSet",
        "terminatingStationNum",
        "terminatingStationNumIsSet",
        "terminatingStationIA5",
        "terminatingStationIA5IsSet",
        "city",
        "cityIsSet",
        "binaryZoneId",
        "binaryZoneIdIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'carrierNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'lineId'
        QtMocHelpers::PropertyData<QList<int>>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'lineIdIsSet'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'entryStationNum'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'entryStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'entryStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(12, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'entryStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'terminatingStationNum'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'terminatingStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'terminatingStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(16, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'terminatingStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'city'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cityIsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'binaryZoneId'
        QtMocHelpers::PropertyData<QByteArray>(20, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'binaryZoneIdIsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LineType, qt_meta_tag_ZN10KItinerary3Fcb3v138LineTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v138LineTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::LineType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v138LineTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v138LineTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v138LineTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v138LineTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::LineType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<LineType *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->carrierNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->carrierIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 4: *reinterpret_cast<QList<int>*>(_v) = _t->lineId; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->lineIdIsSet(); break;
        case 6: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 7: *reinterpret_cast<int*>(_v) = _t->entryStationNum; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->entryStationNumIsSet(); break;
        case 9: *reinterpret_cast<QByteArray*>(_v) = _t->entryStationIA5; break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->entryStationIA5IsSet(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->terminatingStationNum; break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->terminatingStationNumIsSet(); break;
        case 13: *reinterpret_cast<QByteArray*>(_v) = _t->terminatingStationIA5; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->terminatingStationIA5IsSet(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->city; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->cityIsSet(); break;
        case 17: *reinterpret_cast<QByteArray*>(_v) = _t->binaryZoneId; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->binaryZoneIdIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1312PolygoneTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::PolygoneType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1312PolygoneTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::PolygoneType",
        "firstEdge",
        "KItinerary::Fcb::v13::GeoCoordinateType",
        "edges",
        "QList<KItinerary::Fcb::v13::DeltaCoordinate>"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'firstEdge'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::GeoCoordinateType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'edges'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::DeltaCoordinate>>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PolygoneType, qt_meta_tag_ZN10KItinerary3Fcb3v1312PolygoneTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1312PolygoneTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::PolygoneType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312PolygoneTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312PolygoneTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1312PolygoneTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312PolygoneTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::PolygoneType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<PolygoneType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::GeoCoordinateType*>(_v) = _t->firstEdge; break;
        case 1: *reinterpret_cast<QList<KItinerary::Fcb::v13::DeltaCoordinate>*>(_v) = _t->edges; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1320RegionalValidityTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::RegionalValidityType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1320RegionalValidityTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::RegionalValidityType",
        "value",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'value'
        QtMocHelpers::PropertyData<QVariant>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RegionalValidityType, qt_meta_tag_ZN10KItinerary3Fcb3v1320RegionalValidityTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::RegionalValidityType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1320RegionalValidityTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1320RegionalValidityTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1320RegionalValidityTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::RegionalValidityType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RegionalValidityType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariant*>(_v) = _t->valueVariant(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1326ReturnRouteDescriptionTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ReturnRouteDescriptionType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1326ReturnRouteDescriptionTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ReturnRouteDescriptionType",
        "fromStationNum",
        "fromStationNumIsSet",
        "fromStationIA5",
        "fromStationIA5IsSet",
        "toStationNum",
        "toStationNumIsSet",
        "toStationIA5",
        "toStationIA5IsSet",
        "fromStationNameUTF8",
        "fromStationNameUTF8IsSet",
        "toStationNameUTF8",
        "toStationNameUTF8IsSet",
        "validReturnRegionDesc",
        "validReturnRegionDescIsSet",
        "validReturnRegion",
        "QList<KItinerary::Fcb::v13::RegionalValidityType>",
        "validReturnRegionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fromStationNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validReturnRegionDesc'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validReturnRegionDescIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validReturnRegion'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::RegionalValidityType>>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validReturnRegionIsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ReturnRouteDescriptionType, qt_meta_tag_ZN10KItinerary3Fcb3v1326ReturnRouteDescriptionTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ReturnRouteDescriptionType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1326ReturnRouteDescriptionTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1326ReturnRouteDescriptionTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1326ReturnRouteDescriptionTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ReturnRouteDescriptionType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ReturnRouteDescriptionType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->fromStationNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->fromStationNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->fromStationIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->fromStationIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->toStationNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->toStationNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->toStationIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->toStationIA5IsSet(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->fromStationNameUTF8; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->fromStationNameUTF8IsSet(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->toStationNameUTF8; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->toStationNameUTF8IsSet(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->validReturnRegionDesc; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->validReturnRegionDescIsSet(); break;
        case 14: *reinterpret_cast<QList<KItinerary::Fcb::v13::RegionalValidityType>*>(_v) = _t->validReturnRegion; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->validReturnRegionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1316RouteSectionTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::RouteSectionType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1316RouteSectionTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::RouteSectionType",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "fromStationNum",
        "fromStationNumIsSet",
        "fromStationIA5",
        "fromStationIA5IsSet",
        "toStationNum",
        "toStationNumIsSet",
        "toStationIA5",
        "toStationIA5IsSet",
        "fromStationNameUTF8",
        "fromStationNameUTF8IsSet",
        "toStationNameUTF8",
        "toStationNameUTF8IsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'fromStationNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(5, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNum'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(9, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RouteSectionType, qt_meta_tag_ZN10KItinerary3Fcb3v1316RouteSectionTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1316RouteSectionTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::RouteSectionType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316RouteSectionTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316RouteSectionTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1316RouteSectionTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316RouteSectionTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::RouteSectionType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RouteSectionType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->fromStationNum; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->fromStationNumIsSet(); break;
        case 3: *reinterpret_cast<QByteArray*>(_v) = _t->fromStationIA5; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->fromStationIA5IsSet(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->toStationNum; break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->toStationNumIsSet(); break;
        case 7: *reinterpret_cast<QByteArray*>(_v) = _t->toStationIA5; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->toStationIA5IsSet(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->fromStationNameUTF8; break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->fromStationNameUTF8IsSet(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->toStationNameUTF8; break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->toStationNameUTF8IsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1316SeriesDetailTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::SeriesDetailType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1316SeriesDetailTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::SeriesDetailType",
        "supplyingCarrier",
        "supplyingCarrierIsSet",
        "offerIdentification",
        "offerIdentificationIsSet",
        "series",
        "seriesIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'supplyingCarrier'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'supplyingCarrierIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'offerIdentification'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'offerIdentificationIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'series'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'seriesIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SeriesDetailType, qt_meta_tag_ZN10KItinerary3Fcb3v1316SeriesDetailTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::SeriesDetailType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316SeriesDetailTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316SeriesDetailTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316SeriesDetailTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::SeriesDetailType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<SeriesDetailType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->supplyingCarrier; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->supplyingCarrierIsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->offerIdentification; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->offerIdentificationIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->series; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->seriesIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1317CardReferenceTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::CardReferenceType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1317CardReferenceTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::CardReferenceType",
        "cardIssuerNum",
        "cardIssuerNumIsSet",
        "cardIssuerIA5",
        "cardIssuerIA5IsSet",
        "cardIdNum",
        "cardIdNumIsSet",
        "cardIdIA5",
        "cardIdIA5IsSet",
        "cardName",
        "cardNameIsSet",
        "cardType",
        "cardTypeIsSet",
        "leadingCardIdNum",
        "leadingCardIdNumIsSet",
        "leadingCardIdIA5",
        "leadingCardIdIA5IsSet",
        "trailingCardIdNum",
        "trailingCardIdNumIsSet",
        "trailingCardIdIA5",
        "trailingCardIdIA5IsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'cardIssuerNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardIssuerNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardIssuerIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardIssuerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardIdNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardName'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardNameIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardType'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardTypeIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'leadingCardIdNum'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'leadingCardIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'leadingCardIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(15, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'leadingCardIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trailingCardIdNum'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trailingCardIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trailingCardIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(19, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trailingCardIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CardReferenceType, qt_meta_tag_ZN10KItinerary3Fcb3v1317CardReferenceTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::CardReferenceType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317CardReferenceTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317CardReferenceTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317CardReferenceTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::CardReferenceType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CardReferenceType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->cardIssuerNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->cardIssuerNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->cardIssuerIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->cardIssuerIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->cardIdNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->cardIdNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->cardIdIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->cardIdIA5IsSet(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->cardName; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->cardNameIsSet(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->cardType; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->cardTypeIsSet(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->leadingCardIdNum; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->leadingCardIdNumIsSet(); break;
        case 14: *reinterpret_cast<QByteArray*>(_v) = _t->leadingCardIdIA5; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->leadingCardIdIA5IsSet(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->trailingCardIdNum; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->trailingCardIdNumIsSet(); break;
        case 18: *reinterpret_cast<QByteArray*>(_v) = _t->trailingCardIdIA5; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->trailingCardIdIA5IsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1310PlacesTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::PlacesType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1310PlacesTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::PlacesType",
        "coach",
        "coachIsSet",
        "placeString",
        "placeStringIsSet",
        "placeDescription",
        "placeDescriptionIsSet",
        "placeIA5",
        "QList<QByteArray>",
        "placeIA5IsSet",
        "placeNum",
        "QList<int>",
        "placeNumIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'coach'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'coachIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'placeString'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'placeStringIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'placeDescription'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'placeDescriptionIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'placeIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'placeIA5IsSet'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'placeNum'
        QtMocHelpers::PropertyData<QList<int>>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'placeNumIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlacesType, qt_meta_tag_ZN10KItinerary3Fcb3v1310PlacesTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::PlacesType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1310PlacesTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1310PlacesTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1310PlacesTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::PlacesType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<PlacesType *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->coach; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->coachIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->placeString; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->placeStringIsSet(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->placeDescription; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->placeDescriptionIsSet(); break;
        case 6: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->placeIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->placeIA5IsSet(); break;
        case 8: *reinterpret_cast<QList<int>*>(_v) = _t->placeNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->placeNumIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::CompartmentDetailsType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::CompartmentDetailsType",
        "coachType",
        "coachTypeIsSet",
        "compartmentType",
        "compartmentTypeIsSet",
        "specialAllocation",
        "specialAllocationIsSet",
        "coachTypeDescr",
        "coachTypeDescrIsSet",
        "compartmentTypeDescr",
        "compartmentTypeDescrIsSet",
        "specialAllocationDescr",
        "specialAllocationDescrIsSet",
        "position",
        "KItinerary::Fcb::v13::CompartmentPositionType"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'coachType'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'coachTypeIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'compartmentType'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'compartmentTypeIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'specialAllocation'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'specialAllocationIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'coachTypeDescr'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'coachTypeDescrIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'compartmentTypeDescr'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'compartmentTypeDescrIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'specialAllocationDescr'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'specialAllocationDescrIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'position'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CompartmentPositionType>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CompartmentDetailsType, qt_meta_tag_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::CompartmentDetailsType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322CompartmentDetailsTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::CompartmentDetailsType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CompartmentDetailsType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->coachType; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->coachTypeIsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->compartmentType; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->compartmentTypeIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->specialAllocation; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->specialAllocationIsSet(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->coachTypeDescr; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->coachTypeDescrIsSet(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->compartmentTypeDescr; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->compartmentTypeDescrIsSet(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->specialAllocationDescr; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->specialAllocationDescrIsSet(); break;
        case 12: *reinterpret_cast<KItinerary::Fcb::v13::CompartmentPositionType*>(_v) = _t->position; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1315BerthDetailDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::BerthDetailData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1315BerthDetailDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::BerthDetailData",
        "berthType",
        "KItinerary::Fcb::v13::BerthTypeType",
        "numberOfBerths",
        "gender",
        "KItinerary::Fcb::v13::CompartmentGenderType"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'berthType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::BerthTypeType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'numberOfBerths'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'gender'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CompartmentGenderType>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BerthDetailData, qt_meta_tag_ZN10KItinerary3Fcb3v1315BerthDetailDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1315BerthDetailDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::BerthDetailData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315BerthDetailDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315BerthDetailDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1315BerthDetailDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315BerthDetailDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::BerthDetailData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<BerthDetailData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::BerthTypeType*>(_v) = _t->berthType; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->numberOfBerths; break;
        case 2: *reinterpret_cast<KItinerary::Fcb::v13::CompartmentGenderType*>(_v) = _t->gender; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1310TariffTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::TariffType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1310TariffTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::TariffType",
        "numberOfPassengers",
        "passengerType",
        "KItinerary::Fcb::v13::PassengerType",
        "passengerTypeIsSet",
        "ageBelow",
        "ageBelowIsSet",
        "ageAbove",
        "ageAboveIsSet",
        "travelerid",
        "QList<int>",
        "traveleridIsSet",
        "restrictedToCountryOfResidence",
        "restrictedToRouteSection",
        "KItinerary::Fcb::v13::RouteSectionType",
        "restrictedToRouteSectionIsSet",
        "seriesDataDetails",
        "KItinerary::Fcb::v13::SeriesDetailType",
        "seriesDataDetailsIsSet",
        "tariffIdNum",
        "tariffIdNumIsSet",
        "tariffIdIA5",
        "tariffIdIA5IsSet",
        "tariffDesc",
        "tariffDescIsSet",
        "reductionCard",
        "QList<KItinerary::Fcb::v13::CardReferenceType>",
        "reductionCardIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'numberOfPassengers'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'passengerType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::PassengerType>(2, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'passengerTypeIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ageBelow'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'ageBelowIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ageAbove'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'ageAboveIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'travelerid'
        QtMocHelpers::PropertyData<QList<int>>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'traveleridIsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'restrictedToCountryOfResidence'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'restrictedToRouteSection'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::RouteSectionType>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'restrictedToRouteSectionIsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'seriesDataDetails'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::SeriesDetailType>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'seriesDataDetailsIsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariffIdNum'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tariffIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariffIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(21, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tariffIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariffDesc'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tariffDescIsSet'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'reductionCard'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::CardReferenceType>>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'reductionCardIsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TariffType, qt_meta_tag_ZN10KItinerary3Fcb3v1310TariffTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1310TariffTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::TariffType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1310TariffTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1310TariffTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1310TariffTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1310TariffTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::TariffType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TariffType *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->numberOfPassengers; break;
        case 1: *reinterpret_cast<KItinerary::Fcb::v13::PassengerType*>(_v) = _t->passengerType; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->passengerTypeIsSet(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->ageBelow; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->ageBelowIsSet(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->ageAbove; break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->ageAboveIsSet(); break;
        case 7: *reinterpret_cast<QList<int>*>(_v) = _t->travelerid; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->traveleridIsSet(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->restrictedToCountryOfResidence; break;
        case 10: *reinterpret_cast<KItinerary::Fcb::v13::RouteSectionType*>(_v) = _t->restrictedToRouteSection; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->restrictedToRouteSectionIsSet(); break;
        case 12: *reinterpret_cast<KItinerary::Fcb::v13::SeriesDetailType*>(_v) = _t->seriesDataDetails; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->seriesDataDetailsIsSet(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->tariffIdNum; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->tariffIdNumIsSet(); break;
        case 16: *reinterpret_cast<QByteArray*>(_v) = _t->tariffIdIA5; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->tariffIdIA5IsSet(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->tariffDesc; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->tariffDescIsSet(); break;
        case 20: *reinterpret_cast<QList<KItinerary::Fcb::v13::CardReferenceType>*>(_v) = _t->reductionCard; break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->reductionCardIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1313VatDetailTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::VatDetailType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1313VatDetailTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::VatDetailType",
        "country",
        "percentage",
        "amount",
        "amountIsSet",
        "vatId",
        "vatIdIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'country'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'percentage'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'amount'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'amountIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'vatId'
        QtMocHelpers::PropertyData<QByteArray>(5, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'vatIdIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VatDetailType, qt_meta_tag_ZN10KItinerary3Fcb3v1313VatDetailTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::VatDetailType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313VatDetailTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313VatDetailTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313VatDetailTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::VatDetailType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VatDetailType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->country; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->percentage; break;
        case 2: *reinterpret_cast<int*>(_v) = _t->amount; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->amountIsSet(); break;
        case 4: *reinterpret_cast<QByteArray*>(_v) = _t->vatId; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->vatIdIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::IncludedOpenTicketType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::IncludedOpenTicketType",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "externalIssuerId",
        "externalIssuerIdIsSet",
        "issuerAutorizationId",
        "issuerAutorizationIdIsSet",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "validRegion",
        "QList<KItinerary::Fcb::v13::RegionalValidityType>",
        "validRegionIsSet",
        "validFromDay",
        "validFromTime",
        "validFromTimeIsSet",
        "validFromUTCOffset",
        "validFromUTCOffsetIsSet",
        "validUntilDay",
        "validUntilTime",
        "validUntilTimeIsSet",
        "validUntilUTCOffset",
        "validUntilUTCOffsetIsSet",
        "classCode",
        "KItinerary::Fcb::v13::TravelClassType",
        "classCodeIsSet",
        "serviceLevel",
        "serviceLevelIsSet",
        "carrierNum",
        "QList<int>",
        "carrierNumIsSet",
        "carrierIA5",
        "QList<QByteArray>",
        "carrierIA5IsSet",
        "includedServiceBrands",
        "includedServiceBrandsIsSet",
        "excludedServiceBrands",
        "excludedServiceBrandsIsSet",
        "tariffs",
        "QList<KItinerary::Fcb::v13::TariffType>",
        "tariffsIsSet",
        "infoText",
        "infoTextIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'externalIssuerId'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'externalIssuerIdIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuerAutorizationId'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerAutorizationIdIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validRegion'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::RegionalValidityType>>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validRegionIsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTime'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTimeIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromUTCOffset'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTime'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTimeIsSet'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilUTCOffset'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'classCode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelClassType>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'classCodeIsSet'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'serviceLevel'
        QtMocHelpers::PropertyData<QByteArray>(31, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceLevelIsSet'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierNum'
        QtMocHelpers::PropertyData<QList<int>>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(39, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'excludedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(41, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'excludedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariffs'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TariffType>>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'tariffsIsSet'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(48, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IncludedOpenTicketType, qt_meta_tag_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::IncludedOpenTicketType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322IncludedOpenTicketTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::IncludedOpenTicketType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<IncludedOpenTicketType *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 31:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 35:
        case 33:
        case 29:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->externalIssuerId; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->externalIssuerIdIsSet(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->issuerAutorizationId; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->issuerAutorizationIdIsSet(); break;
        case 12: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 13: *reinterpret_cast<QList<KItinerary::Fcb::v13::RegionalValidityType>*>(_v) = _t->validRegion; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->validRegionIsSet(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 16: *reinterpret_cast<int*>(_v) = _t->validFromTime; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->validFromTimeIsSet(); break;
        case 18: *reinterpret_cast<int*>(_v) = _t->validFromUTCOffset; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->validFromUTCOffsetIsSet(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 21: *reinterpret_cast<int*>(_v) = _t->validUntilTime; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->validUntilTimeIsSet(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->validUntilUTCOffset; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->validUntilUTCOffsetIsSet(); break;
        case 25: *reinterpret_cast<KItinerary::Fcb::v13::TravelClassType*>(_v) = _t->classCode; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->classCodeIsSet(); break;
        case 27: *reinterpret_cast<QByteArray*>(_v) = _t->serviceLevel; break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->serviceLevelIsSet(); break;
        case 29: *reinterpret_cast<QList<int>*>(_v) = _t->carrierNum; break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 31: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->carrierIA5; break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 33: *reinterpret_cast<QList<int>*>(_v) = _t->includedServiceBrands; break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->includedServiceBrandsIsSet(); break;
        case 35: *reinterpret_cast<QList<int>*>(_v) = _t->excludedServiceBrands; break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->excludedServiceBrandsIsSet(); break;
        case 37: *reinterpret_cast<QList<KItinerary::Fcb::v13::TariffType>*>(_v) = _t->tariffs; break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->tariffsIsSet(); break;
        case 39: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 41: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 42: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1321RegisteredLuggageTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::RegisteredLuggageType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1321RegisteredLuggageTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::RegisteredLuggageType",
        "registrationId",
        "registrationIdIsSet",
        "maxWeight",
        "maxWeightIsSet",
        "maxSize",
        "maxSizeIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'registrationId'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'registrationIdIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'maxWeight'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'maxWeightIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'maxSize'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'maxSizeIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RegisteredLuggageType, qt_meta_tag_ZN10KItinerary3Fcb3v1321RegisteredLuggageTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::RegisteredLuggageType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1321RegisteredLuggageTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1321RegisteredLuggageTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1321RegisteredLuggageTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::RegisteredLuggageType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RegisteredLuggageType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->registrationId; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->registrationIdIsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->maxWeight; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->maxWeightIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->maxSize; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->maxSizeIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1322LuggageRestrictionTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::LuggageRestrictionType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1322LuggageRestrictionTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::LuggageRestrictionType",
        "maxHandLuggagePieces",
        "maxNonHandLuggagePieces",
        "registeredLuggage",
        "QList<KItinerary::Fcb::v13::RegisteredLuggageType>",
        "registeredLuggageIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'maxHandLuggagePieces'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'maxNonHandLuggagePieces'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'registeredLuggage'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::RegisteredLuggageType>>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'registeredLuggageIsSet'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LuggageRestrictionType, qt_meta_tag_ZN10KItinerary3Fcb3v1322LuggageRestrictionTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::LuggageRestrictionType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322LuggageRestrictionTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322LuggageRestrictionTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1322LuggageRestrictionTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::LuggageRestrictionType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<LuggageRestrictionType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->maxHandLuggagePieces; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->maxNonHandLuggagePieces; break;
        case 2: *reinterpret_cast<QList<KItinerary::Fcb::v13::RegisteredLuggageType>*>(_v) = _t->registeredLuggage; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->registeredLuggageIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1315ReservationDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ReservationData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1315ReservationDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ReservationData",
        "departureDateTime",
        "",
        "issueingDateTime",
        "arrivalDateTime",
        "trainNum",
        "trainNumIsSet",
        "trainIA5",
        "trainIA5IsSet",
        "departureDate",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "serviceBrand",
        "serviceBrandIsSet",
        "serviceBrandAbrUTF8",
        "serviceBrandAbrUTF8IsSet",
        "serviceBrandNameUTF8",
        "serviceBrandNameUTF8IsSet",
        "service",
        "KItinerary::Fcb::v13::ServiceType",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "fromStationNum",
        "fromStationNumIsSet",
        "fromStationIA5",
        "fromStationIA5IsSet",
        "toStationNum",
        "toStationNumIsSet",
        "toStationIA5",
        "toStationIA5IsSet",
        "fromStationNameUTF8",
        "fromStationNameUTF8IsSet",
        "toStationNameUTF8",
        "toStationNameUTF8IsSet",
        "departureTime",
        "departureUTCOffset",
        "departureUTCOffsetIsSet",
        "arrivalDate",
        "arrivalTime",
        "arrivalTimeIsSet",
        "arrivalUTCOffset",
        "arrivalUTCOffsetIsSet",
        "carrierNum",
        "QList<int>",
        "carrierNumIsSet",
        "carrierIA5",
        "QList<QByteArray>",
        "carrierIA5IsSet",
        "classCode",
        "KItinerary::Fcb::v13::TravelClassType",
        "serviceLevel",
        "serviceLevelIsSet",
        "places",
        "KItinerary::Fcb::v13::PlacesType",
        "placesIsSet",
        "additionalPlaces",
        "additionalPlacesIsSet",
        "bicyclePlaces",
        "bicyclePlacesIsSet",
        "compartmentDetails",
        "KItinerary::Fcb::v13::CompartmentDetailsType",
        "compartmentDetailsIsSet",
        "numberOfOverbooked",
        "berth",
        "QList<KItinerary::Fcb::v13::BerthDetailData>",
        "berthIsSet",
        "tariffs",
        "QList<KItinerary::Fcb::v13::TariffType>",
        "tariffsIsSet",
        "priceType",
        "KItinerary::Fcb::v13::PriceTypeType",
        "price",
        "priceIsSet",
        "vatDetail",
        "QList<KItinerary::Fcb::v13::VatDetailType>",
        "vatDetailIsSet",
        "typeOfSupplement",
        "numberOfSupplements",
        "luggage",
        "KItinerary::Fcb::v13::LuggageRestrictionType",
        "luggageIsSet",
        "infoText",
        "infoTextIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'departureDateTime'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'arrivalDateTime'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(4, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'trainNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trainIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'departureDate'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(10, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(16, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(20, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'serviceBrand'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceBrandIsSet'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'serviceBrandAbrUTF8'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceBrandAbrUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'serviceBrandNameUTF8'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceBrandNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'service'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ServiceType>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'fromStationNum'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(34, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNum'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(38, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'departureTime'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureUTCOffset'
        QtMocHelpers::PropertyData<int>(45, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'arrivalDate'
        QtMocHelpers::PropertyData<int>(47, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'arrivalTime'
        QtMocHelpers::PropertyData<int>(48, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'arrivalTimeIsSet'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'arrivalUTCOffset'
        QtMocHelpers::PropertyData<int>(50, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'arrivalUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierNum'
        QtMocHelpers::PropertyData<QList<int>>(52, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'classCode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelClassType>(58, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'serviceLevel'
        QtMocHelpers::PropertyData<QByteArray>(60, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceLevelIsSet'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'places'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::PlacesType>(62, 0x80000000 | 63, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'placesIsSet'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'additionalPlaces'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::PlacesType>(65, 0x80000000 | 63, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'additionalPlacesIsSet'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'bicyclePlaces'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::PlacesType>(67, 0x80000000 | 63, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'bicyclePlacesIsSet'
        QtMocHelpers::PropertyData<bool>(68, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'compartmentDetails'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CompartmentDetailsType>(69, 0x80000000 | 70, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'compartmentDetailsIsSet'
        QtMocHelpers::PropertyData<bool>(71, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberOfOverbooked'
        QtMocHelpers::PropertyData<int>(72, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'berth'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::BerthDetailData>>(73, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'berthIsSet'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariffs'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TariffType>>(76, 0x80000000 | 77, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'tariffsIsSet'
        QtMocHelpers::PropertyData<bool>(78, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'priceType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::PriceTypeType>(79, 0x80000000 | 80, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'price'
        QtMocHelpers::PropertyData<int>(81, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'priceIsSet'
        QtMocHelpers::PropertyData<bool>(82, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'vatDetail'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::VatDetailType>>(83, 0x80000000 | 84, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'vatDetailIsSet'
        QtMocHelpers::PropertyData<bool>(85, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'typeOfSupplement'
        QtMocHelpers::PropertyData<int>(86, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numberOfSupplements'
        QtMocHelpers::PropertyData<int>(87, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'luggage'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::LuggageRestrictionType>(88, 0x80000000 | 89, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'luggageIsSet'
        QtMocHelpers::PropertyData<bool>(90, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(91, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(92, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(93, 0x80000000 | 94, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(95, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ReservationData, qt_meta_tag_ZN10KItinerary3Fcb3v1315ReservationDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1315ReservationDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ReservationData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315ReservationDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315ReservationDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1315ReservationDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315ReservationDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ReservationData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ReservationData *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDateTime _r = _t->departureDateTime((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDateTime _r = _t->arrivalDateTime((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 47:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 45:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->trainNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->trainNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->trainIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->trainIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->departureDate; break;
        case 5: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 11: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 15: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 17: *reinterpret_cast<int*>(_v) = _t->serviceBrand; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->serviceBrandIsSet(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->serviceBrandAbrUTF8; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->serviceBrandAbrUTF8IsSet(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->serviceBrandNameUTF8; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->serviceBrandNameUTF8IsSet(); break;
        case 23: *reinterpret_cast<KItinerary::Fcb::v13::ServiceType*>(_v) = _t->service; break;
        case 24: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 25: *reinterpret_cast<int*>(_v) = _t->fromStationNum; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->fromStationNumIsSet(); break;
        case 27: *reinterpret_cast<QByteArray*>(_v) = _t->fromStationIA5; break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->fromStationIA5IsSet(); break;
        case 29: *reinterpret_cast<int*>(_v) = _t->toStationNum; break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->toStationNumIsSet(); break;
        case 31: *reinterpret_cast<QByteArray*>(_v) = _t->toStationIA5; break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->toStationIA5IsSet(); break;
        case 33: *reinterpret_cast<QString*>(_v) = _t->fromStationNameUTF8; break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->fromStationNameUTF8IsSet(); break;
        case 35: *reinterpret_cast<QString*>(_v) = _t->toStationNameUTF8; break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->toStationNameUTF8IsSet(); break;
        case 37: *reinterpret_cast<int*>(_v) = _t->departureTime; break;
        case 38: *reinterpret_cast<int*>(_v) = _t->departureUTCOffset; break;
        case 39: *reinterpret_cast<bool*>(_v) = _t->departureUTCOffsetIsSet(); break;
        case 40: *reinterpret_cast<int*>(_v) = _t->arrivalDate; break;
        case 41: *reinterpret_cast<int*>(_v) = _t->arrivalTime; break;
        case 42: *reinterpret_cast<bool*>(_v) = _t->arrivalTimeIsSet(); break;
        case 43: *reinterpret_cast<int*>(_v) = _t->arrivalUTCOffset; break;
        case 44: *reinterpret_cast<bool*>(_v) = _t->arrivalUTCOffsetIsSet(); break;
        case 45: *reinterpret_cast<QList<int>*>(_v) = _t->carrierNum; break;
        case 46: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 47: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->carrierIA5; break;
        case 48: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 49: *reinterpret_cast<KItinerary::Fcb::v13::TravelClassType*>(_v) = _t->classCode; break;
        case 50: *reinterpret_cast<QByteArray*>(_v) = _t->serviceLevel; break;
        case 51: *reinterpret_cast<bool*>(_v) = _t->serviceLevelIsSet(); break;
        case 52: *reinterpret_cast<KItinerary::Fcb::v13::PlacesType*>(_v) = _t->places; break;
        case 53: *reinterpret_cast<bool*>(_v) = _t->placesIsSet(); break;
        case 54: *reinterpret_cast<KItinerary::Fcb::v13::PlacesType*>(_v) = _t->additionalPlaces; break;
        case 55: *reinterpret_cast<bool*>(_v) = _t->additionalPlacesIsSet(); break;
        case 56: *reinterpret_cast<KItinerary::Fcb::v13::PlacesType*>(_v) = _t->bicyclePlaces; break;
        case 57: *reinterpret_cast<bool*>(_v) = _t->bicyclePlacesIsSet(); break;
        case 58: *reinterpret_cast<KItinerary::Fcb::v13::CompartmentDetailsType*>(_v) = _t->compartmentDetails; break;
        case 59: *reinterpret_cast<bool*>(_v) = _t->compartmentDetailsIsSet(); break;
        case 60: *reinterpret_cast<int*>(_v) = _t->numberOfOverbooked; break;
        case 61: *reinterpret_cast<QList<KItinerary::Fcb::v13::BerthDetailData>*>(_v) = _t->berth; break;
        case 62: *reinterpret_cast<bool*>(_v) = _t->berthIsSet(); break;
        case 63: *reinterpret_cast<QList<KItinerary::Fcb::v13::TariffType>*>(_v) = _t->tariffs; break;
        case 64: *reinterpret_cast<bool*>(_v) = _t->tariffsIsSet(); break;
        case 65: *reinterpret_cast<KItinerary::Fcb::v13::PriceTypeType*>(_v) = _t->priceType; break;
        case 66: *reinterpret_cast<int*>(_v) = _t->price; break;
        case 67: *reinterpret_cast<bool*>(_v) = _t->priceIsSet(); break;
        case 68: *reinterpret_cast<QList<KItinerary::Fcb::v13::VatDetailType>*>(_v) = _t->vatDetail; break;
        case 69: *reinterpret_cast<bool*>(_v) = _t->vatDetailIsSet(); break;
        case 70: *reinterpret_cast<int*>(_v) = _t->typeOfSupplement; break;
        case 71: *reinterpret_cast<int*>(_v) = _t->numberOfSupplements; break;
        case 72: *reinterpret_cast<KItinerary::Fcb::v13::LuggageRestrictionType*>(_v) = _t->luggage; break;
        case 73: *reinterpret_cast<bool*>(_v) = _t->luggageIsSet(); break;
        case 74: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 75: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 76: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 77: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::CarCarriageReservationData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::CarCarriageReservationData",
        "trainNum",
        "trainNumIsSet",
        "trainIA5",
        "trainIA5IsSet",
        "beginLoadingDate",
        "beginLoadingTime",
        "beginLoadingTimeIsSet",
        "endLoadingTime",
        "endLoadingTimeIsSet",
        "loadingUTCOffset",
        "loadingUTCOffsetIsSet",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "serviceBrand",
        "serviceBrandIsSet",
        "serviceBrandAbrUTF8",
        "serviceBrandAbrUTF8IsSet",
        "serviceBrandNameUTF8",
        "serviceBrandNameUTF8IsSet",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "fromStationNum",
        "fromStationNumIsSet",
        "fromStationIA5",
        "fromStationIA5IsSet",
        "toStationNum",
        "toStationNumIsSet",
        "toStationIA5",
        "toStationIA5IsSet",
        "fromStationNameUTF8",
        "fromStationNameUTF8IsSet",
        "toStationNameUTF8",
        "toStationNameUTF8IsSet",
        "coach",
        "coachIsSet",
        "place",
        "placeIsSet",
        "compartmentDetails",
        "KItinerary::Fcb::v13::CompartmentDetailsType",
        "compartmentDetailsIsSet",
        "numberPlate",
        "numberPlateIsSet",
        "trailerPlate",
        "trailerPlateIsSet",
        "carCategory",
        "boatCategory",
        "boatCategoryIsSet",
        "textileRoof",
        "roofRackType",
        "KItinerary::Fcb::v13::RoofRackType",
        "roofRackHeight",
        "roofRackHeightIsSet",
        "attachedBoats",
        "attachedBoatsIsSet",
        "attachedBicycles",
        "attachedBicyclesIsSet",
        "attachedSurfboards",
        "attachedSurfboardsIsSet",
        "loadingListEntry",
        "loadingListEntryIsSet",
        "loadingDeck",
        "KItinerary::Fcb::v13::LoadingDeckType",
        "carrierNum",
        "QList<int>",
        "carrierNumIsSet",
        "carrierIA5",
        "QList<QByteArray>",
        "carrierIA5IsSet",
        "tariff",
        "KItinerary::Fcb::v13::TariffType",
        "priceType",
        "KItinerary::Fcb::v13::PriceTypeType",
        "price",
        "priceIsSet",
        "vatDetail",
        "QList<KItinerary::Fcb::v13::VatDetailType>",
        "vatDetailIsSet",
        "infoText",
        "infoTextIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'trainNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trainIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'beginLoadingDate'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'beginLoadingTime'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'beginLoadingTimeIsSet'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'endLoadingTime'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'endLoadingTimeIsSet'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'loadingUTCOffset'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'loadingUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(12, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(18, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(22, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'serviceBrand'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceBrandIsSet'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'serviceBrandAbrUTF8'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceBrandAbrUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'serviceBrandNameUTF8'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceBrandNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'fromStationNum'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(34, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNum'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(38, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'coach'
        QtMocHelpers::PropertyData<QByteArray>(44, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'coachIsSet'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'place'
        QtMocHelpers::PropertyData<QByteArray>(46, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'placeIsSet'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'compartmentDetails'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CompartmentDetailsType>(48, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'compartmentDetailsIsSet'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberPlate'
        QtMocHelpers::PropertyData<QByteArray>(51, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numberPlateIsSet'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trailerPlate'
        QtMocHelpers::PropertyData<QByteArray>(53, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trailerPlateIsSet'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carCategory'
        QtMocHelpers::PropertyData<int>(55, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'boatCategory'
        QtMocHelpers::PropertyData<int>(56, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'boatCategoryIsSet'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'textileRoof'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'roofRackType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::RoofRackType>(59, 0x80000000 | 60, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'roofRackHeight'
        QtMocHelpers::PropertyData<int>(61, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'roofRackHeightIsSet'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'attachedBoats'
        QtMocHelpers::PropertyData<int>(63, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'attachedBoatsIsSet'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'attachedBicycles'
        QtMocHelpers::PropertyData<int>(65, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'attachedBicyclesIsSet'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'attachedSurfboards'
        QtMocHelpers::PropertyData<int>(67, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'attachedSurfboardsIsSet'
        QtMocHelpers::PropertyData<bool>(68, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'loadingListEntry'
        QtMocHelpers::PropertyData<int>(69, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'loadingListEntryIsSet'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'loadingDeck'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::LoadingDeckType>(71, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNum'
        QtMocHelpers::PropertyData<QList<int>>(73, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(76, 0x80000000 | 77, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(78, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariff'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TariffType>(79, 0x80000000 | 80, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'priceType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::PriceTypeType>(81, 0x80000000 | 82, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'price'
        QtMocHelpers::PropertyData<int>(83, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'priceIsSet'
        QtMocHelpers::PropertyData<bool>(84, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'vatDetail'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::VatDetailType>>(85, 0x80000000 | 86, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'vatDetailIsSet'
        QtMocHelpers::PropertyData<bool>(87, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(88, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(89, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(90, 0x80000000 | 91, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(92, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CarCarriageReservationData, qt_meta_tag_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::CarCarriageReservationData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1326CarCarriageReservationDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::CarCarriageReservationData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CarCarriageReservationData *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 70:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 68:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->trainNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->trainNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->trainIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->trainIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->beginLoadingDate; break;
        case 5: *reinterpret_cast<int*>(_v) = _t->beginLoadingTime; break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->beginLoadingTimeIsSet(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->endLoadingTime; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->endLoadingTimeIsSet(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->loadingUTCOffset; break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->loadingUTCOffsetIsSet(); break;
        case 11: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 17: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 21: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->serviceBrand; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->serviceBrandIsSet(); break;
        case 25: *reinterpret_cast<QString*>(_v) = _t->serviceBrandAbrUTF8; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->serviceBrandAbrUTF8IsSet(); break;
        case 27: *reinterpret_cast<QString*>(_v) = _t->serviceBrandNameUTF8; break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->serviceBrandNameUTF8IsSet(); break;
        case 29: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 30: *reinterpret_cast<int*>(_v) = _t->fromStationNum; break;
        case 31: *reinterpret_cast<bool*>(_v) = _t->fromStationNumIsSet(); break;
        case 32: *reinterpret_cast<QByteArray*>(_v) = _t->fromStationIA5; break;
        case 33: *reinterpret_cast<bool*>(_v) = _t->fromStationIA5IsSet(); break;
        case 34: *reinterpret_cast<int*>(_v) = _t->toStationNum; break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->toStationNumIsSet(); break;
        case 36: *reinterpret_cast<QByteArray*>(_v) = _t->toStationIA5; break;
        case 37: *reinterpret_cast<bool*>(_v) = _t->toStationIA5IsSet(); break;
        case 38: *reinterpret_cast<QString*>(_v) = _t->fromStationNameUTF8; break;
        case 39: *reinterpret_cast<bool*>(_v) = _t->fromStationNameUTF8IsSet(); break;
        case 40: *reinterpret_cast<QString*>(_v) = _t->toStationNameUTF8; break;
        case 41: *reinterpret_cast<bool*>(_v) = _t->toStationNameUTF8IsSet(); break;
        case 42: *reinterpret_cast<QByteArray*>(_v) = _t->coach; break;
        case 43: *reinterpret_cast<bool*>(_v) = _t->coachIsSet(); break;
        case 44: *reinterpret_cast<QByteArray*>(_v) = _t->place; break;
        case 45: *reinterpret_cast<bool*>(_v) = _t->placeIsSet(); break;
        case 46: *reinterpret_cast<KItinerary::Fcb::v13::CompartmentDetailsType*>(_v) = _t->compartmentDetails; break;
        case 47: *reinterpret_cast<bool*>(_v) = _t->compartmentDetailsIsSet(); break;
        case 48: *reinterpret_cast<QByteArray*>(_v) = _t->numberPlate; break;
        case 49: *reinterpret_cast<bool*>(_v) = _t->numberPlateIsSet(); break;
        case 50: *reinterpret_cast<QByteArray*>(_v) = _t->trailerPlate; break;
        case 51: *reinterpret_cast<bool*>(_v) = _t->trailerPlateIsSet(); break;
        case 52: *reinterpret_cast<int*>(_v) = _t->carCategory; break;
        case 53: *reinterpret_cast<int*>(_v) = _t->boatCategory; break;
        case 54: *reinterpret_cast<bool*>(_v) = _t->boatCategoryIsSet(); break;
        case 55: *reinterpret_cast<bool*>(_v) = _t->textileRoof; break;
        case 56: *reinterpret_cast<KItinerary::Fcb::v13::RoofRackType*>(_v) = _t->roofRackType; break;
        case 57: *reinterpret_cast<int*>(_v) = _t->roofRackHeight; break;
        case 58: *reinterpret_cast<bool*>(_v) = _t->roofRackHeightIsSet(); break;
        case 59: *reinterpret_cast<int*>(_v) = _t->attachedBoats; break;
        case 60: *reinterpret_cast<bool*>(_v) = _t->attachedBoatsIsSet(); break;
        case 61: *reinterpret_cast<int*>(_v) = _t->attachedBicycles; break;
        case 62: *reinterpret_cast<bool*>(_v) = _t->attachedBicyclesIsSet(); break;
        case 63: *reinterpret_cast<int*>(_v) = _t->attachedSurfboards; break;
        case 64: *reinterpret_cast<bool*>(_v) = _t->attachedSurfboardsIsSet(); break;
        case 65: *reinterpret_cast<int*>(_v) = _t->loadingListEntry; break;
        case 66: *reinterpret_cast<bool*>(_v) = _t->loadingListEntryIsSet(); break;
        case 67: *reinterpret_cast<KItinerary::Fcb::v13::LoadingDeckType*>(_v) = _t->loadingDeck; break;
        case 68: *reinterpret_cast<QList<int>*>(_v) = _t->carrierNum; break;
        case 69: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 70: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->carrierIA5; break;
        case 71: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 72: *reinterpret_cast<KItinerary::Fcb::v13::TariffType*>(_v) = _t->tariff; break;
        case 73: *reinterpret_cast<KItinerary::Fcb::v13::PriceTypeType*>(_v) = _t->priceType; break;
        case 74: *reinterpret_cast<int*>(_v) = _t->price; break;
        case 75: *reinterpret_cast<bool*>(_v) = _t->priceIsSet(); break;
        case 76: *reinterpret_cast<QList<KItinerary::Fcb::v13::VatDetailType>*>(_v) = _t->vatDetail; break;
        case 77: *reinterpret_cast<bool*>(_v) = _t->vatDetailIsSet(); break;
        case 78: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 79: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 80: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 81: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1314OpenTicketDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::OpenTicketData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1314OpenTicketDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::OpenTicketData",
        "validFrom",
        "",
        "issueingDateTime",
        "validUntil",
        "referenceNum",
        "referenceNumIsSet",
        "referenceIA5",
        "referenceIA5IsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "extIssuerId",
        "extIssuerIdIsSet",
        "issuerAutorizationId",
        "issuerAutorizationIdIsSet",
        "returnIncluded",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "fromStationNum",
        "fromStationNumIsSet",
        "fromStationIA5",
        "fromStationIA5IsSet",
        "toStationNum",
        "toStationNumIsSet",
        "toStationIA5",
        "toStationIA5IsSet",
        "fromStationNameUTF8",
        "fromStationNameUTF8IsSet",
        "toStationNameUTF8",
        "toStationNameUTF8IsSet",
        "validRegionDesc",
        "validRegionDescIsSet",
        "validRegion",
        "QList<KItinerary::Fcb::v13::RegionalValidityType>",
        "validRegionIsSet",
        "returnDescription",
        "KItinerary::Fcb::v13::ReturnRouteDescriptionType",
        "returnDescriptionIsSet",
        "validFromDay",
        "validFromTime",
        "validFromTimeIsSet",
        "validFromUTCOffset",
        "validFromUTCOffsetIsSet",
        "validUntilDay",
        "validUntilTime",
        "validUntilTimeIsSet",
        "validUntilUTCOffset",
        "validUntilUTCOffsetIsSet",
        "activatedDay",
        "QList<int>",
        "activatedDayIsSet",
        "classCode",
        "KItinerary::Fcb::v13::TravelClassType",
        "serviceLevel",
        "serviceLevelIsSet",
        "carrierNum",
        "carrierNumIsSet",
        "carrierIA5",
        "QList<QByteArray>",
        "carrierIA5IsSet",
        "includedServiceBrands",
        "includedServiceBrandsIsSet",
        "excludedServiceBrands",
        "excludedServiceBrandsIsSet",
        "tariffs",
        "QList<KItinerary::Fcb::v13::TariffType>",
        "tariffsIsSet",
        "price",
        "priceIsSet",
        "vatDetail",
        "QList<KItinerary::Fcb::v13::VatDetailType>",
        "vatDetailIsSet",
        "infoText",
        "infoTextIsSet",
        "includedAddOns",
        "QList<KItinerary::Fcb::v13::IncludedOpenTicketType>",
        "includedAddOnsIsSet",
        "luggage",
        "KItinerary::Fcb::v13::LuggageRestrictionType",
        "luggageIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'validFrom'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'validUntil'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(4, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(15, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extIssuerId'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'extIssuerIdIsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuerAutorizationId'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerAutorizationIdIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'returnIncluded'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'fromStationNum'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(26, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNum'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(30, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validRegionDesc'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validRegionDescIsSet'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validRegion'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::RegionalValidityType>>(38, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validRegionIsSet'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'returnDescription'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ReturnRouteDescriptionType>(41, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'returnDescriptionIsSet'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTime'
        QtMocHelpers::PropertyData<int>(45, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTimeIsSet'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromUTCOffset'
        QtMocHelpers::PropertyData<int>(47, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(49, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTime'
        QtMocHelpers::PropertyData<int>(50, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTimeIsSet'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilUTCOffset'
        QtMocHelpers::PropertyData<int>(52, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'activatedDay'
        QtMocHelpers::PropertyData<QList<int>>(54, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'activatedDayIsSet'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'classCode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelClassType>(57, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'serviceLevel'
        QtMocHelpers::PropertyData<QByteArray>(59, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'serviceLevelIsSet'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierNum'
        QtMocHelpers::PropertyData<QList<int>>(61, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(63, 0x80000000 | 64, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(66, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'excludedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(68, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'excludedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariffs'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TariffType>>(70, 0x80000000 | 71, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'tariffsIsSet'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'price'
        QtMocHelpers::PropertyData<int>(73, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'priceIsSet'
        QtMocHelpers::PropertyData<bool>(74, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'vatDetail'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::VatDetailType>>(75, 0x80000000 | 76, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'vatDetailIsSet'
        QtMocHelpers::PropertyData<bool>(77, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(78, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(79, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedAddOns'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::IncludedOpenTicketType>>(80, 0x80000000 | 81, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedAddOnsIsSet'
        QtMocHelpers::PropertyData<bool>(82, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'luggage'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::LuggageRestrictionType>(83, 0x80000000 | 84, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'luggageIsSet'
        QtMocHelpers::PropertyData<bool>(85, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(86, 0x80000000 | 87, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(88, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OpenTicketData, qt_meta_tag_ZN10KItinerary3Fcb3v1314OpenTicketDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1314OpenTicketDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::OpenTicketData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314OpenTicketDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314OpenTicketDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1314OpenTicketDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314OpenTicketDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::OpenTicketData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<OpenTicketData *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDateTime _r = _t->validFrom((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDateTime _r = _t->validUntil((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 53:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 57:
        case 55:
        case 51:
        case 46:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->extIssuerId; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->extIssuerIdIsSet(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->issuerAutorizationId; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->issuerAutorizationIdIsSet(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->returnIncluded; break;
        case 17: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 18: *reinterpret_cast<int*>(_v) = _t->fromStationNum; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->fromStationNumIsSet(); break;
        case 20: *reinterpret_cast<QByteArray*>(_v) = _t->fromStationIA5; break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->fromStationIA5IsSet(); break;
        case 22: *reinterpret_cast<int*>(_v) = _t->toStationNum; break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->toStationNumIsSet(); break;
        case 24: *reinterpret_cast<QByteArray*>(_v) = _t->toStationIA5; break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->toStationIA5IsSet(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->fromStationNameUTF8; break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->fromStationNameUTF8IsSet(); break;
        case 28: *reinterpret_cast<QString*>(_v) = _t->toStationNameUTF8; break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->toStationNameUTF8IsSet(); break;
        case 30: *reinterpret_cast<QString*>(_v) = _t->validRegionDesc; break;
        case 31: *reinterpret_cast<bool*>(_v) = _t->validRegionDescIsSet(); break;
        case 32: *reinterpret_cast<QList<KItinerary::Fcb::v13::RegionalValidityType>*>(_v) = _t->validRegion; break;
        case 33: *reinterpret_cast<bool*>(_v) = _t->validRegionIsSet(); break;
        case 34: *reinterpret_cast<KItinerary::Fcb::v13::ReturnRouteDescriptionType*>(_v) = _t->returnDescription; break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->returnDescriptionIsSet(); break;
        case 36: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 37: *reinterpret_cast<int*>(_v) = _t->validFromTime; break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->validFromTimeIsSet(); break;
        case 39: *reinterpret_cast<int*>(_v) = _t->validFromUTCOffset; break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->validFromUTCOffsetIsSet(); break;
        case 41: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 42: *reinterpret_cast<int*>(_v) = _t->validUntilTime; break;
        case 43: *reinterpret_cast<bool*>(_v) = _t->validUntilTimeIsSet(); break;
        case 44: *reinterpret_cast<int*>(_v) = _t->validUntilUTCOffset; break;
        case 45: *reinterpret_cast<bool*>(_v) = _t->validUntilUTCOffsetIsSet(); break;
        case 46: *reinterpret_cast<QList<int>*>(_v) = _t->activatedDay; break;
        case 47: *reinterpret_cast<bool*>(_v) = _t->activatedDayIsSet(); break;
        case 48: *reinterpret_cast<KItinerary::Fcb::v13::TravelClassType*>(_v) = _t->classCode; break;
        case 49: *reinterpret_cast<QByteArray*>(_v) = _t->serviceLevel; break;
        case 50: *reinterpret_cast<bool*>(_v) = _t->serviceLevelIsSet(); break;
        case 51: *reinterpret_cast<QList<int>*>(_v) = _t->carrierNum; break;
        case 52: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 53: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->carrierIA5; break;
        case 54: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 55: *reinterpret_cast<QList<int>*>(_v) = _t->includedServiceBrands; break;
        case 56: *reinterpret_cast<bool*>(_v) = _t->includedServiceBrandsIsSet(); break;
        case 57: *reinterpret_cast<QList<int>*>(_v) = _t->excludedServiceBrands; break;
        case 58: *reinterpret_cast<bool*>(_v) = _t->excludedServiceBrandsIsSet(); break;
        case 59: *reinterpret_cast<QList<KItinerary::Fcb::v13::TariffType>*>(_v) = _t->tariffs; break;
        case 60: *reinterpret_cast<bool*>(_v) = _t->tariffsIsSet(); break;
        case 61: *reinterpret_cast<int*>(_v) = _t->price; break;
        case 62: *reinterpret_cast<bool*>(_v) = _t->priceIsSet(); break;
        case 63: *reinterpret_cast<QList<KItinerary::Fcb::v13::VatDetailType>*>(_v) = _t->vatDetail; break;
        case 64: *reinterpret_cast<bool*>(_v) = _t->vatDetailIsSet(); break;
        case 65: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 66: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 67: *reinterpret_cast<QList<KItinerary::Fcb::v13::IncludedOpenTicketType>*>(_v) = _t->includedAddOns; break;
        case 68: *reinterpret_cast<bool*>(_v) = _t->includedAddOnsIsSet(); break;
        case 69: *reinterpret_cast<KItinerary::Fcb::v13::LuggageRestrictionType*>(_v) = _t->luggage; break;
        case 70: *reinterpret_cast<bool*>(_v) = _t->luggageIsSet(); break;
        case 71: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 72: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1313TimeRangeTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::TimeRangeType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1313TimeRangeTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::TimeRangeType",
        "fromTime",
        "untilTime"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fromTime'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'untilTime'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TimeRangeType, qt_meta_tag_ZN10KItinerary3Fcb3v1313TimeRangeTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::TimeRangeType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313TimeRangeTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313TimeRangeTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313TimeRangeTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::TimeRangeType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TimeRangeType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->fromTime; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->untilTime; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1318ValidityPeriodTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ValidityPeriodType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1318ValidityPeriodTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ValidityPeriodType",
        "validFromDay",
        "validFromTime",
        "validFromTimeIsSet",
        "validFromUTCOffset",
        "validFromUTCOffsetIsSet",
        "validUntilDay",
        "validUntilTime",
        "validUntilTimeIsSet",
        "validUntilUTCOffset",
        "validUntilUTCOffsetIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTime'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTimeIsSet'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromUTCOffset'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTime'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTimeIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilUTCOffset'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ValidityPeriodType, qt_meta_tag_ZN10KItinerary3Fcb3v1318ValidityPeriodTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ValidityPeriodType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318ValidityPeriodTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318ValidityPeriodTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318ValidityPeriodTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ValidityPeriodType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ValidityPeriodType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->validFromTime; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->validFromTimeIsSet(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->validFromUTCOffset; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->validFromUTCOffsetIsSet(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 6: *reinterpret_cast<int*>(_v) = _t->validUntilTime; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->validUntilTimeIsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->validUntilUTCOffset; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->validUntilUTCOffsetIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1324ValidityPeriodDetailTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ValidityPeriodDetailType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1324ValidityPeriodDetailTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ValidityPeriodDetailType",
        "validityPeriod",
        "QList<KItinerary::Fcb::v13::ValidityPeriodDetailType>",
        "validityPeriodIsSet",
        "excludedTimeRange",
        "QList<KItinerary::Fcb::v13::TimeRangeType>",
        "excludedTimeRangeIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'validityPeriod'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::ValidityPeriodDetailType>>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validityPeriodIsSet'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'excludedTimeRange'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TimeRangeType>>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'excludedTimeRangeIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ValidityPeriodDetailType, qt_meta_tag_ZN10KItinerary3Fcb3v1324ValidityPeriodDetailTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ValidityPeriodDetailType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1324ValidityPeriodDetailTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1324ValidityPeriodDetailTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1324ValidityPeriodDetailTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ValidityPeriodDetailType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ValidityPeriodDetailType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<KItinerary::Fcb::v13::ValidityPeriodDetailType>*>(_v) = _t->validityPeriod; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->validityPeriodIsSet(); break;
        case 2: *reinterpret_cast<QList<KItinerary::Fcb::v13::TimeRangeType>*>(_v) = _t->excludedTimeRange; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->excludedTimeRangeIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v138PassDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::PassData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v138PassDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::PassData",
        "validFrom",
        "",
        "issueingDateTime",
        "validUntil",
        "referenceNum",
        "referenceNumIsSet",
        "referenceIA5",
        "referenceIA5IsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "passType",
        "passTypeIsSet",
        "passDescription",
        "passDescriptionIsSet",
        "classCode",
        "KItinerary::Fcb::v13::TravelClassType",
        "validFromDay",
        "validFromTime",
        "validFromTimeIsSet",
        "validFromUTCOffset",
        "validFromUTCOffsetIsSet",
        "validUntilDay",
        "validUntilTime",
        "validUntilTimeIsSet",
        "validUntilUTCOffset",
        "validUntilUTCOffsetIsSet",
        "validityPeriodDetails",
        "KItinerary::Fcb::v13::ValidityPeriodDetailType",
        "validityPeriodDetailsIsSet",
        "numberOfValidityDays",
        "numberOfValidityDaysIsSet",
        "numberOfPossibleTrips",
        "numberOfPossibleTripsIsSet",
        "numberOfDaysOfTravel",
        "numberOfDaysOfTravelIsSet",
        "activatedDay",
        "QList<int>",
        "activatedDayIsSet",
        "countries",
        "countriesIsSet",
        "includedCarrierNum",
        "includedCarrierNumIsSet",
        "includedCarrierIA5",
        "QList<QByteArray>",
        "includedCarrierIA5IsSet",
        "excludedCarrierNum",
        "excludedCarrierNumIsSet",
        "excludedCarrierIA5",
        "excludedCarrierIA5IsSet",
        "includedServiceBrands",
        "includedServiceBrandsIsSet",
        "excludedServiceBrands",
        "excludedServiceBrandsIsSet",
        "validRegion",
        "QList<KItinerary::Fcb::v13::RegionalValidityType>",
        "validRegionIsSet",
        "tariffs",
        "QList<KItinerary::Fcb::v13::TariffType>",
        "tariffsIsSet",
        "price",
        "priceIsSet",
        "vatDetail",
        "QList<KItinerary::Fcb::v13::VatDetailType>",
        "vatDetailIsSet",
        "infoText",
        "infoTextIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'validFrom'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(1, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
        // Method 'validUntil'
        QtMocHelpers::MethodData<QDateTime(const QDateTime &) const>(4, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { QMetaType::QDateTime, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(15, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'passType'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'passTypeIsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'passDescription'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'passDescriptionIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'classCode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelClassType>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTime'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTimeIsSet'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromUTCOffset'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTime'
        QtMocHelpers::PropertyData<int>(29, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTimeIsSet'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilUTCOffset'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validityPeriodDetails'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ValidityPeriodDetailType>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validityPeriodDetailsIsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberOfValidityDays'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numberOfValidityDaysIsSet'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberOfPossibleTrips'
        QtMocHelpers::PropertyData<int>(38, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numberOfPossibleTripsIsSet'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberOfDaysOfTravel'
        QtMocHelpers::PropertyData<int>(40, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numberOfDaysOfTravelIsSet'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'activatedDay'
        QtMocHelpers::PropertyData<QList<int>>(42, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'activatedDayIsSet'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'countries'
        QtMocHelpers::PropertyData<QList<int>>(45, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'countriesIsSet'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedCarrierNum'
        QtMocHelpers::PropertyData<QList<int>>(47, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedCarrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedCarrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedCarrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'excludedCarrierNum'
        QtMocHelpers::PropertyData<QList<int>>(52, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'excludedCarrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'excludedCarrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(54, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'excludedCarrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(55, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(56, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'excludedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(58, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'excludedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validRegion'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::RegionalValidityType>>(60, 0x80000000 | 61, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validRegionIsSet'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tariffs'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TariffType>>(63, 0x80000000 | 64, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'tariffsIsSet'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'price'
        QtMocHelpers::PropertyData<int>(66, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'priceIsSet'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'vatDetail'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::VatDetailType>>(68, 0x80000000 | 69, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'vatDetailIsSet'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(71, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(73, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PassData, qt_meta_tag_ZN10KItinerary3Fcb3v138PassDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v138PassDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::PassData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v138PassDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v138PassDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v138PassDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v138PassDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::PassData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<PassData *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDateTime _r = _t->validFrom((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDateTime _r = _t->validUntil((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 45:
        case 41:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 49:
        case 47:
        case 43:
        case 39:
        case 37:
        case 35:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->passType; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->passTypeIsSet(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->passDescription; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->passDescriptionIsSet(); break;
        case 16: *reinterpret_cast<KItinerary::Fcb::v13::TravelClassType*>(_v) = _t->classCode; break;
        case 17: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 18: *reinterpret_cast<int*>(_v) = _t->validFromTime; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->validFromTimeIsSet(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->validFromUTCOffset; break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->validFromUTCOffsetIsSet(); break;
        case 22: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 23: *reinterpret_cast<int*>(_v) = _t->validUntilTime; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->validUntilTimeIsSet(); break;
        case 25: *reinterpret_cast<int*>(_v) = _t->validUntilUTCOffset; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->validUntilUTCOffsetIsSet(); break;
        case 27: *reinterpret_cast<KItinerary::Fcb::v13::ValidityPeriodDetailType*>(_v) = _t->validityPeriodDetails; break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->validityPeriodDetailsIsSet(); break;
        case 29: *reinterpret_cast<int*>(_v) = _t->numberOfValidityDays; break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->numberOfValidityDaysIsSet(); break;
        case 31: *reinterpret_cast<int*>(_v) = _t->numberOfPossibleTrips; break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->numberOfPossibleTripsIsSet(); break;
        case 33: *reinterpret_cast<int*>(_v) = _t->numberOfDaysOfTravel; break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->numberOfDaysOfTravelIsSet(); break;
        case 35: *reinterpret_cast<QList<int>*>(_v) = _t->activatedDay; break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->activatedDayIsSet(); break;
        case 37: *reinterpret_cast<QList<int>*>(_v) = _t->countries; break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->countriesIsSet(); break;
        case 39: *reinterpret_cast<QList<int>*>(_v) = _t->includedCarrierNum; break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->includedCarrierNumIsSet(); break;
        case 41: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->includedCarrierIA5; break;
        case 42: *reinterpret_cast<bool*>(_v) = _t->includedCarrierIA5IsSet(); break;
        case 43: *reinterpret_cast<QList<int>*>(_v) = _t->excludedCarrierNum; break;
        case 44: *reinterpret_cast<bool*>(_v) = _t->excludedCarrierNumIsSet(); break;
        case 45: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->excludedCarrierIA5; break;
        case 46: *reinterpret_cast<bool*>(_v) = _t->excludedCarrierIA5IsSet(); break;
        case 47: *reinterpret_cast<QList<int>*>(_v) = _t->includedServiceBrands; break;
        case 48: *reinterpret_cast<bool*>(_v) = _t->includedServiceBrandsIsSet(); break;
        case 49: *reinterpret_cast<QList<int>*>(_v) = _t->excludedServiceBrands; break;
        case 50: *reinterpret_cast<bool*>(_v) = _t->excludedServiceBrandsIsSet(); break;
        case 51: *reinterpret_cast<QList<KItinerary::Fcb::v13::RegionalValidityType>*>(_v) = _t->validRegion; break;
        case 52: *reinterpret_cast<bool*>(_v) = _t->validRegionIsSet(); break;
        case 53: *reinterpret_cast<QList<KItinerary::Fcb::v13::TariffType>*>(_v) = _t->tariffs; break;
        case 54: *reinterpret_cast<bool*>(_v) = _t->tariffsIsSet(); break;
        case 55: *reinterpret_cast<int*>(_v) = _t->price; break;
        case 56: *reinterpret_cast<bool*>(_v) = _t->priceIsSet(); break;
        case 57: *reinterpret_cast<QList<KItinerary::Fcb::v13::VatDetailType>*>(_v) = _t->vatDetail; break;
        case 58: *reinterpret_cast<bool*>(_v) = _t->vatDetailIsSet(); break;
        case 59: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 60: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 61: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 62: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1311VoucherDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::VoucherData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1311VoucherDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::VoucherData",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "validFromYear",
        "validFromDay",
        "validUntilYear",
        "validUntilDay",
        "value",
        "type",
        "typeIsSet",
        "infoText",
        "infoTextIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromYear'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilYear'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'value'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'type'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'typeIsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VoucherData, qt_meta_tag_ZN10KItinerary3Fcb3v1311VoucherDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1311VoucherDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::VoucherData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311VoucherDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311VoucherDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1311VoucherDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311VoucherDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::VoucherData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<VoucherData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->validFromYear; break;
        case 13: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 14: *reinterpret_cast<int*>(_v) = _t->validUntilYear; break;
        case 15: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 16: *reinterpret_cast<int*>(_v) = _t->value; break;
        case 17: *reinterpret_cast<int*>(_v) = _t->type; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->typeIsSet(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 21: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1316CustomerCardDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::CustomerCardData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1316CustomerCardDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::CustomerCardData",
        "customer",
        "KItinerary::Fcb::v13::TravelerType",
        "customerIsSet",
        "cardIdIA5",
        "cardIdIA5IsSet",
        "cardIdNum",
        "cardIdNumIsSet",
        "validFromYear",
        "validFromDay",
        "validFromDayIsSet",
        "validUntilYear",
        "validUntilDay",
        "validUntilDayIsSet",
        "classCode",
        "KItinerary::Fcb::v13::TravelClassType",
        "classCodeIsSet",
        "cardType",
        "cardTypeIsSet",
        "cardTypeDescr",
        "cardTypeDescrIsSet",
        "customerStatus",
        "customerStatusIsSet",
        "customerStatusDescr",
        "customerStatusDescrIsSet",
        "includedServices",
        "QList<int>",
        "includedServicesIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet",
        "validFrom",
        "validUntil"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'customer'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelerType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'customerIsSet'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(4, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardIdNum'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromYear'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromDayIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilYear'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilDayIsSet'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'classCode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelClassType>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'classCodeIsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardType'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardTypeIsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'cardTypeDescr'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cardTypeDescrIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'customerStatus'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'customerStatusIsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'customerStatusDescr'
        QtMocHelpers::PropertyData<QByteArray>(23, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'customerStatusDescrIsSet'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedServices'
        QtMocHelpers::PropertyData<QList<int>>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedServicesIsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFrom'
        QtMocHelpers::PropertyData<QDate>(31, QMetaType::QDate, QMC::DefaultPropertyFlags),
        // property 'validUntil'
        QtMocHelpers::PropertyData<QDate>(32, QMetaType::QDate, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomerCardData, qt_meta_tag_ZN10KItinerary3Fcb3v1316CustomerCardDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1316CustomerCardDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::CustomerCardData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316CustomerCardDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316CustomerCardDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1316CustomerCardDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1316CustomerCardDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::CustomerCardData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CustomerCardData *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 22:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::TravelerType*>(_v) = _t->customer; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->customerIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->cardIdIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->cardIdIA5IsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->cardIdNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->cardIdNumIsSet(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->validFromYear; break;
        case 7: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->validFromDayIsSet(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->validUntilYear; break;
        case 10: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->validUntilDayIsSet(); break;
        case 12: *reinterpret_cast<KItinerary::Fcb::v13::TravelClassType*>(_v) = _t->classCode; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->classCodeIsSet(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->cardType; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->cardTypeIsSet(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->cardTypeDescr; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->cardTypeDescrIsSet(); break;
        case 18: *reinterpret_cast<int*>(_v) = _t->customerStatus; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->customerStatusIsSet(); break;
        case 20: *reinterpret_cast<QByteArray*>(_v) = _t->customerStatusDescr; break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->customerStatusDescrIsSet(); break;
        case 22: *reinterpret_cast<QList<int>*>(_v) = _t->includedServices; break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->includedServicesIsSet(); break;
        case 24: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        case 26: *reinterpret_cast<QDate*>(_v) = _t->validFrom(); break;
        case 27: *reinterpret_cast<QDate*>(_v) = _t->validUntil(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1315CountermarkDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::CountermarkData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1315CountermarkDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::CountermarkData",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "ticketReferenceIA5",
        "ticketReferenceIA5IsSet",
        "ticketReferenceNum",
        "ticketReferenceNumIsSet",
        "numberOfCountermark",
        "totalOfCountermarks",
        "groupName",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "fromStationNum",
        "fromStationNumIsSet",
        "fromStationIA5",
        "fromStationIA5IsSet",
        "toStationNum",
        "toStationNumIsSet",
        "toStationIA5",
        "toStationIA5IsSet",
        "fromStationNameUTF8",
        "fromStationNameUTF8IsSet",
        "toStationNameUTF8",
        "toStationNameUTF8IsSet",
        "validRegionDesc",
        "validRegionDescIsSet",
        "validRegion",
        "QList<KItinerary::Fcb::v13::RegionalValidityType>",
        "validRegionIsSet",
        "returnIncluded",
        "returnDescription",
        "KItinerary::Fcb::v13::ReturnRouteDescriptionType",
        "returnDescriptionIsSet",
        "validFromDay",
        "validFromTime",
        "validFromTimeIsSet",
        "validFromUTCOffset",
        "validFromUTCOffsetIsSet",
        "validUntilDay",
        "validUntilTime",
        "validUntilTimeIsSet",
        "validUntilUTCOffset",
        "validUntilUTCOffsetIsSet",
        "classCode",
        "KItinerary::Fcb::v13::TravelClassType",
        "carrierNum",
        "QList<int>",
        "carrierNumIsSet",
        "carrierIA5",
        "QList<QByteArray>",
        "carrierIA5IsSet",
        "includedServiceBrands",
        "includedServiceBrandsIsSet",
        "excludedServiceBrands",
        "excludedServiceBrandsIsSet",
        "infoText",
        "infoTextIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ticketReferenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(13, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'ticketReferenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ticketReferenceNum'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'ticketReferenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberOfCountermark'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'totalOfCountermarks'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'groupName'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'fromStationNum'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(24, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNum'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNumIsSet'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationIA5'
        QtMocHelpers::PropertyData<QByteArray>(28, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'fromStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'toStationNameUTF8'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'toStationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validRegionDesc'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validRegionDescIsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validRegion'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::RegionalValidityType>>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validRegionIsSet'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'returnIncluded'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'returnDescription'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ReturnRouteDescriptionType>(40, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'returnDescriptionIsSet'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(43, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTime'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTimeIsSet'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromUTCOffset'
        QtMocHelpers::PropertyData<int>(46, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(48, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTime'
        QtMocHelpers::PropertyData<int>(49, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTimeIsSet'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilUTCOffset'
        QtMocHelpers::PropertyData<int>(51, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'classCode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelClassType>(53, 0x80000000 | 54, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNum'
        QtMocHelpers::PropertyData<QList<int>>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(58, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(61, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'excludedServiceBrands'
        QtMocHelpers::PropertyData<QList<int>>(63, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'excludedServiceBrandsIsSet'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(65, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(67, 0x80000000 | 68, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CountermarkData, qt_meta_tag_ZN10KItinerary3Fcb3v1315CountermarkDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1315CountermarkDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::CountermarkData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315CountermarkDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315CountermarkDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1315CountermarkDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1315CountermarkDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::CountermarkData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<CountermarkData *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 52:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 56:
        case 54:
        case 50:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 12: *reinterpret_cast<QByteArray*>(_v) = _t->ticketReferenceIA5; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->ticketReferenceIA5IsSet(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->ticketReferenceNum; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->ticketReferenceNumIsSet(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->numberOfCountermark; break;
        case 17: *reinterpret_cast<int*>(_v) = _t->totalOfCountermarks; break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->groupName; break;
        case 19: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 20: *reinterpret_cast<int*>(_v) = _t->fromStationNum; break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->fromStationNumIsSet(); break;
        case 22: *reinterpret_cast<QByteArray*>(_v) = _t->fromStationIA5; break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->fromStationIA5IsSet(); break;
        case 24: *reinterpret_cast<int*>(_v) = _t->toStationNum; break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->toStationNumIsSet(); break;
        case 26: *reinterpret_cast<QByteArray*>(_v) = _t->toStationIA5; break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->toStationIA5IsSet(); break;
        case 28: *reinterpret_cast<QString*>(_v) = _t->fromStationNameUTF8; break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->fromStationNameUTF8IsSet(); break;
        case 30: *reinterpret_cast<QString*>(_v) = _t->toStationNameUTF8; break;
        case 31: *reinterpret_cast<bool*>(_v) = _t->toStationNameUTF8IsSet(); break;
        case 32: *reinterpret_cast<QString*>(_v) = _t->validRegionDesc; break;
        case 33: *reinterpret_cast<bool*>(_v) = _t->validRegionDescIsSet(); break;
        case 34: *reinterpret_cast<QList<KItinerary::Fcb::v13::RegionalValidityType>*>(_v) = _t->validRegion; break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->validRegionIsSet(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->returnIncluded; break;
        case 37: *reinterpret_cast<KItinerary::Fcb::v13::ReturnRouteDescriptionType*>(_v) = _t->returnDescription; break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->returnDescriptionIsSet(); break;
        case 39: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 40: *reinterpret_cast<int*>(_v) = _t->validFromTime; break;
        case 41: *reinterpret_cast<bool*>(_v) = _t->validFromTimeIsSet(); break;
        case 42: *reinterpret_cast<int*>(_v) = _t->validFromUTCOffset; break;
        case 43: *reinterpret_cast<bool*>(_v) = _t->validFromUTCOffsetIsSet(); break;
        case 44: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 45: *reinterpret_cast<int*>(_v) = _t->validUntilTime; break;
        case 46: *reinterpret_cast<bool*>(_v) = _t->validUntilTimeIsSet(); break;
        case 47: *reinterpret_cast<int*>(_v) = _t->validUntilUTCOffset; break;
        case 48: *reinterpret_cast<bool*>(_v) = _t->validUntilUTCOffsetIsSet(); break;
        case 49: *reinterpret_cast<KItinerary::Fcb::v13::TravelClassType*>(_v) = _t->classCode; break;
        case 50: *reinterpret_cast<QList<int>*>(_v) = _t->carrierNum; break;
        case 51: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 52: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->carrierIA5; break;
        case 53: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 54: *reinterpret_cast<QList<int>*>(_v) = _t->includedServiceBrands; break;
        case 55: *reinterpret_cast<bool*>(_v) = _t->includedServiceBrandsIsSet(); break;
        case 56: *reinterpret_cast<QList<int>*>(_v) = _t->excludedServiceBrands; break;
        case 57: *reinterpret_cast<bool*>(_v) = _t->excludedServiceBrandsIsSet(); break;
        case 58: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 59: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 60: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 61: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1317ParkingGroundDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ParkingGroundData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1317ParkingGroundDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ParkingGroundData",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "parkingGroundId",
        "fromParkingDate",
        "untilParkingDate",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "accessCode",
        "accessCodeIsSet",
        "location",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "stationNum",
        "stationNumIsSet",
        "stationIA5",
        "stationIA5IsSet",
        "specialInformation",
        "specialInformationIsSet",
        "entryTrack",
        "entryTrackIsSet",
        "numberPlate",
        "numberPlateIsSet",
        "price",
        "priceIsSet",
        "vatDetail",
        "QList<KItinerary::Fcb::v13::VatDetailType>",
        "vatDetailIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'parkingGroundId'
        QtMocHelpers::PropertyData<QByteArray>(5, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fromParkingDate'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'untilParkingDate'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(10, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(14, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'accessCode'
        QtMocHelpers::PropertyData<QByteArray>(16, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'accessCodeIsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'location'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stationNum'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationNumIsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationIA5'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'specialInformation'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'specialInformationIsSet'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'entryTrack'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'entryTrackIsSet'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberPlate'
        QtMocHelpers::PropertyData<QByteArray>(29, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numberPlateIsSet'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'price'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'priceIsSet'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'vatDetail'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::VatDetailType>>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'vatDetailIsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ParkingGroundData, qt_meta_tag_ZN10KItinerary3Fcb3v1317ParkingGroundDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1317ParkingGroundDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ParkingGroundData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317ParkingGroundDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317ParkingGroundDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1317ParkingGroundDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317ParkingGroundDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ParkingGroundData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ParkingGroundData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 4: *reinterpret_cast<QByteArray*>(_v) = _t->parkingGroundId; break;
        case 5: *reinterpret_cast<int*>(_v) = _t->fromParkingDate; break;
        case 6: *reinterpret_cast<int*>(_v) = _t->untilParkingDate; break;
        case 7: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 9: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 13: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 15: *reinterpret_cast<QByteArray*>(_v) = _t->accessCode; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->accessCodeIsSet(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->location; break;
        case 18: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 19: *reinterpret_cast<int*>(_v) = _t->stationNum; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->stationNumIsSet(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->stationIA5; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->stationIA5IsSet(); break;
        case 23: *reinterpret_cast<QString*>(_v) = _t->specialInformation; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->specialInformationIsSet(); break;
        case 25: *reinterpret_cast<QString*>(_v) = _t->entryTrack; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->entryTrackIsSet(); break;
        case 27: *reinterpret_cast<QByteArray*>(_v) = _t->numberPlate; break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->numberPlateIsSet(); break;
        case 29: *reinterpret_cast<int*>(_v) = _t->price; break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->priceIsSet(); break;
        case 31: *reinterpret_cast<QList<KItinerary::Fcb::v13::VatDetailType>*>(_v) = _t->vatDetail; break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->vatDetailIsSet(); break;
        case 33: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1313FIPTicketDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::FIPTicketData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1313FIPTicketDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::FIPTicketData",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "validFromDay",
        "validUntilDay",
        "activatedDay",
        "QList<int>",
        "activatedDayIsSet",
        "carrierNum",
        "carrierNumIsSet",
        "carrierIA5",
        "QList<QByteArray>",
        "carrierIA5IsSet",
        "numberOfTravelDays",
        "includesSupplements",
        "classCode",
        "KItinerary::Fcb::v13::TravelClassType",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'activatedDay'
        QtMocHelpers::PropertyData<QList<int>>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'activatedDayIsSet'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierNum'
        QtMocHelpers::PropertyData<QList<int>>(18, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierNumIsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'carrierIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'carrierIA5IsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberOfTravelDays'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'includesSupplements'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'classCode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelClassType>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FIPTicketData, qt_meta_tag_ZN10KItinerary3Fcb3v1313FIPTicketDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1313FIPTicketDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::FIPTicketData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313FIPTicketDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313FIPTicketDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1313FIPTicketDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1313FIPTicketDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::FIPTicketData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<FIPTicketData *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 18:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 16:
        case 14:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 13: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 14: *reinterpret_cast<QList<int>*>(_v) = _t->activatedDay; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->activatedDayIsSet(); break;
        case 16: *reinterpret_cast<QList<int>*>(_v) = _t->carrierNum; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->carrierNumIsSet(); break;
        case 18: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->carrierIA5; break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->carrierIA5IsSet(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->numberOfTravelDays; break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->includesSupplements; break;
        case 22: *reinterpret_cast<KItinerary::Fcb::v13::TravelClassType*>(_v) = _t->classCode; break;
        case 23: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1318StationPassageDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::StationPassageData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1318StationPassageDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::StationPassageData",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "productIdNum",
        "productIdNumIsSet",
        "productIdIA5",
        "productIdIA5IsSet",
        "productName",
        "productNameIsSet",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "stationNum",
        "QList<int>",
        "stationNumIsSet",
        "stationIA5",
        "QList<QByteArray>",
        "stationIA5IsSet",
        "stationNameUTF8",
        "stationNameUTF8IsSet",
        "areaCodeNum",
        "areaCodeNumIsSet",
        "areaCodeIA5",
        "areaCodeIA5IsSet",
        "areaNameUTF8",
        "areaNameUTF8IsSet",
        "validFromDay",
        "validFromTime",
        "validFromTimeIsSet",
        "validFromUTCOffset",
        "validFromUTCOffsetIsSet",
        "validUntilDay",
        "validUntilTime",
        "validUntilTimeIsSet",
        "validUntilUTCOffset",
        "validUntilUTCOffsetIsSet",
        "numberOfDaysValid",
        "numberOfDaysValidIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdNum'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdNumIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productIdIA5'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productIdIA5IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productName'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productNameIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stationNum'
        QtMocHelpers::PropertyData<QList<int>>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stationNumIsSet'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationNameUTF8'
        QtMocHelpers::PropertyData<QList<QString>>(23, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'areaCodeNum'
        QtMocHelpers::PropertyData<QList<int>>(25, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'areaCodeNumIsSet'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'areaCodeIA5'
        QtMocHelpers::PropertyData<QList<QByteArray>>(27, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'areaCodeIA5IsSet'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'areaNameUTF8'
        QtMocHelpers::PropertyData<QList<QString>>(29, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'areaNameUTF8IsSet'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromDay'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTime'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromTimeIsSet'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validFromUTCOffset'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFromUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilDay'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTime'
        QtMocHelpers::PropertyData<int>(37, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilTimeIsSet'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'validUntilUTCOffset'
        QtMocHelpers::PropertyData<int>(39, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntilUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'numberOfDaysValid'
        QtMocHelpers::PropertyData<int>(41, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numberOfDaysValidIsSet'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StationPassageData, qt_meta_tag_ZN10KItinerary3Fcb3v1318StationPassageDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1318StationPassageDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::StationPassageData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318StationPassageDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318StationPassageDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1318StationPassageDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1318StationPassageDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::StationPassageData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<StationPassageData *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 23:
        case 17:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QByteArray> >(); break;
        case 21:
        case 15:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->productIdNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->productIdNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->productIdIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->productIdIA5IsSet(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->productName; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->productNameIsSet(); break;
        case 14: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 15: *reinterpret_cast<QList<int>*>(_v) = _t->stationNum; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->stationNumIsSet(); break;
        case 17: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->stationIA5; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->stationIA5IsSet(); break;
        case 19: *reinterpret_cast<QList<QString>*>(_v) = _t->stationNameUTF8; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->stationNameUTF8IsSet(); break;
        case 21: *reinterpret_cast<QList<int>*>(_v) = _t->areaCodeNum; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->areaCodeNumIsSet(); break;
        case 23: *reinterpret_cast<QList<QByteArray>*>(_v) = _t->areaCodeIA5; break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->areaCodeIA5IsSet(); break;
        case 25: *reinterpret_cast<QList<QString>*>(_v) = _t->areaNameUTF8; break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->areaNameUTF8IsSet(); break;
        case 27: *reinterpret_cast<int*>(_v) = _t->validFromDay; break;
        case 28: *reinterpret_cast<int*>(_v) = _t->validFromTime; break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->validFromTimeIsSet(); break;
        case 30: *reinterpret_cast<int*>(_v) = _t->validFromUTCOffset; break;
        case 31: *reinterpret_cast<bool*>(_v) = _t->validFromUTCOffsetIsSet(); break;
        case 32: *reinterpret_cast<int*>(_v) = _t->validUntilDay; break;
        case 33: *reinterpret_cast<int*>(_v) = _t->validUntilTime; break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->validUntilTimeIsSet(); break;
        case 35: *reinterpret_cast<int*>(_v) = _t->validUntilUTCOffset; break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->validUntilUTCOffsetIsSet(); break;
        case 37: *reinterpret_cast<int*>(_v) = _t->numberOfDaysValid; break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->numberOfDaysValidIsSet(); break;
        case 39: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1314TicketLinkTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::TicketLinkType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1314TicketLinkTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::TicketLinkType",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "issuerName",
        "issuerNameIsSet",
        "issuerPNR",
        "issuerPNRIsSet",
        "productOwnerNum",
        "productOwnerNumIsSet",
        "productOwnerIA5",
        "productOwnerIA5IsSet",
        "ticketType",
        "KItinerary::Fcb::v13::TicketType",
        "linkMode",
        "KItinerary::Fcb::v13::LinkMode"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuerName'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerNameIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'issuerPNR'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'issuerPNRIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerNum'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerNumIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'productOwnerIA5'
        QtMocHelpers::PropertyData<QByteArray>(11, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'productOwnerIA5IsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ticketType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TicketType>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'linkMode'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::LinkMode>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TicketLinkType, qt_meta_tag_ZN10KItinerary3Fcb3v1314TicketLinkTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1314TicketLinkTypeE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::TicketLinkType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314TicketLinkTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314TicketLinkTypeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1314TicketLinkTypeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1314TicketLinkTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::TicketLinkType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TicketLinkType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->issuerName; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->issuerNameIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->issuerPNR; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->issuerPNRIsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->productOwnerNum; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->productOwnerNumIsSet(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->productOwnerIA5; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->productOwnerIA5IsSet(); break;
        case 12: *reinterpret_cast<KItinerary::Fcb::v13::TicketType*>(_v) = _t->ticketType; break;
        case 13: *reinterpret_cast<KItinerary::Fcb::v13::LinkMode*>(_v) = _t->linkMode; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1317DelayConfirmationE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::DelayConfirmation::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1317DelayConfirmationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::DelayConfirmation",
        "referenceIA5",
        "referenceIA5IsSet",
        "referenceNum",
        "referenceNumIsSet",
        "trainNum",
        "trainNumIsSet",
        "trainIA5",
        "trainIA5IsSet",
        "departureYear",
        "departureYearIsSet",
        "departureDay",
        "departureDayIsSet",
        "departureTime",
        "departureTimeIsSet",
        "departureUTCOffset",
        "departureUTCOffsetIsSet",
        "stationCodeTable",
        "KItinerary::Fcb::v13::CodeTableType",
        "stationNum",
        "stationNumIsSet",
        "stationIA5",
        "stationIA5IsSet",
        "delay",
        "trainCancelled",
        "confirmationType",
        "KItinerary::Fcb::v13::ConfirmationType",
        "affectedTickets",
        "QList<KItinerary::Fcb::v13::TicketLinkType>",
        "affectedTicketsIsSet",
        "infoText",
        "infoTextIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'referenceIA5'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceIA5IsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'referenceNum'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'referenceNumIsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trainNum'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainNumIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'trainIA5'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainIA5IsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'departureYear'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureYearIsSet'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'departureDay'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureDayIsSet'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'departureTime'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureTimeIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'departureUTCOffset'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'departureUTCOffsetIsSet'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationCodeTable'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::CodeTableType>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'stationNum'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationNumIsSet'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'stationIA5'
        QtMocHelpers::PropertyData<QByteArray>(21, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stationIA5IsSet'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'delay'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trainCancelled'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'confirmationType'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ConfirmationType>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'affectedTickets'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TicketLinkType>>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'affectedTicketsIsSet'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DelayConfirmation, qt_meta_tag_ZN10KItinerary3Fcb3v1317DelayConfirmationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1317DelayConfirmationE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::DelayConfirmation::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317DelayConfirmationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317DelayConfirmationE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1317DelayConfirmationE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317DelayConfirmationE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::DelayConfirmation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<DelayConfirmation *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->referenceIA5; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->referenceIA5IsSet(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->referenceNum; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->referenceNumIsSet(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->trainNum; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->trainNumIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->trainIA5; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->trainIA5IsSet(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->departureYear; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->departureYearIsSet(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->departureDay; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->departureDayIsSet(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->departureTime; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->departureTimeIsSet(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->departureUTCOffset; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->departureUTCOffsetIsSet(); break;
        case 16: *reinterpret_cast<KItinerary::Fcb::v13::CodeTableType*>(_v) = _t->stationCodeTable; break;
        case 17: *reinterpret_cast<int*>(_v) = _t->stationNum; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->stationNumIsSet(); break;
        case 19: *reinterpret_cast<QByteArray*>(_v) = _t->stationIA5; break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->stationIA5IsSet(); break;
        case 21: *reinterpret_cast<int*>(_v) = _t->delay; break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->trainCancelled; break;
        case 23: *reinterpret_cast<KItinerary::Fcb::v13::ConfirmationType*>(_v) = _t->confirmationType; break;
        case 24: *reinterpret_cast<QList<KItinerary::Fcb::v13::TicketLinkType>*>(_v) = _t->affectedTickets; break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->affectedTicketsIsSet(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 28: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v139TokenTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::TokenType::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v139TokenTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::TokenType",
        "tokenProviderNum",
        "tokenProviderNumIsSet",
        "tokenProviderIA5",
        "tokenProviderIA5IsSet",
        "tokenSpecification",
        "tokenSpecificationIsSet",
        "token"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'tokenProviderNum'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tokenProviderNumIsSet'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tokenProviderIA5'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tokenProviderIA5IsSet'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'tokenSpecification'
        QtMocHelpers::PropertyData<QByteArray>(5, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tokenSpecificationIsSet'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'token'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TokenType, qt_meta_tag_ZN10KItinerary3Fcb3v139TokenTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::TokenType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v139TokenTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v139TokenTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v139TokenTypeE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::TokenType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TokenType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->tokenProviderNum; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->tokenProviderNumIsSet(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->tokenProviderIA5; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->tokenProviderIA5IsSet(); break;
        case 4: *reinterpret_cast<QByteArray*>(_v) = _t->tokenSpecification; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->tokenSpecificationIsSet(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->token; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1312DocumentDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::DocumentData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1312DocumentDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::DocumentData",
        "token",
        "KItinerary::Fcb::v13::TokenType",
        "tokenIsSet",
        "ticket",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'token'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TokenType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'tokenIsSet'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ticket'
        QtMocHelpers::PropertyData<QVariant>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DocumentData, qt_meta_tag_ZN10KItinerary3Fcb3v1312DocumentDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1312DocumentDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::DocumentData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312DocumentDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312DocumentDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1312DocumentDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1312DocumentDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::DocumentData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<DocumentData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::TokenType*>(_v) = _t->token; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->tokenIsSet(); break;
        case 2: *reinterpret_cast<QVariant*>(_v) = _t->ticketVariant(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1311ControlDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::ControlData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1311ControlDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::ControlData",
        "identificationByCardReference",
        "QList<KItinerary::Fcb::v13::CardReferenceType>",
        "identificationByCardReferenceIsSet",
        "identificationByIdCard",
        "identificationByPassportId",
        "identificationItem",
        "identificationItemIsSet",
        "passportValidationRequired",
        "onlineValidationRequired",
        "randomDetailedValidationRequired",
        "randomDetailedValidationRequiredIsSet",
        "ageCheckRequired",
        "reductionCardCheckRequired",
        "infoText",
        "infoTextIsSet",
        "includedTickets",
        "QList<KItinerary::Fcb::v13::TicketLinkType>",
        "includedTicketsIsSet",
        "extension",
        "KItinerary::Fcb::v13::ExtensionData",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'identificationByCardReference'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::CardReferenceType>>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'identificationByCardReferenceIsSet'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'identificationByIdCard'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'identificationByPassportId'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'identificationItem'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'identificationItemIsSet'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'passportValidationRequired'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'onlineValidationRequired'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'randomDetailedValidationRequired'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'randomDetailedValidationRequiredIsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ageCheckRequired'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'reductionCardCheckRequired'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoText'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'infoTextIsSet'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'includedTickets'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::TicketLinkType>>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'includedTicketsIsSet'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ExtensionData>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ControlData, qt_meta_tag_ZN10KItinerary3Fcb3v1311ControlDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1311ControlDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::ControlData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311ControlDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311ControlDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1311ControlDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1311ControlDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::ControlData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ControlData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<KItinerary::Fcb::v13::CardReferenceType>*>(_v) = _t->identificationByCardReference; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->identificationByCardReferenceIsSet(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->identificationByIdCard; break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->identificationByPassportId; break;
        case 4: *reinterpret_cast<int*>(_v) = _t->identificationItem; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->identificationItemIsSet(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->passportValidationRequired; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->onlineValidationRequired; break;
        case 8: *reinterpret_cast<int*>(_v) = _t->randomDetailedValidationRequired; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->randomDetailedValidationRequiredIsSet(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->ageCheckRequired; break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->reductionCardCheckRequired; break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->infoText; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->infoTextIsSet(); break;
        case 14: *reinterpret_cast<QList<KItinerary::Fcb::v13::TicketLinkType>*>(_v) = _t->includedTickets; break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->includedTicketsIsSet(); break;
        case 16: *reinterpret_cast<KItinerary::Fcb::v13::ExtensionData*>(_v) = _t->extension; break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v1317UicRailTicketDataE_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::UicRailTicketData::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v1317UicRailTicketDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13::UicRailTicketData",
        "issuingDetail",
        "KItinerary::Fcb::v13::IssuingData",
        "travelerDetail",
        "KItinerary::Fcb::v13::TravelerData",
        "travelerDetailIsSet",
        "transportDocument",
        "QList<KItinerary::Fcb::v13::DocumentData>",
        "transportDocumentIsSet",
        "controlDetail",
        "KItinerary::Fcb::v13::ControlData",
        "controlDetailIsSet",
        "extension",
        "QList<KItinerary::Fcb::v13::ExtensionData>",
        "extensionIsSet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'issuingDetail'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::IssuingData>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'travelerDetail'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::TravelerData>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'travelerDetailIsSet'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'transportDocument'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::DocumentData>>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'transportDocumentIsSet'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'controlDetail'
        QtMocHelpers::PropertyData<KItinerary::Fcb::v13::ControlData>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'controlDetailIsSet'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'extension'
        QtMocHelpers::PropertyData<QList<KItinerary::Fcb::v13::ExtensionData>>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extensionIsSet'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UicRailTicketData, qt_meta_tag_ZN10KItinerary3Fcb3v1317UicRailTicketDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10KItinerary3Fcb3v1317UicRailTicketDataE[] = {
    QMetaObject::SuperData::link<KItinerary::Fcb::v13::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::UicRailTicketData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317UicRailTicketDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317UicRailTicketDataE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10KItinerary3Fcb3v1317UicRailTicketDataE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KItinerary3Fcb3v1317UicRailTicketDataE_t>.metaTypes,
    nullptr
} };

void KItinerary::Fcb::v13::UicRailTicketData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<UicRailTicketData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KItinerary::Fcb::v13::IssuingData*>(_v) = _t->issuingDetail; break;
        case 1: *reinterpret_cast<KItinerary::Fcb::v13::TravelerData*>(_v) = _t->travelerDetail; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->travelerDetailIsSet(); break;
        case 3: *reinterpret_cast<QList<KItinerary::Fcb::v13::DocumentData>*>(_v) = _t->transportDocument; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->transportDocumentIsSet(); break;
        case 5: *reinterpret_cast<KItinerary::Fcb::v13::ControlData*>(_v) = _t->controlDetail; break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->controlDetailIsSet(); break;
        case 7: *reinterpret_cast<QList<KItinerary::Fcb::v13::ExtensionData>*>(_v) = _t->extension; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->extensionIsSet(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10KItinerary3Fcb3v13E_t {};
} // unnamed namespace

template <> constexpr inline auto KItinerary::Fcb::v13::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v13E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItinerary::Fcb::v13",
        "GeoUnitType",
        "microDegree",
        "tenthmilliDegree",
        "milliDegree",
        "centiDegree",
        "deciDegree",
        "GeoCoordinateSystemType",
        "wgs84",
        "grs80",
        "HemisphereLongitudeType",
        "north",
        "south",
        "HemisphereLatitudeType",
        "east",
        "west",
        "GenderType",
        "unspecified",
        "female",
        "male",
        "other",
        "PassengerType",
        "adult",
        "senior",
        "child",
        "youth",
        "dog",
        "bicycle",
        "freeAddonPassenger",
        "freeAddonChild",
        "CodeTableType",
        "stationUIC",
        "stationUICReservation",
        "stationERA",
        "localCarrierStationCodeTable",
        "proprietaryIssuerStationCodeTable",
        "TravelClassType",
        "notApplicable",
        "first",
        "second",
        "tourist",
        "comfort",
        "premium",
        "business",
        "all",
        "ServiceType",
        "seat",
        "couchette",
        "berth",
        "carcarriage",
        "CompartmentPositionType",
        "upperLevel",
        "lowerLevel",
        "BerthTypeType",
        "single",
        "special",
        "doubleBerth",
        "t2",
        "t3",
        "t4",
        "CompartmentGenderType",
        "family",
        "mixed",
        "PriceTypeType",
        "noPrice",
        "reservationFee",
        "supplement",
        "travelPrice",
        "RoofRackType",
        "norack",
        "roofRailing",
        "luggageRack",
        "skiRack",
        "boxRack",
        "rackWithOneBox",
        "rackWithTwoBoxes",
        "bicycleRack",
        "otherRack",
        "LoadingDeckType",
        "upper",
        "lower",
        "ConfirmationType",
        "trainDelayConfirmation",
        "travelerDelayConfirmation",
        "trainLinkedTicketDelay",
        "TicketType",
        "openTicket",
        "pass",
        "reservation",
        "carCarriageReservation",
        "LinkMode",
        "issuedTogether",
        "onlyValidInCombination"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GeoUnitType'
        QtMocHelpers::EnumData<GeoUnitType>(1, 1, QMC::EnumFlags{}).add({
            {    2, GeoUnitType::microDegree },
            {    3, GeoUnitType::tenthmilliDegree },
            {    4, GeoUnitType::milliDegree },
            {    5, GeoUnitType::centiDegree },
            {    6, GeoUnitType::deciDegree },
        }),
        // enum 'GeoCoordinateSystemType'
        QtMocHelpers::EnumData<GeoCoordinateSystemType>(7, 7, QMC::EnumFlags{}).add({
            {    8, GeoCoordinateSystemType::wgs84 },
            {    9, GeoCoordinateSystemType::grs80 },
        }),
        // enum 'HemisphereLongitudeType'
        QtMocHelpers::EnumData<HemisphereLongitudeType>(10, 10, QMC::EnumFlags{}).add({
            {   11, HemisphereLongitudeType::north },
            {   12, HemisphereLongitudeType::south },
        }),
        // enum 'HemisphereLatitudeType'
        QtMocHelpers::EnumData<HemisphereLatitudeType>(13, 13, QMC::EnumFlags{}).add({
            {   14, HemisphereLatitudeType::east },
            {   15, HemisphereLatitudeType::west },
        }),
        // enum 'GenderType'
        QtMocHelpers::EnumData<GenderType>(16, 16, QMC::EnumFlags{}).add({
            {   17, GenderType::unspecified },
            {   18, GenderType::female },
            {   19, GenderType::male },
            {   20, GenderType::other },
        }),
        // enum 'PassengerType'
        QtMocHelpers::EnumData<PassengerType>(21, 21, QMC::EnumFlags{}).add({
            {   22, PassengerType::adult },
            {   23, PassengerType::senior },
            {   24, PassengerType::child },
            {   25, PassengerType::youth },
            {   26, PassengerType::dog },
            {   27, PassengerType::bicycle },
            {   28, PassengerType::freeAddonPassenger },
            {   29, PassengerType::freeAddonChild },
        }),
        // enum 'CodeTableType'
        QtMocHelpers::EnumData<CodeTableType>(30, 30, QMC::EnumFlags{}).add({
            {   31, CodeTableType::stationUIC },
            {   32, CodeTableType::stationUICReservation },
            {   33, CodeTableType::stationERA },
            {   34, CodeTableType::localCarrierStationCodeTable },
            {   35, CodeTableType::proprietaryIssuerStationCodeTable },
        }),
        // enum 'TravelClassType'
        QtMocHelpers::EnumData<TravelClassType>(36, 36, QMC::EnumFlags{}).add({
            {   37, TravelClassType::notApplicable },
            {   38, TravelClassType::first },
            {   39, TravelClassType::second },
            {   40, TravelClassType::tourist },
            {   41, TravelClassType::comfort },
            {   42, TravelClassType::premium },
            {   43, TravelClassType::business },
            {   44, TravelClassType::all },
        }),
        // enum 'ServiceType'
        QtMocHelpers::EnumData<ServiceType>(45, 45, QMC::EnumFlags{}).add({
            {   46, ServiceType::seat },
            {   47, ServiceType::couchette },
            {   48, ServiceType::berth },
            {   49, ServiceType::carcarriage },
        }),
        // enum 'CompartmentPositionType'
        QtMocHelpers::EnumData<CompartmentPositionType>(50, 50, QMC::EnumIsScoped).add({
            {   17, CompartmentPositionType::unspecified },
            {   51, CompartmentPositionType::upperLevel },
            {   52, CompartmentPositionType::lowerLevel },
        }),
        // enum 'BerthTypeType'
        QtMocHelpers::EnumData<BerthTypeType>(53, 53, QMC::EnumFlags{}).add({
            {   54, BerthTypeType::single },
            {   55, BerthTypeType::special },
            {   56, BerthTypeType::doubleBerth },
            {   57, BerthTypeType::t2 },
            {   58, BerthTypeType::t3 },
            {   59, BerthTypeType::t4 },
        }),
        // enum 'CompartmentGenderType'
        QtMocHelpers::EnumData<CompartmentGenderType>(60, 60, QMC::EnumIsScoped).add({
            {   17, CompartmentGenderType::unspecified },
            {   61, CompartmentGenderType::family },
            {   18, CompartmentGenderType::female },
            {   19, CompartmentGenderType::male },
            {   62, CompartmentGenderType::mixed },
        }),
        // enum 'PriceTypeType'
        QtMocHelpers::EnumData<PriceTypeType>(63, 63, QMC::EnumFlags{}).add({
            {   64, PriceTypeType::noPrice },
            {   65, PriceTypeType::reservationFee },
            {   66, PriceTypeType::supplement },
            {   67, PriceTypeType::travelPrice },
        }),
        // enum 'RoofRackType'
        QtMocHelpers::EnumData<RoofRackType>(68, 68, QMC::EnumFlags{}).add({
            {   69, RoofRackType::norack },
            {   70, RoofRackType::roofRailing },
            {   71, RoofRackType::luggageRack },
            {   72, RoofRackType::skiRack },
            {   73, RoofRackType::boxRack },
            {   74, RoofRackType::rackWithOneBox },
            {   75, RoofRackType::rackWithTwoBoxes },
            {   76, RoofRackType::bicycleRack },
            {   77, RoofRackType::otherRack },
        }),
        // enum 'LoadingDeckType'
        QtMocHelpers::EnumData<LoadingDeckType>(78, 78, QMC::EnumIsScoped).add({
            {   17, LoadingDeckType::unspecified },
            {   79, LoadingDeckType::upper },
            {   80, LoadingDeckType::lower },
        }),
        // enum 'ConfirmationType'
        QtMocHelpers::EnumData<ConfirmationType>(81, 81, QMC::EnumFlags{}).add({
            {   82, ConfirmationType::trainDelayConfirmation },
            {   83, ConfirmationType::travelerDelayConfirmation },
            {   84, ConfirmationType::trainLinkedTicketDelay },
        }),
        // enum 'TicketType'
        QtMocHelpers::EnumData<TicketType>(85, 85, QMC::EnumFlags{}).add({
            {   86, TicketType::openTicket },
            {   87, TicketType::pass },
            {   88, TicketType::reservation },
            {   89, TicketType::carCarriageReservation },
        }),
        // enum 'LinkMode'
        QtMocHelpers::EnumData<LinkMode>(90, 90, QMC::EnumFlags{}).add({
            {   91, LinkMode::issuedTogether },
            {   92, LinkMode::onlyValidInCombination },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10KItinerary3Fcb3v13E_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10KItinerary3Fcb3v13E =
    KItinerary::Fcb::v13::qt_create_metaobjectdata<qt_meta_tag_ZN10KItinerary3Fcb3v13E_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10KItinerary3Fcb3v13E =
    qt_staticMetaObjectContent_ZN10KItinerary3Fcb3v13E.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10KItinerary3Fcb3v13E =
    qt_staticMetaObjectContent_ZN10KItinerary3Fcb3v13E.relocatingData;

Q_CONSTINIT const QMetaObject KItinerary::Fcb::v13::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10KItinerary3Fcb3v13E.stringdata,
    qt_staticMetaObjectStaticContent_ZN10KItinerary3Fcb3v13E.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10KItinerary3Fcb3v13E.metaTypes,
    nullptr
} };

QT_WARNING_POP
